@@ -18,7 +18,7 @@
 */
 
 #include "Board.h"
-
+#include "PolygonHelper.h"
 
 // Methods to get objects at a grid location
 
@@ -35,7 +35,7 @@ int Board::GetComponentId(int row, int col)	// Pick the most relevant component
 		for (const auto& mapObj : m_compMgr.GetMapIdToComp())
 		{
 			const Component&	comp	= mapObj.second;
-			const bool			bPlug	= IsPlug( comp.GetType() );
+			const bool			bPlug	= CompTypes::IsPlug( comp.GetType() );
 			const bool			bPlaced = comp.GetIsPlaced();
 			if ( bPlaced != bReqPlaced ) continue;
 			if ( bPlug != bReqPlug ) continue;
@@ -51,7 +51,7 @@ int Board::GetComponentId(int row, int col)	// Pick the most relevant component
 		Component& trax = m_compMgr.GetTrax();
 		if ( trax.GetSize() > 0 )
 		{
-			const bool bPlug	= IsPlug( trax.GetType() );
+			const bool bPlug	= CompTypes::IsPlug( trax.GetType() );
 			const bool bPlaced	= trax.GetIsPlaced();
 			if ( bPlaced != bReqPlaced ) continue;
 			if ( bPlug != bReqPlug ) continue;
@@ -118,7 +118,7 @@ void Board::GetSeparations(double& minTrackSeparation_mil, double& minGroundFill
 
 	// If the minimum track separation is determined by the gap,
 	// then all locations have min separation, so don't show warning points in the view
-	if ( minTrackSeparation_mil == dGap ) ClearWarnPoints();
+	if ( minTrackSeparation_mil - dGap == 0.0 ) ClearWarnPoints();
 }
 void Board::CalcMIN_SEPARATION()	// Sets m_dMinSeparation and m_warnPoints[]
 {
@@ -143,9 +143,10 @@ void Board::CalcMIN_SEPARATION()	// Sets m_dMinSeparation and m_warnPoints[]
 		for (int j = minRow; j <= maxRow; j++)
 		for (int i = minCol; i <= maxCol; i++)
 		{
-			const Element*	pA		= Get(k, j, i);
-			const int&		nodeIdA	= pA->GetNodeId();
-			if ( nodeIdA == BAD_NODEID && !pA->GetHasPin() ) continue;	// Skip if no track and no pin
+			const Element*	pA			= Get(k, j, i);
+			const int&		nodeIdA		= pA->GetNodeId();
+			const bool		bHasPinA	= pA->GetHasPin();
+			if ( nodeIdA == BAD_NODEID && !bHasPinA ) continue;	// Skip if no track and no pin
 
 			MyPointF pointA(i, j, 0.005 * GetTRACK_MIL());	// The blob centre for pA (note: track radius !!!)
 			MyPointF padA(pointA);							// The pad centre for pA (pad radius will be set below)
@@ -153,7 +154,7 @@ void Board::CalcMIN_SEPARATION()	// Sets m_dMinSeparation and m_warnPoints[]
 
 			if ( pA->GetHasWire() )
 				padA.m_radius = 0.005 * GetPAD_MIL();	// No custom pad size for wires
-			else if ( pA->GetHasPin() )
+			else if ( bHasPinA )
 			{
 				const Component& comp	= m_compMgr.GetComponentById( pA->GetCompId() );	// Non-wire part must use slot 0
 				// Handle custom pad sizes
@@ -168,16 +169,17 @@ void Board::CalcMIN_SEPARATION()	// Sets m_dMinSeparation and m_warnPoints[]
 				bPadA = false;
 
 			std::list<MyPolygonF> blobA;	// Blob A points (in units of grid squares)
-			CalcBlob(1, pointA, padA, GetPerimeterCode(pA), blobA, pA->GetHasPin());	// 1 ==> scale of 1 grid square
+			CalcBlob(1, pointA, padA, GetPerimeterCode(pA), blobA, bHasPinA);	// 1 ==> scale of 1 grid square
 
 			// Only need to loop half the directions in the following loop (the i,j scan takes care of the other half)
 			for (int jj = std::max(minRow,j-nRings); jj <= j; jj++)
 			for (int ii = std::max(minCol,i-nRings), iiMax = std::min(maxCol,i+nRings); ii <= iiMax; ii++)
 			{
 				if ( jj == j && ii == i ) continue;	// Skip pA
-				const Element*	pB		= Get(k, jj, ii);
-				const int&		nodeIdB	= pB->GetNodeId();
-				if ( nodeIdB == BAD_NODEID && !pB->GetHasPin() ) continue;	// Skip if no track and no pin
+				const Element*	pB			= Get(k, jj, ii);
+				const int&		nodeIdB		= pB->GetNodeId();
+				const bool		bHasPinB	= pB->GetHasPin();
+				if ( nodeIdB == BAD_NODEID && !bHasPinB ) continue;	// Skip if no track and no pin
 				if ( nodeIdB == nodeIdA ) continue;
 
 				MyPointF pointB(ii, jj, 0.005 * GetTRACK_MIL());	// The blob centre for pB (note: track radius !!!)
@@ -186,7 +188,7 @@ void Board::CalcMIN_SEPARATION()	// Sets m_dMinSeparation and m_warnPoints[]
 
 				if ( pB->GetHasWire() )
 					padB.m_radius = 0.005 * GetPAD_MIL();	// No custom pad size for wires
-				else if ( pB->GetHasPin() )
+				else if ( bHasPinB )
 				{
 					const Component& comp	= m_compMgr.GetComponentById( pB->GetCompId() );	// Non-wire part must use slot 0
 					// Handle custom pad sizes
@@ -201,7 +203,7 @@ void Board::CalcMIN_SEPARATION()	// Sets m_dMinSeparation and m_warnPoints[]
 					bPadB = false;
 
 				std::list<MyPolygonF> blobB;	// Blob B points (in units of grid squares)
-				CalcBlob(1, pointB, padB, GetPerimeterCode(pB), blobB, pB->GetHasPin());	// 1 ==> scale of 1 grid square
+				CalcBlob(1, pointB, padB, GetPerimeterCode(pB), blobB, bHasPinB);	// 1 ==> scale of 1 grid square
 
 				const bool bCompareBlobs = !bStandardBlobs || ( abs(jj - j) < 2 && abs(ii - i) < 2 );	// Standard blobs ==> just consider neighbouring grid points
 
@@ -209,20 +211,20 @@ void Board::CalcMIN_SEPARATION()	// Sets m_dMinSeparation and m_warnPoints[]
 				if ( bPadA && bPadB ) polygonHelper.CalcSeparation(padA, padB);
 
 				// Pad A to Blob B
-				if ( bPadA ) for(auto& b : blobB) polygonHelper.CalcSeparation(padA, b);
+				if ( bPadA ) for(const auto& b : blobB) polygonHelper.CalcSeparation(padA, b);
 
 				// Pad B to Blob A
-				if ( bPadB ) for(auto& a : blobA) polygonHelper.CalcSeparation(padB, a);
+				if ( bPadB ) for(const auto& a : blobA) polygonHelper.CalcSeparation(padB, a);
 
 				// Blob A to Blob B
-				if ( bCompareBlobs ) for(auto & a : blobA) for(auto& b : blobB) polygonHelper.CalcSeparation(a, b);
+				if ( bCompareBlobs ) for(const auto& a : blobA) for(const auto& b : blobB) polygonHelper.CalcSeparation(a, b);
 			}
 		}
 		if ( polygonHelper.m_Dmin > m_dMinSeparation ) continue;
 		if ( polygonHelper.m_Dmin < m_dMinSeparation )	// If min for layer is lowest across all layers ...
 			ClearWarnPoints();							// ... wipe all warning points
 
-		for (auto& p : polygonHelper.m_pWarn) m_warnPoints[k].push_back(p);
+		for (auto& p : const_cast<const QPolygonF&>(polygonHelper.m_pWarn)) m_warnPoints[k].push_back(p);
 		m_dMinSeparation = polygonHelper.m_Dmin;
 	}	// Next layer
 }
@@ -283,10 +285,10 @@ void Board::CalcGroundFillBounds()
 		const double X = C * ( comp.GetCol() + comp.GetLastCol() + 1 );
 		const double Y = C * ( comp.GetRow() + comp.GetLastRow() + 1 );
 
-		m_gndL = std::min(m_gndL, (int)(X + W * L));
-		m_gndT = std::min(m_gndT, (int)(Y + W * T));
-		m_gndR = std::max(m_gndR, (int)(X + W * R));
-		m_gndB = std::max(m_gndB, (int)(Y + W * B));
+		m_gndL = std::min(m_gndL, static_cast<int>(X + W * L));
+		m_gndT = std::min(m_gndT, static_cast<int>(Y + W * T));
+		m_gndR = std::max(m_gndR, static_cast<int>(X + W * R));
+		m_gndB = std::max(m_gndB, static_cast<int>(Y + W * B));
 	}
 }
 
@@ -305,13 +307,15 @@ void Board::SetNodeId(Element* p, const int& nodeId, const bool bAllLyrs)	// Hel
 	Element* p2	= std::max(p, q);
 	if ( p1 )
 	{
-		m_adjInfoMgr.UpdateCounts(p1, nodeId);	// Do this BEFORE we call SetNodeId() on the element
-		p1->SetNodeId(nodeId);					// Write node value
+		if ( !GetRoutingEnabled() )	// No point updating m_adjInfoMgr since RebuildAdjacencies() is called after routing
+			m_adjInfoMgr.UpdateCounts(p1, nodeId);	// Do this BEFORE we call SetNodeId() on the element
+		p1->SetNodeId(nodeId);	// Write node value
 	}
 	if ( p2 )
 	{
-		m_adjInfoMgr.UpdateCounts(p2, nodeId);	// Do this BEFORE we call SetNodeId() on the element
-		p2->SetNodeId(nodeId);					// Write node value
+		if ( !GetRoutingEnabled() )	// No point updating m_adjInfoMgr since RebuildAdjacencies() is called after routing
+			m_adjInfoMgr.UpdateCounts(p2, nodeId);	// Do this BEFORE we call SetNodeId() on the element
+		p2->SetNodeId(nodeId);	// Write node value
 	}
 }
 
@@ -391,7 +395,7 @@ bool Board::SetNodeIdByUser(const int& lyr, const int& row, const int& col, cons
 
 	// Set the NodeId on the element and all connected wire points
 	p->GetWireList(wireList);	// Get list containing p and its wired points
-	for (auto& o : wireList)
+	for (const auto& o : wireList)
 	{
 		Element* pW = const_cast<Element*> (o.first);
 		const bool bAllLyrs = pW->GetHasPin();
@@ -407,7 +411,7 @@ bool Board::SetNodeIdByUser(const int& lyr, const int& row, const int& col, cons
 		{
 			size_t	pinIndex;
 			int		compId;
-			for (auto& o : wireList)
+			for (const auto& o : wireList)
 			{
 				Element* pL = const_cast<Element*> (o.first);
 				for (int iSlot = 0; iSlot < 2; iSlot++)
@@ -559,19 +563,16 @@ void Board::AutoFillVero()
 void Board::CalcSolder()	// Work out locations of solder blobs to join veroboard tracks together
 {
 	for (int i = 0, iSize = GetSize(); i < iSize; i++)
-	{
-		Element* p = GetAt(i);
-		p->SetSolderR(false);	// Clear solder
-	}
+		GetAt(i)->SetSolderR(false);	// Clear solder
+
 	const bool& bVertical = GetVerticalStrips();	// Strip direction
-	for (size_t n = 0, N = m_adjInfoMgr.GetSize(); n < N; n++)
-	{
-		const int& nodeId = m_adjInfoMgr.GetAt(n)->GetNodeId();
-		if ( nodeId == BAD_NODEID ) continue;	// Want valid nodeIds
-		const int& numCols = ( bVertical ) ? GetCols() : GetRows();
-		for (int col = 0; col < numCols; col++)	// Loop cols/rows
-			SetSolder(nodeId, col, bVertical);
-	}
+
+	std::vector<int> nodeIds;
+	m_adjInfoMgr.GetBoardNodeIds(nodeIds);	// The set of valid nodeIDs on the board
+
+	for (size_t n = 0, N = nodeIds.size(); n < N; n++)
+		for (int col = 0, numCols = bVertical ? GetCols() : GetRows(); col < numCols; col++)	// Loop cols/rows
+			SetSolder(nodeIds[n], col, bVertical);
 }
 
 void Board::SetSolder(const int& nodeId, const int& col, const bool& bVertical)
@@ -602,7 +603,7 @@ void Board::SetSolder(const int& nodeId, const int& col, const bool& bVertical)
 				{
 					const Component& comp = m_compMgr.GetComponentById( pC->GetCompId() );
 					assert( comp.GetType() != COMP::INVALID );
-					if ( comp.GetType() == COMP::PAD )
+					if ( comp.GetType() == COMP::PAD || comp.GetAllowFlyWire() )
 						rowPads++;
 					else
 						rowPins++;
@@ -616,7 +617,7 @@ void Board::SetSolder(const int& nodeId, const int& col, const bool& bVertical)
 				{
 					const Component& comp = m_compMgr.GetComponentById( pR->GetCompId() );
 					assert( comp.GetType() != COMP::INVALID );
-					if ( comp.GetType() == COMP::PAD )
+					if ( comp.GetType() == COMP::PAD || comp.GetAllowFlyWire() )
 						rowPads++;
 					else
 						rowPins++;
@@ -728,8 +729,9 @@ bool Board::GetDisableChangeType()
 	if ( ( GetGroupMgr().GetNumUserComps() != 1 ) || ( GetCompMode() == COMPSMODE::OFF ) ) return true;
 	const Component& comp	= GetUserComponent();
 	const COMP		 eType	= comp.GetType();
-	return !AllowTypeChange(eType, eType);
+	return !CompTypes::AllowTypeChange(eType, eType);
 }
+
 bool Board::GetDisableChangeCustom()
 {
 	if ( GetMirrored() ) return true;
