@@ -25,7 +25,8 @@
 #include "GroupManager.h"
 #include "ColorManager.h"
 #include "TextManager.h"
-#include "PolygonHelper.h"
+#include "ConnectionMatrix.h"
+#include <QPointF>
 
 #define SMART_PAN_CAN_AUTOCROP true
 
@@ -50,6 +51,8 @@ public:
 		GlueNbrs();		// Set pointers between neighbouring grid elements
 	}
 
+	virtual ~Board() override {}
+
 	Board(const Board& o, bool bFullCopy = true)
 	: ElementGrid()
 	, GuiControl()
@@ -96,6 +99,7 @@ public:
 
 		// Routing algorithm variables are cleared, not copied
 		m_targetPins.clear();
+		m_growingRoutes.clear();
 		m_tmpVec.clear();
 		m_tmpVecSize	= 0;
 		m_bRouteMinimal	= true;
@@ -103,13 +107,6 @@ public:
 		return *this;
 	}
 
-	~Board()
-	{
-		m_infoStr.clear();
-		m_targetPins.clear();
-		m_tmpVec.clear();
-	}
-
 	bool operator==(const Board& o) const	// Compare persisted info
 	{
 		if ( m_infoStr != o.m_infoStr ) return false;
@@ -133,6 +130,7 @@ public:
 		SetInfoStr("Use this box to enter a circuit description or other info");
 		m_compMgr.Clear();
 		m_targetPins.clear();
+		m_growingRoutes.clear();
 		m_tmpVec.clear();
 		m_nodeInfoMgr.DeAllocate();
 		m_adjInfoMgr.DeAllocate();
@@ -493,12 +491,12 @@ public:
 	const bool& GetHasVias() const { return m_bHasVias; }
 	unsigned int Flood(const int& nodeId);
 	unsigned int Flood();
-	void Flood_Helper(bool** ppConn, unsigned int& cost, const bool bBuildTracks);
-	void Flood_Grow(const unsigned int& numRIDs, const int& iFloodNodeId, bool** ppConn, unsigned int& cost, Element* pJ, const int& iNbr, const bool& bBuildTracks, unsigned int& iMH, unsigned int& iMaxMH, bool& bDone);
+	void Flood_Helper(const bool bBuildTracks);
+	void Flood_Grow(const int& iFloodNodeId, Element* pJ, const int& iNbr, const bool& bBuildTracks, unsigned int& iMH, unsigned int& iMaxMH, bool& bDone);
 	void Backtrace(Element* pEnd, const int& nodeId);
-	void BacktraceHelper(Element*& p, const int& nodeId, const int& iDeltaMH, const int& iNbr, const int& iLoop, unsigned int& MH, bool& bOK);
+	bool BacktraceHelper(Element*& p, unsigned int& MH, const int& nodeId, const unsigned int& iDeltaMH, const int& iNbr, const int& iLoop);
 	void Manhatten(Element* p);
-	void ManhattenHelper(const Element* p, const int& iNbr, const int& RID, unsigned int& iMH, unsigned int& iMaxMH);
+	void ManhattenHelper(const Element* p, const int& iNbr, unsigned int& iMH, unsigned int& iMaxMH);
 	void CheckAllComplete();
 	void PasteTracks(bool bTidy);
 	void WipeTracks();
@@ -566,6 +564,12 @@ public:
 	bool SetCurrentPinId(const int& i)		{ return m_compDefiner.SetCurrentPinId(i); }
 	bool SetCurrentShapeId(const int& i)	{ return m_compDefiner.SetCurrentShapeId(i); }
 
+	// Helper for flying wires
+	bool GetAllowFlyWire(Element* p) const
+	{
+		return p->GetNodeId() != BAD_NODEID && p->GetHasPin() && !p->GetHasWire() && p->IsLayer0() && m_compMgr.GetAllowFlyWire(p->GetCompId());
+	}
+
 	// Helpers for locations of close tracks
 	std::list<QPointF>& GetWarnPoints(int iLayer)	{ return m_warnPoints[iLayer]; }
 	void ClearWarnPoints()							{ m_warnPoints[0].clear(); m_warnPoints[1].clear(); }
@@ -669,7 +673,7 @@ public:
 
 		for (const auto& mapObj : m_compMgr.GetMapIdToComp())
 			m_nodeInfoMgr.AddComp(mapObj.second);
-		
+
 		GlueWires();	// Set pointers between wired grid elements
 
 		if ( inStream.GetVersion() < VRT_VERSION_26 )
@@ -691,8 +695,7 @@ public:
 
 		RebuildAdjacencies();
 
-		if ( inStream.GetVersion() < VRT_VERSION_42 )
-			FixCorruption();
+		FixCorruption();
 	}
 
 	virtual void Save(DataStream& outStream) override
@@ -709,6 +712,12 @@ public:
 		m_colorMgr.Save(outStream);		// Call Save() on color manager			// Added in VRT_VERSION_41
 	}
 private:
+	inline void UpdateMH(Element* p, const unsigned int& iRouteID, const unsigned int& iMH, unsigned int& iMaxMH)
+	{
+		m_growingRoutes[static_cast<size_t>(iRouteID)] = true;
+		m_tmpVec[m_tmpVecSize++] = p;	// Add p to set of visited points
+		p->UpdateMH(iRouteID, iMH, iMaxMH);
+	}
 	void FixLegacyWires()
 	{
 		// Find all placed wires and update the surface and hole codes on the board
@@ -762,9 +771,11 @@ private:
 	double					m_dMinSeparation = DBL_MAX;	// Units of grid squares
 
 	// Routing algorithm	// Don't persist or copy
-	std::vector<Element*>	m_targetPins;	// Set of pins to route.
-	std::vector<Element*>	m_tmpVec;		// The set of visited points.
-	size_t					m_tmpVecSize;	// The number of visited points.
-	bool					m_bRouteMinimal;// true ==> don't build tracks between pins that are already connected
-	bool					m_bHasVias;		// true ==> there are routed vias in the design (as opposed to "wires-as-tracks" vias)
+	ConnectionMatrix		m_connectionMatrix;	// Tracks connectivity between target pins
+	std::vector<Element*>	m_targetPins;		// Set of pins to route.
+	std::vector<bool>		m_growingRoutes;	// Flags to indicate if routes to target pins are still growing
+	std::vector<Element*>	m_tmpVec;			// The set of visited points.
+	size_t					m_tmpVecSize;		// The number of visited points.
+	bool					m_bRouteMinimal;	// true ==> don't build tracks between pins that are already connected
+	bool					m_bHasVias;			// true ==> there are routed vias in the design (as opposed to "wires-as-tracks" vias)
 };
