@@ -37,7 +37,7 @@ int Board::CreateComponent(int iRow, int iCol, const COMP& eType, const Componen
 	// Try and produce a simple unique Name for the new part if possible
 	char buffer[256] = {'\0'};
 	std::string nameStr;	// We'll use this string for both Name and Value
-	const std::string prefixStr = ( pComp ) ? pComp->GetPrefixStr() : GetDefaultPrefixStr(eType);	// e.g. "C" for capacitors
+	const std::string prefixStr = ( pComp ) ? pComp->GetPrefixStr() : CompTypes::GetDefaultPrefixStr(eType);	// e.g. "C" for capacitors
 	bool bNameExists(true);
 	for (int iSuffix = 1; iSuffix < INT_MAX && bNameExists; iSuffix++)
 	{
@@ -46,7 +46,7 @@ int Board::CreateComponent(int iRow, int iCol, const COMP& eType, const Componen
 		bNameExists = ( m_compMgr.GetComponentIdFromName(nameStr) != BAD_COMPID );
 	}
 
-	const size_t numPins = ( pComp ) ? pComp->GetNumPins() : GetDefaultNumPins(eType);
+	const size_t numPins = ( pComp ) ? pComp->GetNumPins() : static_cast<size_t>( CompTypes::GetDefaultNumPins(eType) );
 	std::vector<int> nodeList;
 	nodeList.resize(numPins, BAD_NODEID);
 	Component tmp(nameStr, nameStr, eType, nodeList);
@@ -61,6 +61,12 @@ int Board::CreateComponent(int iRow, int iCol, const COMP& eType, const Componen
 		// If pComp has a sensible Value field, then use it
 		if ( pComp->GetValueStr() != pComp->GetNameStr() )
 			tmp.SetValueStr( pComp->GetValueStr() );
+		if ( pComp->GetNameStr().empty() && pComp->GetValueStr().empty() )	// This may be typical for some pads
+		{
+			tmp.SetNameStr("");
+			tmp.SetValueStr("");
+		}
+
 	}
 	const bool bDoPlace = ( pComp == nullptr || pComp->GetIsTemplate() );	// Leave copied components floating
 	return AddComponent(iRow, iCol, tmp, bDoPlace);
@@ -83,9 +89,6 @@ int Board::AddComponent(int iRow, int iCol, const Component& tmp, bool bDoPlace)
 
 	if ( !bDoPlace ) return compId;
 
-	// Try place the component in free space on the board
-	bool bOK(false);
-
 	if ( iRow != -1 && iCol != -1  )	// If we passed in a valid row and col
 	{
 		// Put the component in the top left of the current visible view.
@@ -100,11 +103,12 @@ int Board::AddComponent(int iRow, int iCol, const Component& tmp, bool bDoPlace)
 		comp.SetRow(iRow);
 		comp.SetCol(iCol);
 		comp.SetDirection('W');
-		bOK = PutDown(comp);	// false ==> the component has to float
+		PutDown(comp);
 	}
 	else
 	{
 		// Try place the component in free space on the board.  Just used for Import() method
+		bool bOK(false);
 		while( !bOK )
 		{
 			for (int iRow = 0; iRow <= GetRows() - comp.GetCompRows() && !bOK; iRow++)
@@ -113,7 +117,7 @@ int Board::AddComponent(int iRow, int iCol, const Component& tmp, bool bDoPlace)
 				comp.SetRow(iRow);
 				comp.SetCol(iCol);
 				comp.SetDirection('W');
-				bOK = PutDown(comp);
+				bOK = PutDown(comp);	// false ==> the component has to float
 			}
 			if ( !bOK ) Pan(1, 0);	// No free board space, so pan the board down
 		}
@@ -490,7 +494,7 @@ bool Board::PutDown(Component& comp)	// Tries to place the (floating) component
 		// Handle all connected wires.  (We just need the wirelist on one end, so use pA)
 		pA->GetWireList(wireList);	// Get list containing pA and its wired points
 
-		for (auto& o : wireList)
+		for (const auto& o : wireList)
 		{
 			Element* pW = const_cast<Element*> (o.first);
 			// Set the nodeId's on the wire components ...
@@ -534,6 +538,7 @@ bool Board::TakeOff(Component& comp)
 	// Find out which wire slots are used before we take off the wire.
 	Element*	pA			= ( bWire ) ? Get(lyr, rowTL, colTL) : nullptr;
 	Element*	pB			= ( bWire ) ? Get(lyr, rowTL+compRows-1, colTL+compCols-1) : nullptr;
+	assert( !bWire || (pA != nullptr && pB != nullptr) );
 	int			iOrigIdA[2]	= {BAD_NODEID, BAD_NODEID};	// 1 per layer
 	int			iOrigIdB[2]	= {BAD_NODEID, BAD_NODEID};	// 1 per layer
 	int			iSlotA(-1), iSlotB(-1), tmpCompId;
@@ -629,7 +634,7 @@ bool Board::TakeOff(Component& comp)
 			pEnd->GetWireList(wireList);	// Get list containing pEnd and its wired points
 
 			int origId0(BAD_NODEID), origId1(BAD_NODEID);
-			for (auto& o : wireList)	// See if any of the wires in the set have a valid origId
+			for (const auto& o : wireList)	// See if any of the wires in the set have a valid origId
 			{
 				const Element* pW = o.first;
 				for (int iSlot = 0; iSlot < 2 && origId0 == BAD_NODEID && origId1 == BAD_NODEID; iSlot++)
@@ -648,7 +653,7 @@ bool Board::TakeOff(Component& comp)
 			if ( origId0 == BAD_NODEID && origId1 == BAD_NODEID )	// If no wired points connected to pEnd have a valid origId ...
 			{
 				// ... wipe all their nodeIds
-				for (auto& o : wireList)
+				for (const auto& o : wireList)
 				{
 					Element* pW = const_cast<Element*> (o.first);
 					// Wipe the nodeId's on the wire components ...
@@ -761,14 +766,14 @@ bool Board::ConfirmDestroyUserComps()	// returns false if user-group is empty or
 	m_groupMgr.GetUserCompIds(userCompIds);
 	for (const auto& compId : userCompIds)
 	{
-		switch( m_compMgr.GetComponentById( compId ).GetType() )
+		const COMP& eType = m_compMgr.GetComponentById( compId ).GetType();
+		switch( eType)
 		{
 			case COMP::VERO_NUMBER:
 			case COMP::VERO_LETTER:
 			case COMP::WIRE:
 			case COMP::MARK:	continue;
-			case COMP::INVALID:	assert(0);
-			default:			return true;
+			default:	assert(eType == COMP::INVALID);	return true;
 		}
 	}
 	return false;
@@ -812,9 +817,9 @@ void Board::StretchUserComp(const bool& bGrow)	// Stretch the selected component
 
 		// Work out what the stretch will do to it
 		if ( comp.GetDirection() == 'W' || comp.GetDirection() == 'E' )
-			maxCol += GetStretchIncrement(comp.GetType());
+			maxCol += CompTypes::GetStretchIncrement(comp.GetType());
 		else
-			maxRow += GetStretchIncrement(comp.GetType());
+			maxRow += CompTypes::GetStretchIncrement(comp.GetType());
 
 		if ( maxRow + 1 - GetRows() > 0 )		// If we'll go too far down ...
 			Pan(-(maxRow + 1 - GetRows()), 0);	// ... pan the whole circuit up to make room
@@ -865,7 +870,7 @@ void Board::ChangeTypeUserComp(const COMP& eType)
 	{
 		// Get bottom-right corner of new footprint
 		int numRows(0), numCols(0);
-		GetMakeInstructions(eType, numRows, numCols);
+		CompTypes::GetMakeInstructions(eType, numRows, numCols);
 		if ( comp.GetDirection() == 'N' || comp.GetDirection() == 'S' ) std::swap(numRows, numCols);
 		int maxRow(comp.GetRow() + numRows - 1);
 		int maxCol(comp.GetCol() + numCols - 1);
@@ -877,18 +882,19 @@ void Board::ChangeTypeUserComp(const COMP& eType)
 	}
 
 	// Now change the component type
-	const int  oldPinSeparation	= GetPinSeparation( comp.GetType() );	// For LEDs, and electro caps
+	const int  oldPinSeparation	= CompTypes::GetPinSeparation( comp.GetType() );	// For LEDs, and electro caps
 	const int  oldLength		= ( oldPinSeparation > 0 ) ? oldPinSeparation : comp.GetCols();
 	WipeAutoSetPoints();
 	TakeOff(comp);
 	comp.BuildDefault(eType);
+	comp.SetAllowFlyWire(eType == COMP::PAD_FLYINGWIRE);
 	comp.SetDefaultPinFlags();
 	comp.SetDefaultStrings();
 	comp.SetDefaultLabelOffsets();
 	comp.SetDefaultShapes(GetUsePCBshapes());
 	if ( bStretch )
 	{
-		while ( comp.GetCols() < oldLength ) comp.Stretch(true, GetUsePCBshapes());	// true  ==> grow
+		while ( comp.GetCols() < oldLength ) comp.Stretch(true,  GetUsePCBshapes());	// true  ==> grow
 		while ( comp.GetCols() > oldLength ) comp.Stretch(false, GetUsePCBshapes());	// false ==> shrink
 	}
 	PutDown(comp);
@@ -929,7 +935,7 @@ void Board::CopyComps(const std::list<int>& compIds)	// Make a blank copy of the
 	const bool bMakeNewGroup	= ( compIds.size() > 1 );	// If we have multiple components, the copies will be put into a single new group
 	const int  newGroupId		= ( bMakeNewGroup ) ? m_groupMgr.GetNewGroupId() : BAD_GROUPID;	// Make new groupId bigger than all others
 	if ( bMakeNewGroup && newGroupId == INT_MAX ) return;	// Fail if we've reached the max allowed groupId
-	for (auto& compId : compIds)
+	for (const auto& compId : compIds)
 	{
 		const Component&	comp		= m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() != COMP::INVALID );
@@ -991,7 +997,7 @@ bool Board::MoveComps(const std::list<int>& compIds, const int& deltaRow, const
 	WipeAutoSetPoints();
 
 	// Take off all comps, and finally the trax comp. Move them but keep them floating.
-	for (auto& compId : compIds)
+	for (const auto& compId : compIds)
 	{
 		Component& comp = m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() != COMP::INVALID );
@@ -1019,7 +1025,7 @@ bool Board::MoveComps(const std::list<int>& compIds, const int& deltaRow, const
 	PutDown(trax);
 	if ( trax.GetSize() == 0 || trax.GetIsPlaced() )
 	{
-		for (auto& compId : compIds)
+		for (const auto& compId : compIds)
 		{
 			Component& comp = m_compMgr.GetComponentById( compId );
 			assert( comp.GetType() != COMP::INVALID );
@@ -1069,7 +1075,7 @@ void Board::RotateComps(const std::list<int>& compIds, const bool& bCW)	// Rotat
 	WipeAutoSetPoints();
 
 	// Take off all comps, and finally the trax comp. Move and rotate them but keep them floating.
-	for (auto& compId : compIds)
+	for (const auto& compId : compIds)
 	{
 		Component& comp	= m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() != COMP::INVALID );
@@ -1118,7 +1124,7 @@ void Board::RotateComps(const std::list<int>& compIds, const bool& bCW)	// Rotat
 	PutDown(trax);
 	if ( trax.GetSize() == 0 || trax.GetIsPlaced() )
 	{
-		for (auto& compId : compIds)
+		for (const auto& compId : compIds)
 		{
 			Component& comp = m_compMgr.GetComponentById( compId );
 			assert( comp.GetType() != COMP::INVALID );
@@ -1136,14 +1142,30 @@ void Board::FixCorruption()
 	// Destroy any components that have an invalid component type
 	std::set<int> badCompIds;
 	m_compMgr.GetBadCompIds(badCompIds);
+	if ( badCompIds.empty() ) return;
 
-	for (auto& compId : badCompIds)
+	for (const auto& compId : badCompIds)
 	{
 		Component& comp = m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() == COMP::INVALID );
 		DestroyComponent(comp);
 	}
 
+	bool bBadGrid(false);	// true ==> grid has references to bad components
+	for (int i = 0, iSize = GetSize(); i < iSize && !bBadGrid; i++)
+	{
+		Element* p = GetAt(i);
+		const int& compId	= p->GetCompId();
+		const int& compId2	= p->GetCompId2();
+		bBadGrid = ( compId  != BAD_COMPID && badCompIds.find(compId)  != badCompIds.end() )
+				|| ( compId2 != BAD_COMPID && badCompIds.find(compId2) != badCompIds.end() );
+	}
+	if ( !bBadGrid ) return;
+
+	// The process of floating all components and then unfloating them
+	// can make the wrong competing diagonals be used at some points,
+	// so should only be done if we have a bad grid.
+
 	FloatAllComps();	// Float all components
 
 	// Ensure there are no component related effects on the board elements
@@ -1175,7 +1197,7 @@ void Board::FixCorruption()
 Rect Board::GetFootprintBounds(const std::list<int>& compIds)
 {
 	Rect bounding;
-	for (auto& compId : compIds)
+	for (const auto& compId : compIds)
 	{
 		const Component& comp	= m_compMgr.GetComponentById( compId );
 		assert( comp.GetType() != COMP::INVALID );
