@@ -120,7 +120,7 @@ bool Board::ImportTango(const TemplateManager& templateMgr, const std::string& f
 				bool bCustom(false);	// true ==> We've found a custom template with matching import string
 				Component custom;		// The matching custom template
 
-				const COMP eType = GetTypeFromImportStr(typeStrCut);
+				const COMP eType = CompTypes::GetTypeFromImportStr(typeStrCut);
 				if ( bOK )
 				{
 					bOK = ( eType != COMP::CUSTOM && eType != COMP::TRACKS && eType != COMP::VERO_NUMBER && eType != COMP::VERO_LETTER && eType != COMP::INVALID );
@@ -130,7 +130,7 @@ bool Board::ImportTango(const TemplateManager& templateMgr, const std::string& f
 				}
 
 				// Check pins per component is within limits
-				if ( numPins == 0 ) numPins = ( bCustom ) ? (int) custom.GetNumPins() : GetDefaultNumPins(eType);
+				if ( numPins == 0 ) numPins = ( bCustom )  ? static_cast<int>( custom.GetNumPins() ) : CompTypes::GetDefaultNumPins(eType);
 				if ( bOK )
 				{
 					bOK = ( numPins > 0 );
@@ -141,23 +141,23 @@ bool Board::ImportTango(const TemplateManager& templateMgr, const std::string& f
 				{
 					if ( bOK )
 					{
-						bOK = bCustom || ( nLength >= GetMinLength(eType) );
+						bOK = bCustom || ( nLength >= CompTypes::GetMinLength(eType) );
 						if ( !bOK ) errorStr = "Part section: " + nameStr + "\nInternal error: Part length is too small " + typeStr;
 					}
 					if ( bOK )
 					{
-						bOK = bCustom || ( nLength <= GetMaxLength(eType) );
+						bOK = bCustom || ( nLength <= CompTypes::GetMaxLength(eType) );
 						if ( !bOK ) errorStr = "Part section: " + nameStr + "\nInternal error: Part length is too large " + typeStr;
 					}
 				}
 				if ( bOK )
 				{
-					bOK = bCustom || ( numPins >= GetMinNumPins(eType) );
+					bOK = bCustom || ( numPins >= CompTypes::GetMinNumPins(eType) );
 					if ( !bOK ) errorStr = "Part section: " + nameStr + "\nPart type has fewer pins than VeroRoute supports: " + typeStr;
 				}
 				if ( bOK )
 				{
-					bOK = bCustom || ( numPins <= GetMaxNumPins(eType) );
+					bOK = bCustom || ( numPins <= CompTypes::GetMaxNumPins(eType) );
 					if ( !bOK ) errorStr = "Part section: " + nameStr + "\nPart type has more pins than VeroRoute supports: " + typeStr;
 				}
 				if ( bOK )
@@ -173,7 +173,7 @@ bool Board::ImportTango(const TemplateManager& templateMgr, const std::string& f
 					else
 					{
 						assert( eType != COMP::INVALID );
-						nodeList.resize(numPins, BAD_NODEID);
+						nodeList.resize(static_cast<size_t>(numPins), BAD_NODEID);
 						Component tmp(nameStr, valueStr, eType, nodeList);
 						if ( nLength > 0 )
 						{
@@ -217,7 +217,7 @@ bool Board::ImportTango(const TemplateManager& templateMgr, const std::string& f
 				if ( bOK )
 				{
 					// Paint the component pin using SetNodeIdByUser
-					const size_t	iPinIndex	= atoi(pinStr.c_str()) - 1;
+					const size_t	iPinIndex	= static_cast<size_t>( atoi(pinStr.c_str()) - 1 );
 					const int		compId		= m_compMgr.GetComponentIdFromName(nameStr);
 
 					bOK = compId != BAD_COMPID;
@@ -289,8 +289,6 @@ bool Board::ImportOrcad(const TemplateManager& templateMgr, const std::string& f
 	bool bPartStart(false);
 	std::list<std::string> offBoard;	// List of off-board part names
 
-	int compId(BAD_COMPID);
-
 	while( bOK )	// Loop through file
 	{
 		if ( inStream.eof() ) break;
@@ -381,7 +379,7 @@ bool Board::ImportOrcad(const TemplateManager& templateMgr, const std::string& f
 			bool bCustom(false);	// true ==> We've found a custom template with matching import string
 			Component custom;		// The matching custom template
 
-			const COMP eType = GetTypeFromImportStr(typeStrCut);
+			const COMP eType = CompTypes::GetTypeFromImportStr(typeStrCut);
 
 			bOK = ( eType != COMP::CUSTOM && eType != COMP::TRACKS && eType != COMP::VERO_NUMBER && eType != COMP::VERO_LETTER && eType != COMP::INVALID );
 			if ( !bOK )	// Search template manager
@@ -393,7 +391,7 @@ bool Board::ImportOrcad(const TemplateManager& templateMgr, const std::string& f
 			}
 
 			// Check pins per component is within limits
-			if ( numPins == 0 ) numPins = ( bCustom ) ? (int) custom.GetNumPins() : GetDefaultNumPins(eType);
+			if ( numPins == 0 ) numPins = ( bCustom ) ? static_cast<int>( custom.GetNumPins() ) : CompTypes::GetDefaultNumPins(eType);
 			bOK = ( numPins > 0 );
 			if ( !bOK )
 			{
@@ -404,13 +402,13 @@ bool Board::ImportOrcad(const TemplateManager& templateMgr, const std::string& f
 			// Check length is within limits for components with fixed numbers of pins
 			if ( nLength > 0 )
 			{
-				bOK = bCustom || ( nLength >= GetMinLength(eType) );
+				bOK = bCustom || ( nLength >= CompTypes::GetMinLength(eType) );
 				if ( !bOK )
 				{
 					errorStr = "Part: " + nameStr + "\nInternal error: Part length is too small " + typeStr;
 					break;
 				}
-				bOK = bCustom || ( nLength <= GetMaxLength(eType) );
+				bOK = bCustom || ( nLength <= CompTypes::GetMaxLength(eType) );
 				if ( !bOK )
 				{
 					errorStr = "Part: " + nameStr + "\nInternal error: Part length is too large " + typeStr;
@@ -418,13 +416,13 @@ bool Board::ImportOrcad(const TemplateManager& templateMgr, const std::string& f
 				}
 			}
 
-			bOK = bCustom || ( numPins >= GetMinNumPins(eType) );
+			bOK = bCustom || ( numPins >= CompTypes::GetMinNumPins(eType) );
 			if ( !bOK )
 			{
 				errorStr = "Part: " + nameStr + "\nPart type has fewer pins than VeroRoute supports: " + typeStr;
 				break;
 			}
-			bOK = bCustom || ( numPins <= GetMaxNumPins(eType) );
+			bOK = bCustom || ( numPins <= CompTypes::GetMaxNumPins(eType) );
 			if ( !bOK )
 			{
 				errorStr = "Part: " + nameStr + "\nPart type has more pins than VeroRoute supports: " + typeStr;
@@ -440,15 +438,15 @@ bool Board::ImportOrcad(const TemplateManager& templateMgr, const std::string& f
 			else
 			{
 				assert( eType != COMP::INVALID );
-				nodeList.resize(numPins, BAD_NODEID);
+				nodeList.resize(static_cast<size_t>(numPins), BAD_NODEID);
 				Component tmp(nameStr, valueStr, eType, nodeList);
 				if ( nLength > 0 )
 				{
 					while ( tmp.GetCols() < nLength ) tmp.Stretch(true);	// grow
 					while ( tmp.GetCols() > nLength ) tmp.Stretch(false);	// shrink
 				}
-				compId = AddComponent(-1, -1, tmp);
-				bOK = ( compId!= BAD_COMPID );	// Create part and place it
+				const int compId = AddComponent(-1, -1, tmp);
+				bOK = ( compId != BAD_COMPID );	// Create part and place it
 			}
 			if ( !bOK )
 			{
@@ -467,7 +465,6 @@ bool Board::ImportOrcad(const TemplateManager& templateMgr, const std::string& f
 			{
 				// If we have just a ')' on the line then we're done with the pins ...
 				bPartStart = false;	// ... and we're done with the part, move onto the next one
-				compId = BAD_COMPID;
 				continue;
 			}
 
@@ -501,7 +498,7 @@ bool Board::ImportOrcad(const TemplateManager& templateMgr, const std::string& f
 			}
 
 			// Paint the component pin using SetNodeIdByUser
-			const size_t	iPinIndex	= atoi(pinStr.c_str()) - 1;
+			const size_t	iPinIndex	= static_cast<size_t>( atoi(pinStr.c_str()) - 1 );
 			const int		compId		= m_compMgr.GetComponentIdFromName( nameStr );
 
 			bOK = compId != BAD_COMPID;
@@ -550,7 +547,7 @@ bool Board::ImportOrcad(const TemplateManager& templateMgr, const std::string& f
 bool Board::BreakComponentIntoPads(Component& comp)
 {
 	const COMP& eType = comp.GetType();
-	if ( eType == COMP::MARK || eType == COMP::PAD || eType == COMP::WIRE || eType == COMP::VERO_NUMBER || eType == COMP::VERO_LETTER ) return false;	// Not real components
+	if ( eType == COMP::MARK || eType == COMP::PAD || eType == COMP::PAD_FLYINGWIRE || eType == COMP::WIRE || eType == COMP::VERO_NUMBER || eType == COMP::VERO_LETTER ) return false;	// Not real components
 	if ( !comp.GetIsPlaced() ) return false;	// Can't break a floating component
 
 	std::vector<int> nodeList = { BAD_NODEID };	// Re-used for each new pad
@@ -560,7 +557,7 @@ bool Board::BreakComponentIntoPads(Component& comp)
 	{
 		// Create a new PAD component for the pin, with suitable name, value, nodeId
 		static char buffer[32];
-		sprintf(buffer, "_%d", (int)(iPinIndex+1));
+		sprintf(buffer, "_%d", static_cast<int>(iPinIndex+1));
 		nodeList[0] = comp.GetNodeId(iPinIndex);
 		Component tmp(comp.GetNameStr() + std::string(buffer), comp.GetValueStr(), COMP::PAD, nodeList);
 
