@@ -52,14 +52,14 @@ void Board::WipeAutoSetPoints(int nodeId)
 		if ( p->GetHasWire() )
 		{
 			p->GetWireList(wireList);	// Get list containing p and its wired points ...
-			for (auto& o : wireList)	// ... and disable wipe if any of them are USERSET
+			for (const auto& o : wireList)	// ... and disable wipe if any of them are USERSET
 			{
 				const Element* pW = o.first;
 				if ( pW == p ) continue;	// Skip p
 				bWipe &= ( !pW->ReadFlagBits(USERSET) );
 				if ( !bWipe ) break;
 			}
-			for (auto& o : wireList)
+			for (const auto& o : wireList)
 			{
 				Element* pW = const_cast<Element*> (o.first);
 				if ( bWipe ) SetNodeId(pW, BAD_NODEID, bAllLyrs);
@@ -168,15 +168,20 @@ void Board::Route(bool bMinimal)
 									CopyTo(Ibest);		// ... log the improved route (it's the best so far)
 							}
 						}
-						if ( pI->GetCost() > 0 ) CopyFrom(Iripped);	// Revert to ripped-up I
+						if ( pI->GetCost() > 0 )
+							CopyFrom(Iripped);	// Revert to ripped-up I
 					}
 					if ( j == 0 ) break; else j--;
 				}
-				if ( pI->GetCost() > 0 ) CopyFrom(Ibest);
+				if ( pI->GetCost() > 0 )
+					CopyFrom(Ibest);
 			}
 		}
 		if ( !bAllowRipUp ) break;
 	}
+
+	RebuildAdjacencies();
+
 //	const auto elapsed = std::chrono::steady_clock::now() - start;
 //	const auto duration_ms	= std::chrono::duration_cast<std::chrono::milliseconds>(elapsed).count();
 //	std::cout << "Time : " << duration_ms << std::endl;
@@ -225,53 +230,57 @@ unsigned int Board::Flood()
 	// The return value is a cost that shows how unconnected the pins are.
 	// Zero cost means the pins are all inter-connected.
 
-	if ( m_targetPins.size() < 2 ) return 0;	// Return cost of zero
+	const size_t N = m_targetPins.size();
 
-	// Allocate the connection matrix ppConn[][] to indicate which pairs of targetPins are connected
-	const size_t N	= m_targetPins.size();
-	const size_t N2	= N * N;
-	bool*	pConn	= new bool[N2];
-	bool**	ppConn	= new bool*[N];
-	memset(pConn, 0, N2 * sizeof(bool));
-	for (size_t i = 0; i < N; i++) ppConn[i] = pConn + i * N;
-	for (size_t i = 0; i < N; i++) ppConn[i][i] = true;	// Each pin is connected to itself
-	unsigned int cost = (unsigned int)(N2 - N);	// Cost = number of false values in the connection matrix
+	if ( N < 2 ) return 0;	// Return cost of zero
+
+	// Allocate the connection matrix to indicate which pairs of targetPins are connected
+	m_connectionMatrix.Allocate( N );
+
+	// All target pins that support "flying wires" are connected to each other
+	for (size_t j = 0; j < N; j++)
+		if ( GetAllowFlyWire( m_targetPins[j] ) )
+			for (size_t k = j+1; k < N; k++)
+				if ( GetAllowFlyWire( m_targetPins[k] ) )
+					m_connectionMatrix.Connect(j, k);	// Make j-k connection and enforce transitivity
 
 	if ( m_bRouteMinimal )	// For minimal routing, first do a preliminary flood to see which pins are connected
-		Flood_Helper(ppConn, cost, false);	// false ==> don't build new tracks
+		Flood_Helper(false);	// false ==> don't build new tracks
 
 	if ( GetRoutingEnabled() )
-		Flood_Helper(ppConn, cost, true);	// true ==> build new tracks
+		Flood_Helper(true);	// true ==> build new tracks
 
-	// Deallocate connection matrix
-	delete[] ppConn;
-	delete[] pConn;
-	return cost;
+	m_connectionMatrix.DeAllocate();
+	return m_connectionMatrix.GetCost();
 }
 
-void Board::Flood_Helper(bool** ppConn, unsigned int& cost, const bool bBuildTracks)
+void Board::Flood_Helper(const bool bBuildTracks)
 {
 	for (int i = 0, iSize = GetSize(); i < iSize; i++)	// Loop all grid points
 		GetAt(i)->ResetMH();	// Wipe RouteId. Set "infinite" MH distance.  Zero max MH parameter.
 
-	m_tmpVec.resize(GetSize(), nullptr);	// Clear the set of visited points
-	m_tmpVecSize = 0;
+	m_tmpVec.resize(static_cast<size_t>(GetSize()), nullptr);
+	m_tmpVecSize = 0;			// Clear the set of visited points
 
-	unsigned int iMH(0), iMaxMH(0);
+	const size_t N = m_targetPins.size();
+	m_growingRoutes.resize(N);	// Allocate flags to track route growth from pins
+
+	unsigned int iMH(0), iMaxMH(0), iMHlastGrowthCheck(0);
 
 	// Add each target pin to the set of visited points, with a unique routeId, and MH value of zero
-	unsigned int RID(BAD_ROUTEID);
+	unsigned int iRouteID(BAD_ROUTEID);
 	for (auto p : m_targetPins)
 	{
-		m_tmpVec[m_tmpVecSize++] = p;			// Add p to set of visited points
-		RID++;	assert( RID < BAD_ROUTEID );	// Should be safely < UINT_MAX in practice
-		p->UpdateMH(RID, iMH, iMaxMH);
+		iRouteID++;	assert( iRouteID < BAD_ROUTEID );	// Should be safely < UINT_MAX in practice
+		UpdateMH(p, iRouteID, iMH, iMaxMH);				// Add p to set of visited points
 	}
 
+	std::fill(m_growingRoutes.begin(), m_growingRoutes.end(), false);	// Clear flags tracking route growth from pins
+
+	const bool			bHaveWires	 = m_compMgr.GetHavePlacedWires();
 	const bool			bMultiLayer	 = GetLyrs() > 1;
 	const bool			bViasEnabled = bMultiLayer && GetViasEnabled();
 	const int&			iFloodNodeId = m_targetPins[0]->GetNodeId();
-	const unsigned int	numRIDs		 = RID + 1;	assert( m_targetPins.size() == (size_t) numRIDs );
 	const bool			bDiagsOK	 = ( GetDiagsMode() != DIAGSMODE::OFF );
 	const unsigned int	iMaxDeltaMH	 = ( bViasEnabled ) ? MH_LVIA : bDiagsOK ? MH_DIAG : MH_LRTB;	// The max MH increment in single-layer mode depends on if diagonals are allowed
 
@@ -308,107 +317,97 @@ void Board::Flood_Helper(bool** ppConn, unsigned int& cost, const bool bBuildTra
 				switch( iType )
 				{
 					case 0:	// Type 0 ==> Change layer at a pin
-						if ( pJ->GetHasPin() && pJ->GetMH() + MH_LPIN == iMH )
-							Flood_Grow(numRIDs, iFloodNodeId, ppConn, cost, pJ, NBR_X, bBuildTracks, iMH, iMaxMH, bDone);
+						if ( pJ->GetMH() + MH_LPIN == iMH && pJ->GetHasPin() )
+							Flood_Grow(iFloodNodeId, pJ, NBR_X, bBuildTracks, iMH, iMaxMH, bDone);
 						break;
 					case 1:	// Type 1 ==> Move within layer
 						for (int iDiag = 0, iDiagMax = ( bDiagsOK ) ? 2 : 1; iDiag < iDiagMax && !bDone; iDiag++)	// First pass ==> Non-diagonal nbrs.  Second pass diagonal nbrs
 						{
-							const int iDeltaMH = ( iDiag ) ? MH_DIAG : MH_LRTB;
+							const unsigned int iDeltaMH = ( iDiag ) ? MH_DIAG : MH_LRTB;
 							if ( pJ->GetMH() + iDeltaMH != iMH ) continue;	// pJ has wrong MH for connection
 
 							for (int iNbr = iDiag; iNbr < 8 && !bDone; iNbr += 2)	// Even/Odd iNbr ==> Non-diagonal/Diagonal
-								Flood_Grow(numRIDs, iFloodNodeId, ppConn, cost, pJ, iNbr, bBuildTracks, iMH, iMaxMH, bDone);
+								Flood_Grow(iFloodNodeId, pJ, iNbr, bBuildTracks, iMH, iMaxMH, bDone);
 						}
 						break;
 					case 2:	// Type 2 ==> Change layer at a via
-						if ( !pJ->GetHasPin() && pJ->GetMH() + MH_LVIA == iMH )
-							Flood_Grow(numRIDs, iFloodNodeId, ppConn, cost, pJ, NBR_X, bBuildTracks, iMH, iMaxMH, bDone);
+						if ( pJ->GetMH() + MH_LVIA == iMH && !pJ->GetHasPin() )
+							Flood_Grow(iFloodNodeId, pJ, NBR_X, bBuildTracks, iMH, iMaxMH, bDone);
 						break;
 				}
 			}
 		}
+
+		// Periodically (every sufficiently large MH increase) examine which routes have grown.
+		// If all growing routes are connected to each other then we're done.
+		if ( !bDone && iMH >= iMHlastGrowthCheck + iMaxDeltaMH + (bHaveWires ? MH_WIRE : 0) )
+		{
+			bDone = true;
+			for (size_t i = 0; i < N && bDone; i++)
+				if ( m_growingRoutes[i] )
+					for (size_t j = i+1; j < N && bDone; j++)
+						if ( m_growingRoutes[j] )
+							bDone = m_connectionMatrix.GetAreConnected(i,j);
+			if ( !bDone )
+			{
+				iMHlastGrowthCheck = iMH;
+				std::fill(m_growingRoutes.begin(), m_growingRoutes.end(), false);	// Clear flags tracking route growth from pins
+			}
+		}
 	}
 }
 
-void Board::Flood_Grow(const unsigned int& numRIDs, const int& iFloodNodeId, bool** ppConn, unsigned int& cost, Element* pJ, const int& iNbr, const bool& bBuildTracks, unsigned int& iMH, unsigned int& iMaxMH, bool& bDone)
+void Board::Flood_Grow(const int& iFloodNodeId, Element* pJ, const int& iNbr, const bool& bBuildTracks, unsigned int& iMH, unsigned int& iMaxMH, bool& bDone)
 {
 	WIRELIST wireList;	// Helper for chains of wires
 
 	Element* pK = pJ->GetNbr(iNbr);	assert( pK );
 	if ( pK == nullptr ) return;
 
-	const bool 			bOK	= pJ->GetNodeId() == iFloodNodeId;	// true ==> pJ already painted with correct NodeId
-	const unsigned int& j 	= pJ->GetRouteId();
-	const unsigned int& k 	= pK->GetRouteId();
+	const bool			bOK	= pJ->GetNodeId() == iFloodNodeId;	// true ==> pJ already painted with correct NodeId
+	const unsigned int&	j	= pJ->GetRouteId();
+	const unsigned int&	k	= pK->GetRouteId();
 
 	const bool bDirOK = ( bOK && pJ->GetUsed(iNbr) ) ||	// i.e. if already painted with correct nodeId
 						( bBuildTracks && pJ->HaveNoBlankPins(iNbr) && !pJ->IsBlocked(iNbr, iFloodNodeId) && !pJ->IsUselessWire(iNbr, iFloodNodeId) );
 	if ( !bDirOK ) return;
 
-	if ( pK->GetMH() == BAD_MH ) // Grow route with RID j (from pJ to pK)
+	if ( pK->GetMH() == BAD_MH ) // Grow route with ID j (from pJ to pK)
 	{
 		const int& nodeId = pK->GetNodeId();
 		if ( nodeId == iFloodNodeId || nodeId == BAD_NODEID )
 		{
-			m_tmpVec[m_tmpVecSize++] = pK;	// Add pK to set of visited points
-			pK->UpdateMH(j, iMH, iMaxMH);
+			UpdateMH(pK, j, iMH, iMaxMH);	// Add pK to set of visited points
+
 			const bool bWire = pK->IsLayer0() && pK->GetHasWire();	// Constrain wire-routing to layer 0
 			if ( bWire )
 			{
 				pK->GetWireList(wireList);	// Get list of pK and its wired points
-				for (auto& o : wireList)	// Ideally want these in order of increasing MH
+				for (const auto& o : wireList)	// Ideally want these in order of increasing MH
 				{
 					Element* pW = const_cast<Element*> (o.first);
 					if ( pW == pK ) continue;	// Skip pK
 					assert( pK->GetNodeId() == pW->GetNodeId() );			// Sanity check
 					if ( pW->GetMH() != BAD_MH ) continue;					// Don't overwrite visited points (even if MH is improved)
 					const unsigned int iOtherMH = iMH + MH_WIRE * o.second;	// Each wire increases MH by MH_WIRE
-					m_tmpVec[m_tmpVecSize++] = pW;							// Add pW to set of visited points
-					pW->UpdateMH(j, iOtherMH, iMaxMH);
+					UpdateMH(pW, j, iOtherMH, iMaxMH);						// Add pW to set of visited points
 				}
 			}
 		}
 		return;
 	}
-	if ( ppConn[j][k] ) return;
+	if ( m_connectionMatrix.GetAreConnected(j,k) ) return;
 
-	// Routes with RIDs j and k have met and don't have a connection yet ...
+	// Routes with IDs j and k have met and don't have a connection yet ...
 	if ( bBuildTracks )	// If building tracks ...
 	{
 		Backtrace(pJ, iFloodNodeId);	// ... trace pJ back to its source, painting iFloodNodeId along the way
 		Backtrace(pK, iFloodNodeId);	// ... trace pK back to its source, painting iFloodNodeId along the way
 	}
 
-	// Make j-k connection and enforce transitivity
-	typedef std::pair<unsigned int, unsigned int> CONNECTION;
-	std::list<CONNECTION> list;		// Helper for updating the connection matrix
-	list.push_back( CONNECTION(j,k) );
-	while ( !list.empty() )
-	{
-		auto iter = list.begin();	// Read info from first list entry ...
-		const auto a = iter->first;
-		const auto b = iter->second;
-		list.erase( iter );			// ... then remove the list entry
+	m_connectionMatrix.Connect(j, k);	// Make j-k connection and enforce transitivity
 
-		if ( !ppConn[a][b] )	// If no a-b connection ...
-		{
-			ppConn[a][b] = ppConn[b][a] = true;	// Make a-b connection ...
-			cost -= 2;							// Update cost
-			for (unsigned int c = 0; c < numRIDs; c++)	// Update 1st-order transitive relations
-			{
-				if ( ppConn[a][c] )
-				{
-					if ( !ppConn[b][c] ) list.push_back( CONNECTION(b,c) );	// a-c connection ==> b-c connection
-				}
-				else
-				{
-					if (  ppConn[b][c] ) list.push_back( CONNECTION(a,c) );	// b-c connection ==> a-c connection
-				}
-			}
-		}
-	}
-	bDone = ( cost == 0 );	// Zero cost ==> done
+	bDone = ( m_connectionMatrix.GetCost() == 0 );	// Zero cost ==> done
 }
 
 void Board::Backtrace(Element* pEnd, const int& nodeId)
@@ -429,43 +428,43 @@ void Board::Backtrace(Element* pEnd, const int& nodeId)
 	{
 		assert( !p->GetIsHole() );
 
-		const bool bAllLyrs = p->GetHasPin();
+		const bool bHasPin = p->GetHasPin();
 
 		Element* pW0 = p->GetW(0);
 		Element* pW1 = p->GetW(1);
 		const bool bWire = p->IsLayer0() && p->GetHasWire();	// Constrain wire-routing to layer 0
-		if ( !p->GetHasPin() || bWire ) // For non-pins and wires
+		if ( !bHasPin || bWire ) // For non-pins and wires
 		{
 			if ( p->GetNodeId() == BAD_NODEID )	// Set NodeId if not set yet.
 			{
-				SetNodeId(p, nodeId, bAllLyrs);
-				WipeFlagBits(p, USERSET, bAllLyrs);
-				MarkFlagBits(p, AUTOSET, bAllLyrs);
+				SetNodeId(p, nodeId, bHasPin);
+				WipeFlagBits(p, USERSET, bHasPin);
+				MarkFlagBits(p, AUTOSET, bHasPin);
 				if ( bWire )
 				{
 					p->GetWireList(wireList);	// Get list of p and its wired points
-					for (auto& o : wireList)
+					for (const auto& o : wireList)
 					{
 						Element* pW = const_cast<Element*> (o.first);
 						if ( pW == p ) continue;	// Skip p
-						SetNodeId(pW, nodeId, bAllLyrs);
-						WipeFlagBits(pW, USERSET, bAllLyrs);
-						MarkFlagBits(pW, AUTOSET, bAllLyrs);
+						SetNodeId(pW, nodeId, bHasPin);
+						WipeFlagBits(pW, USERSET, bHasPin);
+						MarkFlagBits(pW, AUTOSET, bHasPin);
 					}
 				}
 			}
 			else if ( p->ReadFlagBits(USERSET) )
 			{
 				assert( p->GetNodeId() == nodeId );
-				MarkFlagBits(p, AUTOSET, bAllLyrs);
+				MarkFlagBits(p, AUTOSET, bHasPin);
 				if ( bWire )
 				{
 					p->GetWireList(wireList);	// Get list of p and its wired points
-					for (auto& o : wireList)
+					for (const auto& o : wireList)
 					{
 						Element* pW = const_cast<Element*> (o.first);
 						if ( pW == p ) continue;	// Skip p
-						MarkFlagBits(pW, AUTOSET, bAllLyrs);
+						MarkFlagBits(pW, AUTOSET, bHasPin);
 					}
 				}
 			}
@@ -473,7 +472,7 @@ void Board::Backtrace(Element* pEnd, const int& nodeId)
 
 		if ( MH == 0 ) break;
 
-		bool bOK(false);
+		bool bOK(false);	// Will be set true after a successful backtrace (with modified p and MH)
 		// Now decide where to back trace to.
 
 		// Check wires first...
@@ -484,24 +483,25 @@ void Board::Backtrace(Element* pEnd, const int& nodeId)
 
 		for (int iLoop = 0; iLoop < 2 && !bOK; iLoop++)	// First pass to give preference to nbrs that are not wire ends
 		{
+			const bool bHasPin = ( bMultiLayer ) ? p->GetHasPin() : false;	// Only care if p has a pin if multilayer routing
 			const int iTypeMin(bMultiLayer ? 0 : 1), iTypeMax(bViasEnabled ? 2 : 1);
 			for (int iType = iTypeMin; iType <= iTypeMax && !bOK; iType++)
 			{
 				switch( iType )
 				{
 					case 0:	// Type 0 ==> Change layer at a pin
-						if ( p->GetHasPin() ) BacktraceHelper(p, nodeId, MH_LPIN, NBR_X, iLoop, MH, bOK);
+						if ( bHasPin ) bOK = BacktraceHelper(p, MH, nodeId, MH_LPIN, NBR_X, iLoop);
 						break;
 					case 1:	// Type 1 ==> Move within layer
 						for (int iDiag = 0, iDiagMax = ( bDiagsOK ) ? 2 : 1; iDiag < iDiagMax && !bOK; iDiag++)	// First pass ==> Non-diagonal nbrs.  Second pass diagonal nbrs
 						{
-							const int iDeltaMH = ( iDiag ) ? MH_DIAG : MH_LRTB;
+							const unsigned int iDeltaMH = ( iDiag ) ? MH_DIAG : MH_LRTB;
 							for (int iNbr = iDiag; iNbr < 8 && !bOK; iNbr += 2)	// Even/Odd iNbr ==> Non-diagonal/Diagonal
-								BacktraceHelper(p, nodeId, iDeltaMH, iNbr, iLoop, MH, bOK);
+								bOK = BacktraceHelper(p, MH, nodeId, iDeltaMH, iNbr, iLoop);
 						}
 						break;
 					case 2:	// Type 2 ==> Change layer at a via
-						if ( !p->GetHasPin() ) BacktraceHelper(p, nodeId, MH_LVIA, NBR_X, iLoop, MH, bOK);
+						if ( !bHasPin ) bOK = BacktraceHelper(p, MH, nodeId, MH_LVIA, NBR_X, iLoop);
 						break;
 				}
 			}
@@ -513,16 +513,17 @@ void Board::Backtrace(Element* pEnd, const int& nodeId)
 	}
 }
 
-void Board::BacktraceHelper(Element*& p, const int& nodeId, const int& iDeltaMH, const int& iNbr, const int& iLoop, unsigned int& MH, bool & bOK)
+bool Board::BacktraceHelper(Element*& p, unsigned int& MH, const int& nodeId, const unsigned int& iDeltaMH, const int& iNbr, const int& iLoop)
 {
 	Element* pNbr = p->GetNbr(iNbr);
-	if ( pNbr->GetRouteId() != p->GetRouteId() ) return;		// Skip if nbr has wrong routeId
+	if ( pNbr->GetRouteId() != p->GetRouteId() ) return false;	// Skip if nbr has wrong routeId
 	const bool bWire = pNbr->IsLayer0() && pNbr->GetHasWire();	// Constrain wire-routing to layer 0
-	if ( iLoop == 0 &&  bWire ) return;							// Skip if nbr is a wire
-	if ( iLoop == 1 && !bWire ) return;							// Skip if nbr is a non-wire
-	if ( p->IsBlocked(iNbr, nodeId) ) return;					// Skip if blocked
-	if ( pNbr->GetMH() != MH - iDeltaMH ) return;				// Skip if wrong MH change
-	p = pNbr;	MH -= iDeltaMH;		bOK = true;
+	if ( iLoop == 0 &&  bWire ) return false;					// Skip if nbr is a wire
+	if ( iLoop == 1 && !bWire ) return false;					// Skip if nbr is a non-wire
+	if ( p->IsBlocked(iNbr, nodeId) ) return false;				// Skip if blocked
+	if ( pNbr->GetMH() != MH - iDeltaMH ) return false;			// Skip if wrong MH change
+	p = pNbr;	MH -= iDeltaMH;
+	return true;	// We backtraced OK, and have modified p and MH
 }
 
 void Board::Manhatten(Element* p)
@@ -534,11 +535,19 @@ void Board::Manhatten(Element* p)
 
 	WIRELIST wireList;	// Helper for chains of wires
 
-	for (int i = 0, iSize = GetSize(); i < iSize; i++)	// Loop all grid points
-		GetAt(i)->ResetMH();	// Wipe RouteId. Set "infinite" MH distance.  Zero max MH parameter.
+	m_targetPins.clear();	// Reusing this to hold only flying wire pads for the nodeId being traced
 
-	m_tmpVec.resize(GetSize(), nullptr);	// Clear the set of visited points
-	m_tmpVecSize = 0;
+	for (int i = 0, iSize = GetSize(); i < iSize; i++)	// Loop all grid points
+	{
+		Element* pL = GetAt(i);
+		if ( GetAllowFlyWire(pL) && pL->GetNodeId() == iTraceNodeId )
+			m_targetPins.push_back(pL);
+
+		pL->ResetMH();	// Wipe RouteId. Set "infinite" MH distance.  Zero max MH parameter.
+	}
+
+	m_tmpVec.resize(static_cast<size_t>(GetSize()), nullptr);
+	m_tmpVecSize = 0;	// Clear the set of visited points
 
 	const bool			bMultiLayer	 = GetLyrs() > 1;
 	const bool			bViasEnabled = bMultiLayer && GetViasEnabled();
@@ -546,25 +555,30 @@ void Board::Manhatten(Element* p)
 	const unsigned int	iMaxDeltaMH	 = ( bViasEnabled ) ? MH_LVIA : bDiagsOK ? MH_DIAG : MH_LRTB;	// The max MH increment in single-layer mode depends on if diagonals are allowed
 
 	size_t jjStart(0);
-	const unsigned int RID(0);
+	const unsigned int iRouteID(0);
 	unsigned int iMH(0), iMaxMH(0);
 
-	// Add p to set of visited points, with MH value of zero
-	m_tmpVec[m_tmpVecSize++] = p;
-	p->UpdateMH(RID, iMH, iMaxMH);
+	if ( m_growingRoutes.empty() ) m_growingRoutes.resize(1, 0);	// Only ever use iRouteID == 0 in Manhatten()
+
+	// All pins that support "flying wires" and have the same nodeID are connected to each other
+	if ( GetAllowFlyWire(p) )
+		for (const auto& pL : m_targetPins)
+			UpdateMH(pL, iRouteID, iMH, iMaxMH);	// Add pL to set of visited points (with MH value of zero)
+	else
+		UpdateMH(p, iRouteID, iMH, iMaxMH);	// Add p to set of visited points (with MH value of zero)
+
 	const bool bWire = p->IsLayer0() && p->GetHasWire();	// Constrain wire-routing to layer 0
 	if ( bWire )
 	{
 		p->GetWireList(wireList);	// Get list of p and its wired points
-		for (auto& o : wireList)	// Ideally want these in order of increasing MH
+		for (const auto& o : wireList)	// Ideally want these in order of increasing MH
 		{
 			Element* pW = const_cast<Element*> (o.first);
 			if ( pW == p ) continue;	// Skip p
 			assert( p->GetNodeId() == pW->GetNodeId() );			// Sanity check
 			if ( pW->GetMH() != BAD_MH ) continue;					// Don't overwrite visited points (even if MH is improved)
 			const unsigned int iOtherMH = iMH + MH_WIRE * o.second;	// Each wire increases MH by MH_WIRE
-			m_tmpVec[m_tmpVecSize++] = pW;							// Add pW to set of visited points
-			pW->UpdateMH(RID, iOtherMH, iMaxMH);
+			UpdateMH(pW, iRouteID, iOtherMH, iMaxMH);				// Add pW to set of visited points
 		}
 	}
 	while ( true )
@@ -596,22 +610,22 @@ void Board::Manhatten(Element* p)
 				switch( iType )
 				{
 					case 0:	// Type 0 ==> Change layer at a pin
-						if ( pJ->GetHasPin() && pJ->GetMH() + MH_LPIN == iMH )
-							ManhattenHelper(pJ, NBR_X, RID, iMH, iMaxMH);
+						if ( pJ->GetMH() + MH_LPIN == iMH && pJ->GetHasPin() )
+							ManhattenHelper(pJ, NBR_X, iMH, iMaxMH);
 						break;
 					case 1:	// Type 1 ==> Move within layer
 						for (int iDiag = 0, iDiagMax = ( bDiagsOK ) ? 2 : 1; iDiag < iDiagMax; iDiag++)	// First pass ==> Non-diagonal nbrs.  Second pass diagonal nbrs
 						{
-							const int iDeltaMH = ( iDiag ) ? MH_DIAG : MH_LRTB;
+							const unsigned int iDeltaMH = ( iDiag ) ? MH_DIAG : MH_LRTB;
 							if ( pJ->GetMH() + iDeltaMH != iMH ) continue;	// pJ has wrong MH for connection
 
 							for (int iNbr = iDiag; iNbr < 8; iNbr += 2)	// Even/Odd iNbr ==> Non-diagonal/Diagonal
-								ManhattenHelper(pJ, iNbr, RID, iMH, iMaxMH);
+								ManhattenHelper(pJ, iNbr, iMH, iMaxMH);
 						}
 						break;
 					case 2:	// Type 2 ==> Change layer at a via
-						if ( !pJ->GetHasPin() && pJ->GetMH() + MH_LVIA == iMH )
-							ManhattenHelper(pJ, NBR_X, RID, iMH, iMaxMH);
+						if ( pJ->GetMH() + MH_LVIA == iMH && !pJ->GetHasPin() )
+							ManhattenHelper(pJ, NBR_X, iMH, iMaxMH);
 						break;
 				}
 			}
@@ -619,7 +633,7 @@ void Board::Manhatten(Element* p)
 	}
 }
 
-void Board::ManhattenHelper(const Element* p, const int& iNbr, const int& RID, unsigned int& iMH, unsigned int& iMaxMH)
+void Board::ManhattenHelper(const Element* p, const int& iNbr, unsigned int& iMH, unsigned int& iMaxMH)
 {
 	if ( !ReadCodeBit(iNbr, p->GetRoutable()) ) return;	// Skip non-routable nbrs
 
@@ -628,23 +642,28 @@ void Board::ManhattenHelper(const Element* p, const int& iNbr, const int& RID, u
 
 	WIRELIST wireList;	// Helper for chains of wires
 
+	const unsigned iRouteID(0);
 	if ( p->GetUsed(iNbr) && pK->GetMH() == BAD_MH )
 	{
-		m_tmpVec[m_tmpVecSize++] = pK;	// Add pK to set of visited points
-		pK->UpdateMH(RID, iMH, iMaxMH);
+		// All pins that support "flying wires" and have the same nodeID are connected to each other
+		if ( GetAllowFlyWire(pK) )
+			for (const auto& pL : m_targetPins)
+				UpdateMH(pL, iRouteID, iMH, iMaxMH);	// Add pL to set of visited points
+		else
+			UpdateMH(pK, iRouteID, iMH, iMaxMH);	// Add pK to set of visited points
+
 		const bool bWire = pK->IsLayer0() && pK->GetHasWire();	// Constrain wire-routing to layer 0
 		if ( bWire )
 		{
 			pK->GetWireList(wireList);	// Get list of pK and its wired points
-			for (auto& o : wireList)	// Ideally want these in order of increasing MH
+			for (const auto& o : wireList)	// Ideally want these in order of increasing MH
 			{
 				Element* pW = const_cast<Element*> (o.first);
 				if ( pW == pK ) continue;	// Skip pK
 				assert( pK->GetNodeId() == pW->GetNodeId() );			// Sanity check
 				if ( pW->GetMH() != BAD_MH ) continue;					// Don't overwrite visited points (even if MH is improved)
 				const unsigned int iOtherMH = iMH + MH_WIRE * o.second;	// Each wire increases MH by MH_WIRE
-				m_tmpVec[m_tmpVecSize++] = pW;							// Add pW to set of visited points
-				pW->UpdateMH(RID, iOtherMH, iMaxMH);
+				UpdateMH(pW, iRouteID, iOtherMH, iMaxMH);				// Add pW to set of visited points
 			}
 		}
 	}
@@ -654,8 +673,6 @@ void Board::CheckAllComplete()
 {
 	assert( !GetRoutingEnabled() );	// If routing is enabled, use the "RoutedOK" flags instead of the "Complete" flags.
 
-	// New algorithm.
-	/*
 	// Calling Route() when routing is not enabled sets the cost info without building new tracks.
 	// So we can use that to set the "Complete" flags
 	Route(true);
@@ -665,31 +682,6 @@ void Board::CheckAllComplete()
 		pNodeInfo->SetComplete( pNodeInfo->GetCost() == 0 );
 	}
 	m_nodeInfoMgr.SortByLowestDifficulty(m_compMgr);
-	return;
-	*/
-
-	// Old algorithm.  In most cases this is faster than the new algorithm, but worst-case performance is worse
-	for (size_t n = 0, nSize = m_nodeInfoMgr.GetSize(); n < nSize; n++)
-	{
-		NodeInfo* pNodeInfo = m_nodeInfoMgr.GetAt(n);
-		pNodeInfo->SetComplete(false);
-
-		const int& nodeId = pNodeInfo->GetNodeId();
-		if ( nodeId == BAD_NODEID ) continue;
-
-		BuildTargetPins(nodeId);
-
-		bool bComplete(true);
-		for (auto iterI = m_targetPins.begin(), iterEnd = m_targetPins.end(); iterI != iterEnd && bComplete; ++iterI)
-		{
-			Manhatten(*iterI);
-
-			for (auto iterJ = iterI; iterJ != iterEnd && bComplete; ++iterJ)
-				bComplete = ( (*iterJ)->GetMH() != BAD_MH );
-		}
-		pNodeInfo->SetComplete(bComplete);
-	}
-	m_nodeInfoMgr.SortByLowestDifficulty(m_compMgr);
 }
 
 void Board::PasteTracks(bool bTidy)
@@ -705,26 +697,36 @@ void Board::PasteTracks(bool bTidy)
 	size_t	iPinIndex;
 	int		tmpCompId;
 
-	for (int i = 0, iSize = GetSize(); i < iSize; i++)
-	{
-		Element* p = GetAt(i);
+	// If we are doing a paste (not a tidy) and have a placed trax component, only paste within it then destroy the trax component
+	Component&	trax		= m_compMgr.GetTrax();
+	const bool	bRestrict	= !bTidy && ( trax.GetSize() > 0 && trax.GetIsPlaced() );
+	const int	lyrs		= bRestrict ? trax.GetLyr() + 1  : GetLyrs();
+	const int	rows		= bRestrict ? trax.GetCompRows() : GetRows();
+	const int	cols		= bRestrict ? trax.GetCompCols() : GetCols();
 
-		const bool bAllLyrs = p->GetHasPin();
+	for (int k = bRestrict ? trax.GetLyr() : 0;			k < lyrs; k++)
+	for (int j = 0, jRow = bRestrict ? trax.GetRow() : 0;	j < rows; j++, jRow++)
+	for (int i = 0, iCol = bRestrict ? trax.GetCol() : 0;	i < cols; i++, iCol++)
+	{
+		if ( bRestrict && !trax.GetCompElement(j,i)->ReadFlagBits(RECTSET) ) continue;	// Skip points outside grey area
+
+		Element*	p		= Get(k, jRow, iCol);
+		const bool	bHasPin	= p->GetHasPin();
 
 		// Tidy clears all non-pins and wires that are USER_SET ...
-		if ( bTidy && ( !p->GetHasPin() || p->GetHasWire() ) && p->ReadFlagBits(USERSET) && !p->ReadFlagBits(AUTOSET|VEROSET) )
+		if ( bTidy && ( !bHasPin || p->GetHasWire() ) && p->ReadFlagBits(USERSET) && !p->ReadFlagBits(AUTOSET|VEROSET) )
 		{
-			SetNodeId(p, BAD_NODEID, bAllLyrs);
+			SetNodeId(p, BAD_NODEID, bHasPin);
 			for (int iSlot = 0; iSlot < 2; iSlot++)
 			{
 				Element* pW = p->GetW(iSlot);
-				if ( pW ) SetNodeId(pW, BAD_NODEID, bAllLyrs);
+				if ( pW ) SetNodeId(pW, BAD_NODEID, bHasPin);
 			}
 		}
 
 		// Make the point USERSET
-		WipeFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
-		MarkFlagBits(p, USERSET, bAllLyrs);
+		WipeFlagBits(p, AUTOSET|VEROSET, bHasPin);
+		MarkFlagBits(p, USERSET, bHasPin);
 
 		// For wires, the "Paste" operation either paints the board at the wire-ends or wipes it.
 		// Fix-up the nodeId info on any wire components ...
@@ -744,50 +746,45 @@ void Board::PasteTracks(bool bTidy)
 			}
 		}
 	}
-	SetRoutingEnabled(false);
+	if ( bRestrict )
+	{
+		m_compMgr.ClearTrax();
+		m_rectMgr.Clear();
+	}
+	if ( !bRestrict )
+		SetRoutingEnabled(false);
 }
 
 void Board::WipeTracks()
 {
 	FloatAllComps();	// Float all components
 
-	// If we have a placed trax component, only wipe the board within it then destroy it
-	Component& trax = m_compMgr.GetTrax();
-	if ( trax.GetSize() > 0 && trax.GetIsPlaced() )
+	// If we have a placed trax component, only wipe the board within it then destroy the trax component
+	Component&	trax		= m_compMgr.GetTrax();
+	const bool	bRestrict	= ( trax.GetSize() > 0 && trax.GetIsPlaced() );
+	const int	lyrs		= bRestrict ? trax.GetLyr() + 1  : GetLyrs();
+	const int	rows		= bRestrict ? trax.GetCompRows() : GetRows();
+	const int	cols		= bRestrict ? trax.GetCompCols() : GetCols();
+
+	for (int k = bRestrict ? trax.GetLyr() : 0;			k < lyrs; k++)
+	for (int j = 0, jRow = bRestrict ? trax.GetRow() : 0;	j < rows; j++, jRow++)
+	for (int i = 0, iCol = bRestrict ? trax.GetCol() : 0;	i < cols; i++, iCol++)
 	{
-		const bool bAllLyrs(false);
-		const int& lyr = trax.GetLyr();
+		if ( bRestrict && !trax.GetCompElement(j,i)->ReadFlagBits(RECTSET) ) continue;	// Skip points outside grey area
+		Element* p = Get(k, jRow, iCol);
+		assert( !p->GetHasPin() && !p->GetIsHole() && !p->GetHasComp() );	// Sanity check
 
-		for (int j = 0, jRow = trax.GetRow(), rows = trax.GetCompRows(); j < rows; j++, jRow++)
-		for (int i = 0, iCol = trax.GetCol(), cols = trax.GetCompCols(); i < cols; i++, iCol++)
-		{
-			if ( !trax.GetCompElement(j,i)->ReadFlagBits(RECTSET) ) continue;
-			Element* p = Get(lyr, jRow, iCol);
-			assert( !p->GetHasPin() && !p->GetIsHole() && !p->GetHasComp() );	// Sanity check
-
-			SetNodeId(p, BAD_NODEID, bAllLyrs);
-			p->SetSurface(SURFACE_FREE);
-			WipeFlagBits(p, AUTOSET|VEROSET|RECTSET, bAllLyrs);
-			MarkFlagBits(p, USERSET, bAllLyrs);
-		}
+		SetNodeId(p, BAD_NODEID, !bRestrict);
+		p->SetSurface(SURFACE_FREE);
+		WipeFlagBits(p, bRestrict ? (AUTOSET|VEROSET|RECTSET) : (AUTOSET|VEROSET), !bRestrict);
+		MarkFlagBits(p, USERSET, !bRestrict);
+	}
+	if ( bRestrict )
+	{
 		m_compMgr.ClearTrax();
 		m_rectMgr.Clear();
 	}
-	else	// ... otherwise wipe all the points on the board. The floating trax component won't get wiped
-	{
-		const bool bAllLyrs(true);
-
-		for (int k = 0, lyrs = GetLyrs(); k < lyrs; k++)
-		for (int j = 0, rows = GetRows(); j < rows; j++)
-		for (int i = 0, cols = GetCols(); i < cols; i++)
-		{
-			Element* p = Get(k, j, i);
-			assert( !p->GetHasPin() && !p->GetIsHole() && !p->GetHasComp() );	// Sanity check
-			SetNodeId(p, BAD_NODEID, bAllLyrs);
-			p->SetSurface(SURFACE_FREE);
-			WipeFlagBits(p, AUTOSET|VEROSET, bAllLyrs);
-			MarkFlagBits(p, USERSET, bAllLyrs);
-		}
-	}
+	if ( !bRestrict )
+		SetRoutingEnabled(false);
 	PlaceFloaters();	// Unfloat components
 }
