@@ -45,20 +45,3 @@
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
 #endif
-
-struct MergeOffsets
-{
-	int deltaNodeId 	= 0;
-	int deltaCompId 	= 0;
-	int deltaGroupId	= 0;
-	int deltaLyr		= 0;
-	int deltaRow	 	= 0;
-	int deltaCol 		= 0;
-};
-
-// The interface definition for handling merge offsets
-struct Merge
-{
-	virtual	void UpdateMergeOffsets(MergeOffsets&) = 0;
-	virtual void ApplyMergeOffsets(const MergeOffsets&) = 0;
-};
