@@ -27,6 +27,7 @@ void CompDefiner::Populate(const Component& o)
 	SetPinFlags( o.GetPinFlags() );
 	SetPadWidth( o.GetPadWidth() );
 	SetHoleWidth( o.GetHoleWidth() );
+	SetAllowFlyWire( o.GetAllowFlyWire() );
 
 	// Copy strings
 	SetValueStr( o.GetValueStr() );
@@ -59,7 +60,7 @@ void CompDefiner::Populate(const Component& o)
 	// Copy shapes
 	m_mapShapes.clear();
 	int iShapeId(0);
-	for (auto& shape : o.GetShapes())
+	for (const auto& shape : o.GetShapes())
 	{
 		AddShape(iShapeId, shape);
 		iShapeId++;
@@ -74,6 +75,7 @@ void CompDefiner::Build(Component& comp) const
 	comp.SetPinFlags( GetPinFlags() );
 	comp.SetPadWidth( GetPadWidth() );
 	comp.SetHoleWidth( GetHoleWidth() );
+	comp.SetAllowFlyWire( GetAllowFlyWire() );
 	comp.SetValueStr( GetValueStr() );
 	comp.SetPrefixStr( GetPrefixStr() );
 	comp.SetTypeStr( GetTypeStr() );
@@ -139,6 +141,7 @@ bool CompDefiner::SetWidth(const int& i)
 		m_grid.Allocate(1, m_grid.GetRows(), i);
 		m_grid.Clear( Pin(BAD_PINCHAR, SURFACE_FULL, HOLE_FREE) );
 		m_mapShapes.clear();
+		SetAllowFlyWire(false);
 		AddRect();	// Provide a Rect by default
 	}
 	return bChanged;
@@ -154,6 +157,7 @@ bool CompDefiner::SetHeight(const int& i)
 		m_grid.Allocate(1, i, m_grid.GetCols());
 		m_grid.Clear( Pin(BAD_PINCHAR, SURFACE_FULL, HOLE_FREE) );
 		m_mapShapes.clear();
+		SetAllowFlyWire(false);
 		AddRect();	// Provide a Rect by default
 	}
 	return bChanged;
@@ -198,8 +202,25 @@ int CompDefiner::GetShapeId(const double& dRowIn, const double& dColIn) const	//
 		bool bOK(false);
 		switch( s.GetType() )
 		{
-			case SHAPE::LINE:	// Check for distance within a narrow ellipse with foci at the endpoints
+			case SHAPE::RECT:
+			case SHAPE::ROUNDED_RECT:
+				dArea = fabs(DX*DY);			// Area of the rectangle
+				bOK   = fabs(2.0*ry) <= ( ( DY <= 0.1 ) ? epsilon : DY );
+				bOK  &= fabs(2.0*rx) <= ( ( DX <= 0.1 ) ? epsilon : DX );
+				break;
+			case SHAPE::ELLIPSE:
+			case SHAPE::ARC:
+			case SHAPE::CHORD:
 			{
+				dArea	= M_PI * 0.25*DX*DY;	// Area of the ellipse
+				double epsilon(dArea < 0.1 ? 0.1 : 0);
+				bOK		= rx*DY*rx*DY + ry*DX*ry*DX <= 0.25*DX*DX*DY*DY + epsilon;
+				break;
+			}
+			default:	// SHAPE::LINE
+			{
+				assert(s.GetType() == SHAPE::LINE);
+				// Check for distance within a narrow ellipse with foci at the endpoints
 				// Get true X1,X2,Y1,Y2 locations w.r.t. footprint centre
 				const double x1  = s.GetX1() - CX;
 				const double x2  = s.GetX2() - CX;
@@ -219,22 +240,6 @@ int CompDefiner::GetShapeId(const double& dRowIn, const double& dColIn) const	//
 				bOK		= sqrt(dx1*dx1 + dy1*dy1) + sqrt(dx2*dx2 + dy2*dy2) < dArea + epsilon;
 				break;
 			}
-			case SHAPE::RECT:
-			case SHAPE::ROUNDED_RECT:
-				dArea = fabs(DX*DY);			// Area of the rectangle
-				bOK   = fabs(2.0*ry) <= ( ( DY <= 0.1 ) ? epsilon : DY );
-				bOK  &= fabs(2.0*rx) <= ( ( DX <= 0.1 ) ? epsilon : DX );
-				break;
-			case SHAPE::ELLIPSE:
-			case SHAPE::ARC:
-			case SHAPE::CHORD:
-			{
-				dArea	= M_PI * 0.25*DX*DY;	// Area of the ellipse
-				double epsilon(dArea < 0.1 ? 0.1 : 0);
-				bOK		= rx*DY*rx*DY + ry*DX*ry*DX <= 0.25*DX*DX*DY*DY + epsilon;
-				break;
-			}
-			default: assert(0);
 		}
 		if ( !bOK ) continue;
 		if ( iBestId == BAD_ID || dArea <= dMinArea )
@@ -251,7 +256,7 @@ bool CompDefiner::GetIsValid() const
 	if ( StringHelper::IsEmptyStr(m_typeStr) ) return false;
 	if ( StringHelper::IsEmptyStr(m_valueStr) ) return false;
 	if ( StringHelper::HasSpaces(m_importStr) ) return false;	// Import string must not have spaces
-	if ( GetTypeFromImportStr(m_importStr) != COMP::INVALID ) return false;	// Reserved string
+	if ( CompTypes::GetTypeFromImportStr(m_importStr) != COMP::INVALID ) return false;	// Reserved string
 	// Following is copied from Board::Import() method.
 	// List of package identifiers for footprints with variable numbers of pins/lengths.
 	const int NUM_VARIABLE_PIN_PARTS = 13;
@@ -264,7 +269,7 @@ bool CompDefiner::GetIsValid() const
 	}
 	if ( m_grid.GetSize() == 0 ) return false;	// Should not be possible
 	bool bOK(false);
-	for (auto& mapObj : m_mapShapes)
+	for (const auto& mapObj : m_mapShapes)
 		if ( mapObj.second.GetDrawLine() && !mapObj.second.GetDrawFill() ) { bOK = true; break; }
 	if ( !bOK ) return false;	// Must have a shape with line and no fill (so  it can't be faded out)
 	// Pin indexes must be consecutive at start at 0
@@ -275,7 +280,7 @@ bool CompDefiner::GetIsValid() const
 
 	pinIndexes.sort();
 	size_t iTest(0);
-	for (auto& pinIndex : pinIndexes)
+	for (const auto& pinIndex : pinIndexes)
 	{
 		if ( pinIndex != iTest ) return false;
 		iTest++;
