@@ -33,7 +33,7 @@ void Component::SetDefaultPinFlags()
 		case COMP::DIP_RECTIFIER:
 		case COMP::STRIP_100:
 		case COMP::BLOCK_100:
-		case COMP::BLOCK_200:		return SetPinFlags(PIN_LABELS | iCustomFlag);
+		case COMP::BLOCK_200:		return SetPinFlags(static_cast<uchar>(PIN_LABELS | iCustomFlag));
 		case COMP::SWITCH_ST:
 		case COMP::SWITCH_DT:
 		case COMP::SWITCH_ST_DIP:
@@ -49,11 +49,12 @@ void Component::SetDefaultPinFlags()
 		case COMP::RELAY_JQC_3F_APPROX:
 		case COMP::RELAY_S1A050000:
 		case COMP::RELAY_TRCD:
-		case COMP::FUSE_HOLDER:		return SetPinFlags(PIN_RECT | iCustomFlag);
+		case COMP::FUSE_HOLDER:		return SetPinFlags(static_cast<uchar>(PIN_RECT | iCustomFlag));
 		case COMP::RELAY_DIP_4PIN:
-		case COMP::RELAY_DIP_8PIN:	return SetPinFlags(PIN_RECT | PIN_LABELS | iCustomFlag);
+		case COMP::RELAY_DIP_8PIN:	return SetPinFlags(static_cast<uchar>(PIN_RECT | PIN_LABELS | iCustomFlag));
 		case COMP::MARK:
 		case COMP::PAD:
+		case COMP::PAD_FLYINGWIRE:
 		case COMP::WIRE:
 		case COMP::RESISTOR:
 		case COMP::INDUCTOR:
@@ -94,13 +95,12 @@ void Component::SetDefaultPinFlags()
 		case COMP::TRIM_3362W:
 		case COMP::TRIM_3362X:
 		case COMP::TRIM_3362Z:
-		case COMP::CRYSTAL:	assert( GetPinFlags() == iCustomFlag );	return SetPinFlags(iCustomFlag);
+		case COMP::CRYSTAL:	assert( GetPinFlags() == iCustomFlag );	return SetPinFlags(static_cast<uchar>(iCustomFlag));
 		case COMP::VERO_NUMBER:
 		case COMP::VERO_LETTER:
 		case COMP::CUSTOM:
-		case COMP::TRACKS:
-		case COMP::INVALID:		return;
-		default:	assert(0);	return;	// Unhandled eType
+		case COMP::TRACKS:	return;
+		default:	assert(GetType() == COMP::INVALID);	return;	// Unhandled eType
 	}
 }
 
@@ -120,6 +120,7 @@ void Component::SetDefaultShapes(const bool& bUsePCBshapes)
 											AddOne( Shape(SHAPE::LINE,		true, false,	-0.21,  0.21, -0.21,  0.21) );
 											AddOne( Shape(SHAPE::LINE,		true, false,	-0.21,  0.21,  0.21, -0.21) );	break;
 		case COMP::PAD:						AddTwo( Shape(SHAPE::ELLIPSE,	true, true,		-0.45,  0.45, -0.45,  0.45) );	break;
+		case COMP::PAD_FLYINGWIRE:			AddTwo( Shape(SHAPE::ROUNDED_RECT, true, true,	-0.45,  0.45, -0.45,  0.45) );	break;
 		case COMP::LED:						AddTwo( Shape(SHAPE::CHORD,		true, true,		-0.75,  0.75, -0.75,  0.75, 30, -30) );break;
 		case COMP::CAP_ELECTRO_200_NP:		AddTwo( Shape(SHAPE::ELLIPSE,	true, true,		-1.00,  1.00, -1.00,  1.00) );	break;
 		case COMP::CAP_ELECTRO_250_NP:		AddTwo( Shape(SHAPE::ELLIPSE,	true, true,		-1.25,  1.25, -1.25,  1.25) );	break;
@@ -323,7 +324,8 @@ void Component::SetDefaultColor()
 		case COMP::CUSTOM:
 		case COMP::TRACKS:
 		case COMP::MARK:					return;
-		case COMP::PAD:						return SetFillColor(MyRGB(0xFFFFDF));
+		case COMP::PAD:
+		case COMP::PAD_FLYINGWIRE:			return SetFillColor(MyRGB(0xFFFFDF));
 		case COMP::LED:						return SetFillColor(MyRGB(0xFF6644));
 		case COMP::CAP_ELECTRO_200:
 		case COMP::CAP_ELECTRO_200_NP:
@@ -393,8 +395,7 @@ void Component::SetDefaultColor()
 		case COMP::STRIP_100:				return SetFillColor(MyRGB(0xFFFFDF));
 		case COMP::BLOCK_100:				return SetFillColor(MyRGB(0xFFFFDF));
 		case COMP::BLOCK_200:				return SetFillColor(MyRGB(0xFFFFDF));
-		case COMP::INVALID:					return;
-		default:	assert(0);	// Unhandled eType
+		default:	assert(GetType() == COMP::INVALID);	return;// Unhandled eType
 	}
 }
 
@@ -404,7 +405,6 @@ void Component::SetDefaultLabelOffsets()
 	m_iLabelOffsetCol = 0;
 	switch( GetType() )
 	{
-//		case COMP::PAD:					m_iLabelOffsetRow =  14;	return;
 		case COMP::LED:					m_iLabelOffsetRow =   8;	return;
 		case COMP::CRYSTAL:
 		case COMP::CAP_ELECTRO_200:
