@@ -23,7 +23,6 @@
 #include "RectManager.h"
 #include "CompDefiner.h"
 
-
 // For 2-layer boards, the following says which is the prefered layer for a pin to make connections.
 // This only affects rendered connections between adjacent pins..
 // It does not affect connectivity or routing.
@@ -43,6 +42,7 @@ class Component : public FootPrint
 {
 public:
 	Component()	: FootPrint() { Clear(); }
+	virtual ~Component() override {}
 	Component(const Component& o) : FootPrint() { *this = o; }
 	void Clear()
 	{
@@ -56,6 +56,7 @@ public:
 		m_iPinFlags		= 0;
 		m_iPadWidth		= 70;
 		m_iHoleWidth	= 40;
+		m_bAllowFlyWire	= false;
 		m_nodeIdPins.clear();
 		m_origIdPins1.clear();
 		m_origIdPins2.clear();
@@ -91,6 +92,7 @@ public:
 
 		BuildDefault(eType);	// Build method for default component
 
+		SetAllowFlyWire(eType == COMP::PAD_FLYINGWIRE);
 		SetDefaultPinFlags();
 		SetDefaultStrings();
 		SetDefaultLabelOffsets();
@@ -141,6 +143,7 @@ public:
 		m_iPinFlags			= o.m_iPinFlags;
 		m_iPadWidth			= o.m_iPadWidth;
 		m_iHoleWidth		= o.m_iHoleWidth;
+		m_bAllowFlyWire		= o.m_bAllowFlyWire;
 		AllocatePins( o.GetNumPins() );
 		std::copy(o.m_nodeIdPins.begin(),	o.m_nodeIdPins.end(),	m_nodeIdPins.begin());
 		std::copy(o.m_origIdPins1.begin(),	o.m_origIdPins1.end(),	m_origIdPins1.begin());
@@ -179,6 +182,7 @@ public:
 				&& m_iPinFlags			== o.m_iPinFlags
 				&& m_iPadWidth			== o.m_iPadWidth
 				&& m_iHoleWidth			== o.m_iHoleWidth
+				&& m_bAllowFlyWire		== o.m_bAllowFlyWire
 				&& GetNumPins()			== o.GetNumPins()
 				&& GetNumShapes()		== o.GetNumShapes();
 		for (size_t i = 0; i < GetNumPins() && bOK; i++)
@@ -200,18 +204,6 @@ public:
 	}
 	bool operator<(const Component& o) const	{ return m_id < o.m_id; }
 	bool operator==(const Component& o) const	{ return m_id == o.m_id; }
-	~Component()
-	{
-		m_nodeIdPins.clear();
-		m_origIdPins1.clear();
-		m_origIdPins2.clear();
-		m_layerPrefs.clear();
-		m_pinOffsetRow.clear();
-		m_pinOffsetCol.clear();
-		m_pinLabels.clear();
-		m_pinAligns.clear();
-		m_shapes.clear();
-	}
 	void SetId(const int& i)					{ m_id = i; }
 	void SetNameStr(const std::string& s)		{ m_nameStr = s; }
 	void SetValueStr(const std::string& s)		{ m_valueStr = s; }
@@ -296,6 +288,7 @@ public:
 	void SetPinFlags(const uchar& i)			{ m_iPinFlags = i; }
 	void SetPadWidth(const int& i)				{ m_iPadWidth = i; }
 	void SetHoleWidth(const int& i)				{ m_iHoleWidth = i; }
+	void SetAllowFlyWire(const bool& b)			{ m_bAllowFlyWire = b; }
 	void AddOne(const Shape& s)					{ m_shapes.push_back(s); }
 	void AddTwo(const Shape& s)	// Adds the shape twice.  Once with fill only, and once with line only
 	{
@@ -355,17 +348,18 @@ public:
 		static Shape	defaultShape;
 		return ( iShapeIndex < m_shapes.size() ) ? m_shapes[iShapeIndex] : defaultShape;
 	}
-	const int&			GetLyr() const			{ return m_lyr; }
-	const int&			GetRow() const			{ return m_row; }
-	const int&			GetCol() const			{ return m_col; }
+	const int&			GetLyr() const				{ return m_lyr; }
+	const int&			GetRow() const				{ return m_row; }
+	const int&			GetCol() const				{ return m_col; }
 //	const int&			GetLabelOffsetRow() const	{ return m_iLabelOffsetRow; }
 //	const int&			GetLabelOffsetCol() const	{ return m_iLabelOffsetCol; }
-	const char&			GetDirection() const	{ return m_direction; }
-	const bool&			GetIsPlaced() const		{ return m_bIsPlaced; }
-	const uchar&		GetPinFlags() const		{ return m_iPinFlags; }
-	const int&			GetPadWidth() const		{ return m_iPadWidth; }
-	const int&			GetHoleWidth() const	{ return m_iHoleWidth; }
-	const std::vector<Shape>& GetShapes() const	{ return m_shapes; }
+	const char&			GetDirection() const		{ return m_direction; }
+	const bool&			GetIsPlaced() const			{ return m_bIsPlaced; }
+	const uchar&		GetPinFlags() const			{ return m_iPinFlags; }
+	const int&			GetPadWidth() const			{ return m_iPadWidth; }
+	const int&			GetHoleWidth() const		{ return m_iHoleWidth; }
+	const bool&			GetAllowFlyWire() const		{ return m_bAllowFlyWire; }
+	const std::vector<Shape>& GetShapes() const		{ return m_shapes; }
 
 	// Helpers for labels
 	void SetDefaultLabelOffsets();
@@ -385,7 +379,7 @@ public:
 		L = -dW;	R = dW;		T = -dH;	B = dH;
 		// Then consider the list of shapes (for direction 'W')
 		double l,r,t,b;	// Working variables
-		for (auto& o : m_shapes)
+		for (const auto& o : m_shapes)
 		{
 			if ( !bFill && o.GetDrawFill() ) continue;	// If view is not drawing filled shapes, skip them
 			o.GetSafeBounds(l, r, t, b);
@@ -408,7 +402,7 @@ public:
 		if ( GetType() == COMP::DIP || GetType() == COMP::SIP )
 		{
 			char buffer[32] = {'\0'};
-			sprintf(buffer, "%d", (int) GetNumPins());
+			sprintf(buffer, "%d", static_cast<int>(GetNumPins()));
 			str += std::string(buffer);		// e.g. "DIP16"
 		}
 		return str;
@@ -421,17 +415,17 @@ public:
 			 GetType() == COMP::STRIP_100 || GetType() == COMP::BLOCK_100 || GetType() == COMP::BLOCK_200 )
 		{
 			char buffer[32] = {'\0'};
-			sprintf(buffer, "%d", (int) GetNumPins());
+			sprintf(buffer, "%d", static_cast<int>(GetNumPins()));
 			str += std::string(buffer);		// e.g. "DIP16"
 		}
 		return str;
 	}
 	// Helpers (account for component direction)
-	const int&			GetCompRows() const	{ return GetRows( GetDirection() ); }
-	const int&			GetCompCols() const	{ return GetCols( GetDirection() ); }
-	int					GetLastRow() const	{ return GetRow() + GetCompRows() - 1; }
-	int					GetLastCol() const	{ return GetCol() + GetCompCols() - 1; }
-	void				GetCompPinOffsets(const size_t& iPinIndex, int& Xmil, int& Ymil) const
+	const int&	GetCompRows() const	{ return GetRows( GetDirection() ); }
+	const int&	GetCompCols() const	{ return GetCols( GetDirection() ); }
+	int			GetLastRow() const	{ return GetRow() + GetCompRows() - 1; }
+	int			GetLastCol() const	{ return GetCol() + GetCompCols() - 1; }
+	void GetCompPinOffsets(const size_t& iPinIndex, int& Xmil, int& Ymil) const
 	{
 		switch ( GetDirection() )
 		{
@@ -441,7 +435,7 @@ public:
 			default:	Xmil =  GetPinOffsetCol(iPinIndex);	Ymil =  GetPinOffsetRow(iPinIndex); return;
 		}
 	}
-	void				SetCompPinOffsets(const size_t& iPinIndex, const int& Xmil, const int& Ymil)
+	void SetCompPinOffsets(const size_t& iPinIndex, const int& Xmil, const int& Ymil)
 	{
 		switch ( GetDirection() )
 		{
@@ -451,15 +445,38 @@ public:
 			default:	SetPinOffsetCol(iPinIndex,  Xmil);	SetPinOffsetRow(iPinIndex,  Ymil); return;
 		}
 	}
-	void				IncCompPinOffsets(const size_t& iPinIndex, const int& dX, const int& dY)
+	void IncCompPinOffsets(const size_t& iPinIndex, const int& dX, const int& dY)
 	{
-			int Xmil, Ymil;
-			GetCompPinOffsets(iPinIndex, Xmil, Ymil);
-			SetCompPinOffsets(iPinIndex, Xmil + dX, Ymil + dY);
+		int Xmil, Ymil;
+		GetCompPinOffsets(iPinIndex, Xmil, Ymil);
+		SetCompPinOffsets(iPinIndex, Xmil + dX, Ymil + dY);
+	}
+	bool GetUniformPinOffsets() const	// Check if all pins have the same offsets
+	{
+		const size_t iSize = GetNumPins();
+		if ( iSize < 2 ) return iSize == 1;
+		bool bAllSame(true);
+		for (size_t i = 1; i < iSize && bAllSame; i++)
+			bAllSame = ( m_pinOffsetRow[i] == m_pinOffsetRow[0] ) && ( m_pinOffsetCol[i] == m_pinOffsetCol[0] );
+		return bAllSame;
+	}
+	void GetCompShapeOffsets(int& Xmil, int& Ymil) const
+	{
+		if ( GetUniformPinOffsets() )
+		{
+			switch ( GetDirection() )
+			{
+				case 'E':	Xmil = -m_pinOffsetCol[0];	Ymil = -m_pinOffsetRow[0]; return;
+				case 'N':	Xmil = -m_pinOffsetRow[0];	Ymil =  m_pinOffsetCol[0]; return;
+				case 'S':	Xmil =  m_pinOffsetRow[0];	Ymil = -m_pinOffsetCol[0]; return;
+				default:	Xmil =  m_pinOffsetCol[0];	Ymil =  m_pinOffsetRow[0]; return;
+			}
+		}
+		Xmil = Ymil = 0;	// Default to 0 offsets
 	}
 	const CompElement*	GetCompElement(const int& compRow, const int& compCol) const
 	{
-		return FootPrint::Get(0, compRow, compCol, (char)GetDirection());
+		return FootPrint::Get(0, compRow, compCol, GetDirection());
 	}
 	bool GetHasNodeId(int nodeId) const
 	{
@@ -481,8 +498,8 @@ public:
 		for (size_t i = 0; i < GetNumPins(); i++)
 		{
 			if ( m_nodeIdPins[i] != BAD_NODEID ) return true;
-			if ( m_pinLabels[i]  != GetDefaultPinLabel(i) ) return true;
-			if ( m_pinAligns[i]  != GetDefaultPinAlign(i, GetNumPins(), GetType()) ) return true;
+			if ( m_pinLabels[i]  != CompTypes::GetDefaultPinLabel(i) ) return true;
+			if ( m_pinAligns[i]  != CompTypes::GetDefaultPinAlign(i, GetNumPins(), GetType()) ) return true;
 		}
 		return false;
 	}
@@ -518,14 +535,14 @@ public:
 
 		switch( GetType() )
 		{
-			case COMP::SIP:				return AllocatePins( GetCols() );
-			case COMP::DIP:				return AllocatePins( 2 * GetCols() );
-			case COMP::STRIP_100:		return AllocatePins( GetCols() );
-			case COMP::BLOCK_100:		return AllocatePins( GetCols() );
-			case COMP::BLOCK_200:		return AllocatePins( ( GetCols() - 1 ) / 2 );
-			case COMP::SWITCH_ST:		return AllocatePins( GetCols() + 1 );
-			case COMP::SWITCH_DT:		return AllocatePins( 3 * ( GetCols() + 1 ) / 2 );
-			case COMP::SWITCH_ST_DIP:	return AllocatePins( 2 * GetCols() );
+			case COMP::SIP:				return AllocatePins( static_cast<size_t>( GetCols() ) );
+			case COMP::DIP:				return AllocatePins( static_cast<size_t>( 2 * GetCols() ) );
+			case COMP::STRIP_100:		return AllocatePins( static_cast<size_t>( GetCols() ) );
+			case COMP::BLOCK_100:		return AllocatePins( static_cast<size_t>( GetCols() ) );
+			case COMP::BLOCK_200:		return AllocatePins( static_cast<size_t>( ( GetCols() - 1 ) / 2 ) );
+			case COMP::SWITCH_ST:		return AllocatePins( static_cast<size_t>( GetCols() + 1 ) );
+			case COMP::SWITCH_DT:		return AllocatePins( static_cast<size_t>( 3 * ( GetCols() + 1 ) / 2 ) );
+			case COMP::SWITCH_ST_DIP:	return AllocatePins( static_cast<size_t>( 2 * GetCols() ) );
 			default:					return;
 		}
 	}
@@ -538,8 +555,8 @@ public:
 	{
 		for (size_t i = 0; i < GetNumPins(); i++)
 		{
-			m_pinLabels[i] = GetDefaultPinLabel(i);
-			m_pinAligns[i] = GetDefaultPinAlign(i, GetNumPins(), GetType());
+			m_pinLabels[i] = CompTypes::GetDefaultPinLabel(i);
+			m_pinAligns[i] = CompTypes::GetDefaultPinAlign(i, GetNumPins(), GetType());
 		}
 	}
 	Rect GetFootprintRect() const
@@ -569,7 +586,7 @@ public:
 		{
 			bool bOK(true);
 			MyRGB tmp(iColor);
-			for (auto& o : m_shapes)
+			for (const auto& o : m_shapes)
 				if ( o.GetFillColor() == tmp ) { bOK = false; break; }
 			if ( bOK ) return tmp;
 		}
@@ -611,11 +628,11 @@ public:
 	void SetDefaultStrings(bool bForce = true)
 	{
 		if ( GetPrefixStr().empty() || bForce )
-			SetPrefixStr( GetDefaultPrefixStr( GetType() ) );
+			SetPrefixStr( CompTypes::GetDefaultPrefixStr( GetType() ) );
 		if ( GetTypeStr().empty() || bForce )
-			SetTypeStr( GetDefaultTypeStr( GetType() ) );
+			SetTypeStr( CompTypes::GetDefaultTypeStr( GetType() ) );
 		if ( GetImportStr().empty() || bForce || GetType() != COMP::CUSTOM )
-			SetImportStr( GetDefaultImportStr( GetType() ) );
+			SetImportStr( CompTypes::GetDefaultImportStr( GetType() ) );
 	}
 	void SetDefaultShapes(const bool& bUsePCBshapes = false);
 	void SetDefaultColor();
@@ -657,6 +674,9 @@ public:
 			inStream.Load(m_iPadWidth);				// Added in VRT_VERSION_39
 			inStream.Load(m_iHoleWidth);			// Added in VRT_VERSION_39
 		}
+		m_bAllowFlyWire = false;
+		if ( inStream.GetVersion() >= VRT_VERSION_47 )
+			inStream.Load(m_bAllowFlyWire);			// Added in VRT_VERSION_47
 		unsigned int numPins(0);
 		inStream.Load(numPins);
 		AllocatePins(numPins);
@@ -712,6 +732,7 @@ public:
 		outStream.Save(m_iPinFlags);			// Added in VRT_VERSION_19
 		outStream.Save(m_iPadWidth);			// Added in VRT_VERSION_39
 		outStream.Save(m_iHoleWidth);			// Added in VRT_VERSION_39
+		outStream.Save(m_bAllowFlyWire);		// Added in VRT_VERSION_47
 		const unsigned int numPins = static_cast<unsigned int>( GetNumPins() );
 		outStream.Save(numPins);
 		for (unsigned int i = 0; i < numPins; i++)
@@ -749,6 +770,7 @@ private:
 	uchar						m_iPinFlags;		// 1 ==> PIN_RECT, 2 ==> PIN_LABELS, 4 ==> PIN_CUSTOM
 	int							m_iPadWidth;		// Used if the PIN_CUSTOM flag is set
 	int							m_iHoleWidth;		// Used if the PIN_CUSTOM flag is set
+	bool						m_bAllowFlyWire;	// For single pin parts only
 	// Current placement in board
 	int							m_lyr;				// Board layer for component
 	int							m_row;				// Board row for top-left element of footprint
