@@ -87,8 +87,8 @@ public:
 	}
 
 	Element() : Pin(), TrackElement() { ZeroConnectionPointers(); }
-	Element(const Element& o) : Pin(o), TrackElement(o)	{ assert(0); *this = o; }	// The assert just shows this is never used
-	~Element() {}
+	Element(const Element& o) : Pin(o), TrackElement(o)	{ *this = o; }	// This is never used
+	virtual ~Element() override {}
 	void ZeroConnectionPointers()
 	{
 		memset(m_pNbr,	0, NUM_NBRS * sizeof(Element*));
@@ -147,11 +147,11 @@ public:
 		m_MH		= BAD_MH;		// Set "infinite" MH distance.
 		m_maxMH		= 0;			// Zero max MH parameter
 	}
-	void UpdateMH(const unsigned int& routeID, const unsigned int& iMH, unsigned int& iMaxMH)
+	void UpdateMH(const unsigned int& iRouteID, const unsigned int& iMH, unsigned int& iMaxMH)
 	{
 		assert( m_MH == BAD_MH );	// Should only ever write the MH once
 		iMaxMH		= std::max(iMaxMH, iMH);	// Update iMaxMH for output before storing it
-		m_routeId	= routeID;
+		m_routeId	= iRouteID;
 		m_MH		= iMH;
 		m_maxMH		= iMaxMH;
 	}
@@ -201,7 +201,7 @@ public:
 			default:	pinIndex = BAD_PININDEX;	compId = BAD_COMPID;	assert(0);
 		}
 	}
-	bool GetWireExists(Element* p) const
+	bool GetWireExists(const Element* p) const
 	{
 		return p != nullptr && ( GetW(0) == p || GetW(1) == p );
 	}
@@ -429,6 +429,6 @@ private:
 	unsigned int	m_MH		= BAD_MH;		// Manhatten distance to another element.  For the routing/connectivity algorithm.
 	unsigned int	m_maxMH		= 0;			// For the routing algorithm.
 	// Connection pointers. Set by Board::GlueNbrs() and Board::GlueWires().	Don't persist.
-	Element*		m_pNbr[(size_t)NUM_NBRS];	// 0 to 7 <==> NBR_L to NBR_LB,	  8 ==> NBR_X
-	Element*		m_pW[2];					// Up to 2 wires per element. These point to the other end of the wire(s).
+	Element*		m_pNbr[static_cast<size_t>(NUM_NBRS)];	// 0 to 7 <==> NBR_L to NBR_LB,	  8 ==> NBR_X
+	Element*		m_pW[2];								// Up to 2 wires per element. These point to the other end of the wire(s).
 };
