@@ -24,9 +24,9 @@
 class RectManager : public Persist
 {
 public:
-	RectManager()	{ Clear(); }
-	~RectManager()	{ Clear(); }
-	void Clear()	{ m_startRow = m_startCol = 0; m_current.SetInvalid(); m_bounding.SetInvalid(), m_list.clear(); }
+	RectManager()	{}
+	virtual ~RectManager() {}
+	void Clear()	{ m_startRow = m_startCol = 0; m_current.SetInvalid(); m_bounding.SetInvalid(); m_list.clear(); }
 	RectManager(const RectManager& o)	{ *this = o; }
 	RectManager& operator=(const RectManager& o)
 	{
@@ -35,7 +35,7 @@ public:
 		m_current	= o.m_current;
 		m_bounding	= o.m_bounding;
 		m_list.clear();
-		for (auto& oo : o.m_list) m_list.push_back( oo );
+		for (const auto& oo : o.m_list) m_list.push_back( oo );
 		return *this;
 	}
 	bool operator==(const RectManager& o) const	// Compare persisted info
@@ -79,7 +79,7 @@ public:
 	{
 		if ( m_current.ContainsPoint(row, col) ) return true;		// Check current before the list
 		if ( !m_bounding.ContainsPoint(row, col) ) return false;	// Check bounding rect of the list first
-		for (auto& o : m_list) if ( o.ContainsPoint(row,col) ) return true;
+		for (const auto& o : m_list) if ( o.ContainsPoint(row,col) ) return true;
 		return false;
 	}
 	void MoveAll(int iDown, int iRight)
@@ -128,8 +128,8 @@ private:
 		m_bounding |= rect;	// Update bounding rectangle
 	}
 private:
-	int					m_startRow;
-	int					m_startCol;
+	int					m_startRow = 0;
+	int					m_startCol = 0;
 	mutable Rect		m_current;	// The rectangle currently being defined by the user
 	mutable Rect		m_bounding;	// The bounding rectangle of m_list
 	std::vector<Rect>	m_list;		// The list of completed rectangles
