@@ -31,10 +31,12 @@
 #include "templatesdialog.h"
 #include "pindialog.h"
 #include "finddialog.h"
+#include "PolygonHelper.h"
 
 MainWindow::MainWindow(const QString& localDataPathStr, const QString& tutorialsPathStr, QWidget* parent)
 : QMainWindow(parent)
 , ui(new Ui::MainWindow)
+, m_mouseActionString("Action")
 , m_localDataPathStr(localDataPathStr.toStdString())
 , m_tutorialsPathStr(tutorialsPathStr.toStdString())
 {
@@ -43,13 +45,13 @@ MainWindow::MainWindow(const QString& localDataPathStr, const QString& tutorials
 	m_historyMgr.SetPathStr(m_localDataPathStr);
 	m_templateMgr.SetPathStr(m_localDataPathStr);
 
-	InitMapsCompTypeToStr();			// Put all comp types into a list ...
-	GetTemplateManager().AddDefaults();	// .. and create default templates
+	CompTypes::InitMapsCompTypeToStr();	// Put all comp types into a list ...
+	GetTemplateManager().AddDefaults();			// .. and create default templates
 
 	ui->setupUi(this);
 
 	// Create menu actions for recent files
-	for (int i = 0; i < MAX_RECENT_FILES; ++i)
+	for (size_t i = 0; i < MAX_RECENT_FILES; ++i)
 	{
 		m_recentFileAction[i] = new QAction(this);
 		m_recentFileAction[i]->setVisible(false);
@@ -109,6 +111,7 @@ MainWindow::MainWindow(const QString& localDataPathStr, const QString& tutorials
 	m_labelStatus->setFrameStyle(QFrame::NoFrame);
 	ui->statusBar->addPermanentWidget(m_labelStatus, 0);
 
+	m_rulerPen			= QPen(QColor(255,255,0,192), 0,	Qt::SolidLine, Qt::FlatCap);	// using alpha
 	m_backgroundPen		= QPen(Qt::white, 0,				Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
 	m_darkGreyPen		= QPen(QColor(96,96,96,255), 0,		Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
 	m_blackPen			= QPen(Qt::black, 0,				Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
@@ -120,7 +123,7 @@ MainWindow::MainWindow(const QString& localDataPathStr, const QString& tutorials
 	m_dotPen			= QPen(QColor(96,96,96,255), 0,		Qt::DotLine,   Qt::RoundCap, Qt::RoundJoin);
 	m_dashPen			= QPen(QColor(96,96,96,255), 0,		Qt::DashLine,  Qt::RoundCap, Qt::RoundJoin);
 	m_backgroundBrush	= QBrush(Qt::white,					Qt::SolidPattern);
-	m_darkBrush			= QBrush(QColor(0, 0, 0, 150),		Qt::SolidPattern);	// using alpha
+	m_darkBrush			= QBrush(QColor(0,0,0,150),			Qt::SolidPattern);	// using alpha
 	m_varBrush			= QBrush(QColor(255,255,255,0),		Qt::SolidPattern);
 
 	QObject::connect(ui->actionNew,						SIGNAL(triggered()), this, SLOT(New()));
@@ -146,6 +149,8 @@ MainWindow::MainWindow(const QString& localDataPathStr, const QString& tutorials
 	QObject::connect(ui->actionToggleFlipH,				SIGNAL(triggered()), this, SLOT(ToggleFlipH()));
 	QObject::connect(ui->actionToggleFlipV,				SIGNAL(triggered()), this, SLOT(ToggleFlipV()));
 	QObject::connect(ui->actionTogglePinLabels,			SIGNAL(triggered()), this, SLOT(TogglePinLabels()));
+	QObject::connect(ui->actionToggleFlyWires,			SIGNAL(triggered()), this, SLOT(ToggleFlyWires()));
+	QObject::connect(ui->actionToggleRuler,				SIGNAL(triggered()), this, SLOT(ToggleRuler()));
 	QObject::connect(ui->actionVeroV,					SIGNAL(triggered()), this, SLOT(VeroV()));
 	QObject::connect(ui->actionVeroH,					SIGNAL(triggered()), this, SLOT(VeroH()));
 	QObject::connect(ui->actionFat,						SIGNAL(triggered()), this, SLOT(Fat()));
@@ -165,6 +170,7 @@ MainWindow::MainWindow(const QString& localDataPathStr, const QString& tutorials
 	QObject::connect(ui->actionDelete,					SIGNAL(triggered()), this, SLOT(Delete()));
 	QObject::connect(ui->actionMarker,					SIGNAL(triggered()), this, SLOT(AddMarker()));
 	QObject::connect(ui->actionPad,						SIGNAL(triggered()), this, SLOT(AddPad()));
+	QObject::connect(ui->actionPad_FlyWire,				SIGNAL(triggered()), this, SLOT(AddPadFlyWire()));
 	QObject::connect(ui->actionStrip100,				SIGNAL(triggered()), this, SLOT(AddStrip100()));
 	QObject::connect(ui->actionBlock100,				SIGNAL(triggered()), this, SLOT(AddBlock100()));
 	QObject::connect(ui->actionBlock200,				SIGNAL(triggered()), this, SLOT(AddBlock200()));
@@ -342,12 +348,15 @@ void MainWindow::CheckFolders()
 void MainWindow::ResetView(bool bTutorial)
 {
 	m_mousePos = QPoint(0,0);
-	m_bMouseClick	= m_bLeftClick	= m_bRightClick = m_bCtrlKeyDown  = m_bShiftKeyDown	=false;
+	m_bMouseClick	= m_bLeftClick	= m_bRightClick = m_bCtrlKeyDown  = m_bShiftKeyDown	= false;
 	m_bPaintPins	= m_bPaintBoard	= m_bPaintFlood = m_bPaintLyrPref = m_bDefiningRect = m_bResizingText = m_bWritePDF = m_bWriteGerber = m_bTwoLayerGerber = false;
 	m_XGRIDOFFSET	= m_YGRIDOFFSET	= m_XCORRECTION = m_YCORRECTION = 0;
 
 	// Try to set m_gridRow, m_gridCol to match the current NodeId in the board
 	m_gridRow = m_gridCol = 0;
+
+	ResetRuler();	// Reset the ruler
+
 	bool bOK(false);	// true ==> found
 	const int k = m_board.GetCurrentLayer();
 	for (int j = 0, jMax = m_board.GetRows(); j < jMax && !bOK; j++)
@@ -363,8 +372,8 @@ void MainWindow::ResetView(bool bTutorial)
 	m_infoDlg->Update();
 	m_infoDlg->SetReadOnly(bTutorial);
 	m_infoDlg->ShowButtons(bTutorial);
-	m_infoDlg->EnablePrev(bTutorial && m_iTutorialNumber > 0);	// Tutorials go to 0 to 21
-	m_infoDlg->EnableNext(bTutorial && m_iTutorialNumber < 21);	// Tutorials go to 0 to 21
+	m_infoDlg->EnablePrev(bTutorial && m_iTutorialNumber > 0);	// Tutorials go to 0 to 22
+	m_infoDlg->EnableNext(bTutorial && m_iTutorialNumber < 22);	// Tutorials go to 0 to 22
 	if ( !bTutorial ) m_iTutorialNumber = -1;	// Cancel tutorial mode
 	UpdateControls();
 	UpdateBOM();
@@ -424,6 +433,7 @@ void MainWindow::RepaintWithListNodes(bool bNow)
 	ListNodes();	// Slow due lots of MH calcs
 	m_bRepaint = true;
 	if ( bNow ) repaint(); else update();
+	UpdateRulerInfo();
 }
 
 void MainWindow::RepaintWithRouting(bool bNow)
@@ -432,6 +442,7 @@ void MainWindow::RepaintWithRouting(bool bNow)
 	HandleRouting();
 	m_bRepaint = true;
 	if ( bNow ) repaint(); else update();
+	UpdateRulerInfo();
 }
 
 void MainWindow::RepaintSkipRouting(bool bNow)
@@ -439,13 +450,14 @@ void MainWindow::RepaintSkipRouting(bool bNow)
 	UpdateWindowTitle();
 	m_bRepaint = true;
 	if ( bNow ) repaint(); else update();
+	UpdateRulerInfo();
 }
 
 void MainWindow::ShowCurrentRectSize()
 {
-	auto& rect = m_board.GetRectMgr().GetCurrent();
+	const auto& rect = m_board.GetRectMgr().GetCurrent();
 	char buffer[256] = {'\0'};
-	sprintf(buffer,"Current rectangle (%d x %d)    (%gmm x %gmm)", rect.GetCols(), rect.GetRows(), rect.GetCols()*2.54, rect.GetRows()*2.54);
+	sprintf(buffer,"Current rectangle = (%d mil x %d mil),    (%.2f mm x %.2f mm)", rect.GetCols()*100, rect.GetRows()*100, rect.GetCols()*2.54, rect.GetRows()*2.54);
 	ui->statusBar->showMessage(QString(buffer), 1000);
 }
 
@@ -527,7 +539,7 @@ void MainWindow::OpenRecent()
 										 QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::No ) return;
 	}
 
-	QAction* pAction = qobject_cast<QAction*> ( sender() );
+	QAction* pAction = qobject_cast<QAction*>( sender() );
 	if ( pAction )
 	{
 		QString fileName = pAction->data().toString();
@@ -734,7 +746,7 @@ void MainWindow::ClearRecentFiles()
 {
 	QSettings	settings("veroroute","veroroute");	// Organisation = "veroroute", Application = "veroroute"
 	settings.setValue("recentFiles", QStringList());
-	for (int i = 0; i < MAX_RECENT_FILES; i++)
+	for (size_t i = 0; i < MAX_RECENT_FILES; i++)
 		m_recentFileAction[i]->setVisible(false);
 	ui->actionClearRecent->setEnabled(false);
 	m_separator->setVisible(false);
@@ -807,8 +819,8 @@ void MainWindow::ZoomHelper(int delta)	// delta == change in GRIDPIXELS
 	RepaintSkipRouting(true);
 
 	// Try to have same grid position under mouse after zoom
-	pH->setValue(L + X * delta * 1.0 / W);
-	pV->setValue(T + Y * delta * 1.0 / W);
+	pH->setValue(static_cast<int>(L + X * delta * 1.0 / W));
+	pV->setValue(static_cast<int>(T + Y * delta * 1.0 / W));
 }
 
 // Edit menu items
@@ -991,7 +1003,7 @@ void MainWindow::ShowAbout()
 {
 	std::string str = std::string("VeroRoute - Qt based Veroboard/Perfboard/PCB layout & routing application.\n\n")
 					+ std::string("Version ") + std::string(szVEROROUTE_VERSION) + std::string("\n\n")
-					+ std::string("Copyright (C) 2017  Alex Lawrow    ( dralx@users.sourceforge.net )\n\n")
+					+ std::string("Copyright (C) 2017-2021  Alex Lawrow    ( dralx@users.sourceforge.net )\n\n")
 					+ std::string("This program is free software: you can redistribute it and/or modify\n")
 					+ std::string("it under the terms of the GNU General Public License as published by\n")
 					+ std::string("the Free Software Foundation, either version 3 of the License, or\n")
@@ -1088,7 +1100,7 @@ void MainWindow::HandleNetworkReply(QNetworkReply* pReply)
 
 	if ( dSiteVersion > dThisVersion )
 		QMessageBox::information(this, tr(bufferThis), tr(bufferOther));
-	else if ( dSiteVersion == dThisVersion )
+	else if ( dSiteVersion - dThisVersion == 0.0 )
 		QMessageBox::information(this, tr(bufferThis), tr("You have the latest version."));
 	else
 		QMessageBox::information(this, tr(bufferThis), tr("You have a newer version than Sourceforge."));
@@ -1104,6 +1116,7 @@ void MainWindow::SetShowText(bool b)			{ if ( m_board.SetShowText(b) )			{ Updat
 void MainWindow::SetFlipH(bool b)				{ if ( m_board.SetFlipH(b) )			{ UpdateHistory("Toggle flip horizontal");	UpdateControls(); RepaintSkipRouting(); } }
 void MainWindow::SetFlipV(bool b)				{ if ( m_board.SetFlipV(b) )			{ UpdateHistory("Toggle flip vertical");	UpdateControls(); RepaintSkipRouting(); } }
 void MainWindow::SetShowPinLabels(bool b)		{ if ( m_board.SetShowPinLabels(b) )	{ UpdateHistory("Toggle pin labels");		UpdateControls(); RepaintSkipRouting(); } }
+void MainWindow::SetShowFlyWires(bool b)		{ if ( m_board.SetShowFlyWires(b) )		{ UpdateHistory("Toggle flying wires");		UpdateControls(); RepaintSkipRouting(); } }
 void MainWindow::SetFill(bool b)
 {
 	if ( m_board.SetGroundFill(b) )
@@ -1122,6 +1135,16 @@ void MainWindow::ToggleText()			{ SetShowText( !m_board.GetShowText() ); if ( !m
 void MainWindow::ToggleFlipH()			{ SetFlipH( !m_board.GetFlipH() ); }
 void MainWindow::ToggleFlipV()			{ SetFlipV( !m_board.GetFlipV() ); }
 void MainWindow::TogglePinLabels()		{ SetShowPinLabels( !m_board.GetShowPinLabels() ); }
+void MainWindow::ToggleFlyWires()		{ SetShowFlyWires( !m_board.GetShowFlyWires() ); }
+void MainWindow::ToggleRuler()			{ m_bRuler = !m_bRuler; if ( !m_bRuler ) ResetRuler();
+										  UpdateControls(); RepaintSkipRouting();
+										}
+void MainWindow::ResetRuler()
+{
+	m_bRuler = m_bModifyRulerA = false;
+	int X(m_board.GetCols()/3), Y(m_board.GetRows() / 2);
+	m_rulerA = QPoint(X,Y); m_rulerB = QPoint(2*X,Y);
+}
 
 // Toolbar items
 void MainWindow::VeroV()				{ SetTracksVeroV(true); }
@@ -1132,14 +1155,7 @@ void MainWindow::Curved()				{ SetTracksCurved(true); }
 void MainWindow::ToggleDiagsMin()		{ if ( m_board.GetDiagsMode() == DIAGSMODE::MIN ) SetDiagonalsOff(true); else SetDiagonalsMin(true); }
 void MainWindow::ToggleDiagsMax()		{ if ( m_board.GetDiagsMode() == DIAGSMODE::MAX ) SetDiagonalsOff(true); else SetDiagonalsMax(true); }
 void MainWindow::ToggleFill()			{ SetFill( !m_board.GetGroundFill() ); }
-void MainWindow::ToggleSelectArea()
-{
-	SetDefiningRect( !GetDefiningRect() );
-	if ( GetDefiningRect() )
-		centralWidget()->setCursor(Qt::SizeFDiagCursor);
-	else
-		centralWidget()->setCursor(Qt::OpenHandCursor);
-}
+void MainWindow::ToggleSelectArea()		{ SetDefiningRect( !GetDefiningRect() ); }
 
 // Part controls
 void MainWindow::SetCompName(const QString& str)
@@ -1169,7 +1185,7 @@ void MainWindow::SetCompType(const QString& str)
 	if ( m_board.GetDisableChangeType() || str.isEmpty() ) return;
 	if ( m_board.GetGroupMgr().GetNumUserComps() != 1 ) return;
 	Component&	comp	= m_board.GetUserComponent();
-	const COMP	eType	= GetTypeFromTypeStr( str.toStdString() );
+	const COMP	eType	= CompTypes::GetTypeFromTypeStr( str.toStdString() );
 	if ( eType == comp.GetType() ) return;	// No change
 	m_board.ChangeTypeUserComp(eType);
 	UpdateHistory("Change part type");
@@ -1324,15 +1340,17 @@ void MainWindow::Paste()		// On hitting the Paste button ...
 	if ( !m_board.GetRoutingEnabled() ) return;
 	m_board.PasteTracks(false);	// false ==> Don't wipe redundant track portions
 	if ( m_board.GetVeroTracks() )  m_board.AutoFillVero();
+	SetDefiningRect( false );
 	UpdateHistory("Paste Track");
 	UpdateControls();
 	RepaintWithListNodes();
 }
-void MainWindow::Tidy()	// On hitting the Paste+Tidy button ...
+void MainWindow::Tidy()			// On hitting the Paste+Tidy button ...
 {
 	if ( m_board.GetRoutingEnabled() ) return;
 	m_board.PasteTracks(true);	// true ==> Wipe redundant track portions
 	if ( m_board.GetVeroTracks() ) m_board.AutoFillVero();
+	SetDefiningRect( false );
 	UpdateHistory("Tidy Tracks");
 	UpdateControls();
 	RepaintWithListNodes();
@@ -1341,7 +1359,7 @@ void MainWindow::WipeTracks()	// On hitting the Wipe All button ...
 {
 	if ( m_board.GetDisableWipe() ) return;
 	m_board.WipeTracks();
-	m_board.SetRoutingEnabled(false);
+	SetDefiningRect( false );
 	UpdateHistory("Wipe Tracks");
 	UpdateControls();
 	RepaintWithListNodes();
@@ -1374,7 +1392,7 @@ void MainWindow::SelectNodeColor()
 }
 
 // Track controls
-DIAGSMODE oldDiagsMode(DIAGSMODE::OFF);
+static DIAGSMODE oldDiagsMode(DIAGSMODE::OFF);
 
 void MainWindow::SetTracksVeroV(bool b)
 {
@@ -1457,6 +1475,7 @@ void MainWindow::SetDiagonalsMax(bool b)
 }
 
 // Rendering options
+void MainWindow::SetBrightness(int i)		{ if ( m_board.SetBackgroundColor(MyRGB(i,i,i)) )	{ UpdateHistory("Background brightness change");	DestroyPixmapCache();	RepaintSkipRouting(); } }
 void MainWindow::SetPadWidth(int i)			{ if ( m_board.SetPAD_MIL(i)   ) { UpdateHistory("Pad width change");				UpdateControls();	DestroyPixmapCache();	RepaintSkipRouting(); } }
 void MainWindow::SetTrackWidth(int i)		{ if ( m_board.SetTRACK_MIL(i) ) { UpdateHistory("Track width change");				UpdateControls();	DestroyPixmapCache();	RepaintSkipRouting(); } }
 void MainWindow::SetHoleWidth(int i)		{ if ( m_board.SetHOLE_MIL(i)  ) { UpdateHistory("Hole width change");				UpdateControls();	RepaintSkipRouting(); } }
@@ -1468,13 +1487,13 @@ void MainWindow::SetViaPadWidth(int i)		{ if ( m_board.SetVIAPAD_MIL(i)) { Updat
 void MainWindow::SetViaHoleWidth(int i)		{ if ( m_board.SetVIAHOLE_MIL(i)){ UpdateHistory("Via hole width change");			UpdateControls();	DestroyPixmapCache();	RepaintSkipRouting(); } }
 void MainWindow::SetTextSizeComp(int i)		{ if ( m_board.SetTextSizeComp(i) )		  { UpdateHistory("Text size change (component)");	RepaintSkipRouting(); } }
 void MainWindow::SetTextSizePins(int i)		{ if ( m_board.SetTextSizePins(i) )		  { UpdateHistory("Text size change (pins)");		RepaintSkipRouting(); } }
+
 void MainWindow::SetTargetRows(int i)		{ if ( m_board.SetTargetRows(i) )		  { UpdateHistory("Target board height change");	RepaintSkipRouting(); } }
 void MainWindow::SetTargetCols(int i)		{ if ( m_board.SetTargetCols(i) )		  { UpdateHistory("Target board width change");		RepaintSkipRouting(); } }
 void MainWindow::SetShowTarget(bool b)		{ if ( m_board.SetShowTarget(b) )		  { UpdateHistory("Target board on/off");			RepaintSkipRouting(); } }
 void MainWindow::SetShowCloseTracks(bool b)	{ if ( m_board.SetShowCloseTracks(b) )	  { UpdateHistory("Show closest tracks on/off");	UpdateControls();	RepaintSkipRouting(); } }
 void MainWindow::SetAntialiasOff(bool b)	{ if ( b && m_board.SetRenderQuality(0) ) { UpdateHistory("Anti-alias off");  DestroyPixmapCache(); RepaintSkipRouting(); } }
 void MainWindow::SetAntialiasOn(bool b)		{ if ( b && m_board.SetRenderQuality(1) ) { UpdateHistory("Anti-alias on");   DestroyPixmapCache(); RepaintSkipRouting(); } }
-void MainWindow::SetAntialiasHigh(bool b)	{ if ( b && m_board.SetRenderQuality(2) ) { UpdateHistory("Anti-alias high"); DestroyPixmapCache(); RepaintSkipRouting(); } }
 
 // Wire dialog
 void MainWindow::SetWireShare(bool b)		{ if ( m_board.SetWireShare(b) ) { UpdateHistory("Wire hole-sharing on/off");	RepaintSkipRouting(); } }
@@ -1491,10 +1510,13 @@ void MainWindow::SizeChanged(int i)			{ if ( GetCurrentTextId() != BAD_TEXTID &&
 void MainWindow::ToggleBold()				{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetStyle(GetCurrentTextRect().GetStyle() ^ TEXT_BOLD     ) ){ UpdateTextDialog(); RepaintSkipRouting(); } }
 void MainWindow::ToggleItalic()				{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetStyle(GetCurrentTextRect().GetStyle() ^ TEXT_ITALIC   ) ){ UpdateTextDialog(); RepaintSkipRouting(); } }
 void MainWindow::ToggleUnderline()			{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetStyle(GetCurrentTextRect().GetStyle() ^ TEXT_UNDERLINE) ){ UpdateTextDialog(); RepaintSkipRouting(); } }
-void MainWindow::AlignL()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlags(Qt::AlignLeft) )									{ UpdateTextDialog(); RepaintSkipRouting(); } }
-void MainWindow::AlignC()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlags(Qt::AlignHCenter) )								{ UpdateTextDialog(); RepaintSkipRouting(); } }
-void MainWindow::AlignR()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlags(Qt::AlignRight) )									{ UpdateTextDialog(); RepaintSkipRouting(); } }
-void MainWindow::AlignJ()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlags(Qt::AlignJustify) )								{ UpdateTextDialog(); RepaintSkipRouting(); } }
+void MainWindow::AlignL()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlagsH(Qt::AlignLeft) )									{ UpdateTextDialog(); RepaintSkipRouting(); } }
+void MainWindow::AlignC()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlagsH(Qt::AlignHCenter) )								{ UpdateTextDialog(); RepaintSkipRouting(); } }
+void MainWindow::AlignR()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlagsH(Qt::AlignRight) )									{ UpdateTextDialog(); RepaintSkipRouting(); } }
+void MainWindow::AlignJ()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlagsH(Qt::AlignJustify) )								{ UpdateTextDialog(); RepaintSkipRouting(); } }
+void MainWindow::AlignTop()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlagsV(Qt::AlignTop) )									{ UpdateTextDialog(); RepaintSkipRouting(); } }
+void MainWindow::AlignMid()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlagsV(Qt::AlignVCenter) )								{ UpdateTextDialog(); RepaintSkipRouting(); } }
+void MainWindow::AlignBot()					{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetFlagsV(Qt::AlignBottom) )								{ UpdateTextDialog(); RepaintSkipRouting(); } }
 void MainWindow::SetText(const QString& s)	{ if ( GetCurrentTextId() != BAD_TEXTID && GetCurrentTextRect().SetStr(s.toStdString()) )									{ UpdateTextDialog(); RepaintSkipRouting(); } }
 void MainWindow::ChooseTextColor()
 {
@@ -1670,21 +1692,21 @@ void MainWindow::UpdateRecentFiles(const QString* pFileName, bool bAdd)
 		files.removeAll(fileName);
 		if ( bAdd )
 			files.prepend(fileName);
-		while ( files.size() > MAX_RECENT_FILES )
+		while ( static_cast<size_t>(files.size()) > MAX_RECENT_FILES )
 			files.removeLast();
 		settings.setValue("recentFiles", files);
 	}
 
-	const int numFiles = std::min(files.size(), MAX_RECENT_FILES);
-	for (int i = 0; i < numFiles; i++)
+	const size_t numFiles = std::min(static_cast<size_t>(files.size()), MAX_RECENT_FILES);
+	for (size_t i = 0; i < numFiles; i++)
 	{
-		const QString&	fileName	= files[i];
+		const QString&	fileName	= files[static_cast<int>(i)];
 		const QString	text		= tr("&%1 %2").arg( i + 1 ).arg( fileName );
 		m_recentFileAction[i]->setText(text);
 		m_recentFileAction[i]->setData(fileName);
 		m_recentFileAction[i]->setVisible(true);
 	}
-	for (int i = numFiles; i < MAX_RECENT_FILES; i++)
+	for (size_t i = numFiles; i < MAX_RECENT_FILES; i++)
 		m_recentFileAction[i]->setVisible(false);
 
 	ui->actionClearRecent->setEnabled(numFiles > 0);
@@ -1703,6 +1725,23 @@ void MainWindow::UpdateWindowTitle()
 	}
 	setWindowTitle(title);
 }
+
+void MainWindow::UpdateRulerInfo()
+{
+	if ( m_bRuler && !m_bWriteGerber && !m_bMouseClick && !GetCtrlKeyDown() )	// Ctrl key ==> status bar might be displaying pad offsets
+	{
+		QPointF A,B;
+		GetRulerExact(m_board, m_rulerA, A);
+		GetRulerExact(m_board, m_rulerB, B);
+
+		const qreal	d_mil	= PolygonHelper::Length(B-A) * 100;	// mil
+		const qreal	d_mm	= d_mil * 0.0254;
+		char buffer[256] = {'\0'};
+		sprintf(buffer,"Distance = %.2f mil,    %.4f mm", d_mil, d_mm);
+		ui->statusBar->showMessage(QString(buffer), 1000);
+	}
+}
+
 void MainWindow::UpdateControls()
 {
 	GroupManager&	groupMgr		=  m_board.GetGroupMgr();
@@ -1767,12 +1806,19 @@ void MainWindow::UpdateControls()
 	ui->actionToggleFlipV->setEnabled( !bCompEdit );
 	const bool bPinLabels = m_board.GetCompMode() != COMPSMODE::OFF && ( bNoTracks || bColor );
 	ui->actionTogglePinLabels->setEnabled( bPinLabels );
+	const bool bFlyWires  = m_board.GetCompMode() != COMPSMODE::OFF && ( bNoTracks || bColor );
+	ui->actionToggleFlyWires->setEnabled( bFlyWires );
+	ui->actionToggleRuler->setEnabled( true );
 
 	ui->actionToggleGrid->setChecked( m_board.GetShowGrid() && !bPCB );
 	ui->actionToggleText->setChecked( m_board.GetShowText() && !bCompEdit && !bPCB );
 	ui->actionToggleFlipH->setChecked( m_board.GetFlipH()   && !bCompEdit );
 	ui->actionToggleFlipV->setChecked( m_board.GetFlipV()   && !bCompEdit );
 	ui->actionTogglePinLabels->setChecked( m_board.GetShowPinLabels() && bPinLabels );
+	ui->actionToggleFlyWires->setChecked( m_board.GetShowFlyWires() && bFlyWires );
+	ui->actionToggleFlyWires->setText( m_board.GetShowFlyWires() ? QString("Hide Flying Wires") : QString("Show Flying Wires"));
+	ui->actionToggleRuler->setChecked( m_bRuler );
+	ui->actionToggleRuler->setText( m_bRuler ? QString("Hide Distance Tool") : QString("Show Distance Tool"));
 
 	ui->actionPinDlg->setEnabled( !bCompEdit );
 	ui->actionControlDlg->setEnabled( !bCompEdit );
@@ -1835,15 +1881,7 @@ void MainWindow::UpdateTextDialog(bool bFull)
 }
 
 // Helpers
-void MainWindow::SetQuality(QPainter& painter)
-{
-	switch( m_board.GetRenderQuality() )
-	{
-		case 0:		return painter.setRenderHint(QPainter::Antialiasing, false);
-		case 1:		return painter.setRenderHint(QPainter::Antialiasing, true);
-		default:	return painter.setRenderHint(QPainter::HighQualityAntialiasing, true);
-	}
-}
+void MainWindow::SetQuality(QPainter& painter) { painter.setRenderHint(QPainter::Antialiasing, m_board.GetRenderQuality() != 0); }
 bool MainWindow::CanZoomIn() const	{ return m_board.GetGRIDPIXELS() < 256; }	// 256 == MAX_GRIDPIXELS
 bool MainWindow::CanZoomOut() const	{ return m_board.GetGRIDPIXELS() > 6;   }	//   6 == MIN_GRIDPIXELS
 bool MainWindow::GetIsModified() const
@@ -1911,5 +1949,5 @@ QString MainWindow::GetSaveFileName(const QString& caption, const QString& nameF
 
 QColor MainWindow::GetBackgroundColor() const
 {
-	return ( m_board.GetTrackMode() == TRACKMODE::PCB ) ? Qt::black : Qt::white;	// For screen only.  PDF is always white.
+	return ( m_board.GetTrackMode() == TRACKMODE::PCB ) ? Qt::black : m_board.GetBackgroundColor().GetQColor();	// For screen only.  PDF is always white.
 }
