@@ -19,19 +19,19 @@
 
 #pragma once
 
-#include <QApplication>
-#include <QMainWindow>
-#include <QDockWidget>
-#include <QListWidgetItem>
 #include <QtGui>
-#include <QWidget>
-#include <QFileDialog>
+#include <QApplication>
 #include <QColorDialog>
-#include <QMessageBox>
+#include <QDockWidget>
+#include <QFileDialog>
 #include <QLabel>
+#include <QListWidgetItem>
+#include <QMainWindow>
+#include <QMessageBox>
 #include <QNetworkAccessManager>
-#include <QNetworkRequest>
 #include <QNetworkReply>
+#include <QNetworkRequest>
+#include <QWidget>
 #include "HistoryManager.h"
 #include "GWriter.h"
 #include "myscrollarea.h"
@@ -50,7 +50,7 @@ class BomDialog;
 class PinDialog;
 class FindDialog;
 
-#define MAX_RECENT_FILES 10
+const size_t MAX_RECENT_FILES = 10;
 
 class MainWindow : public QMainWindow
 {
@@ -69,7 +69,7 @@ class MainWindow : public QMainWindow
 	Q_OBJECT
 
 public:
-	explicit MainWindow(const QString& localDataPathStr, const QString& tutorialsPathStr, QWidget* parent = 0);
+	explicit MainWindow(const QString& localDataPathStr, const QString& tutorialsPathStr, QWidget* parent = nullptr);
 	~MainWindow();
 
 	void ResetView(bool bTutorial = false);
@@ -86,6 +86,7 @@ public:
 	void GetLRTB(const GuiControl& guiCtrl, const Component& comp, int& L, int& R, int& T, int& B) const;
 	void GetLRTB(const GuiControl& guiCtrl, const Rect& rect, int& L, int& R, int& T, int& B) const;
 	void GetXY(const GuiControl& guiCtrl, const Component& comp, int& X, int& Y) const;
+	void GetRulerExact(Board& board, const QPoint& p, QPointF& pOut) const;
 
 	const bool&	GetCtrlKeyDown() const	{ return m_bCtrlKeyDown;	}
 	const bool&	GetShiftKeyDown() const	{ return m_bShiftKeyDown;	}
@@ -97,11 +98,11 @@ public:
 	const bool&	GetResizingText() const	{ return m_bResizingText;	}
 	void		SetCtrlKeyDown(bool b)	{ m_bCtrlKeyDown	= b; }
 	void		SetShiftKeyDown(bool b)	{ m_bShiftKeyDown	= b; }
-	void		SetPaintPins(bool b)	{ m_bPaintPins		= b; }
-	void		SetPaintBoard(bool b)	{ m_bPaintBoard		= b; }
-	void		SetPaintFlood(bool b)	{ m_bPaintFlood		= b; }
-	void		SetPaintLyrPref(bool b)	{ m_bPaintLyrPref	= b; }
-	void		SetDefiningRect(bool b)	{ m_bDefiningRect	= b; }
+	void		SetPaintPins(bool b);
+	void		SetPaintBoard(bool b);
+	void		SetPaintFlood(bool b);
+	void		SetPaintLyrPref(bool b);
+	void		SetDefiningRect(bool b);
 	void		SetResizingText(bool b)	{ m_bResizingText	= b; }
 protected:
 	void paintEvent(QPaintEvent* event);
@@ -146,6 +147,8 @@ public slots:
 	void ToggleFlipH();
 	void ToggleFlipV();
 	void TogglePinLabels();
+	void ToggleFlyWires();
+	void ToggleRuler();
 	// Toolbar items
 	void VeroV();
 	void VeroH();
@@ -168,6 +171,7 @@ public slots:
 	// Add menu items
 	void AddMarker()			{ AddPart(COMP::MARK); }
 	void AddPad()				{ AddPart(COMP::PAD); }
+	void AddPadFlyWire()		{ AddPart(COMP::PAD_FLYINGWIRE); }
 	void AddWire()				{ AddPart(COMP::WIRE); }
 	void AddResistor()			{ AddPart(COMP::RESISTOR); }
 	void AddInductor()			{ AddPart(COMP::INDUCTOR); }
@@ -261,6 +265,7 @@ public slots:
 	void SetFlipH(bool b);
 	void SetFlipV(bool b);
 	void SetShowPinLabels(bool b);
+	void SetShowFlyWires(bool b);
 	void SetFill(bool b);
 	// Part controls
 	void SetCompName(const QString& str);
@@ -302,6 +307,7 @@ public slots:
 	void SetDiagonalsMin(bool b);
 	void SetDiagonalsMax(bool b);
 	// Rendering options
+	void SetBrightness(int i);
 	void SetTrackWidth(int i);
 	void SetHoleWidth(int i);
 	void SetPadWidth(int i);
@@ -319,7 +325,6 @@ public slots:
 	void SetShowCloseTracks(bool b);
 	void SetAntialiasOff(bool b);
 	void SetAntialiasOn(bool b);
-	void SetAntialiasHigh(bool b);
 	// Find parts by name/value
 	void Find(bool bUseName, bool bExact, const QString& str);
 	// Wire options
@@ -334,6 +339,9 @@ public slots:
 	void AlignC();
 	void AlignR();
 	void AlignJ();
+	void AlignTop();
+	void AlignMid();
+	void AlignBot();
 	void SetText(const QString&);
 	void ChooseTextColor();
 	// For component editor dialog
@@ -447,6 +455,7 @@ private:
 	// GUI update
 	void UpdateRecentFiles(const QString* pFileName, bool bAdd);
 	void UpdateWindowTitle();
+	void UpdateRulerInfo();
 	void UpdateControls();
 	void UpdateCompDialog();
 	void EnableCompDialogControls();
@@ -456,6 +465,8 @@ private:
 
 	// Helpers
 	void SetQuality(QPainter& p);
+	void ResetRuler();
+	bool CanModifyRuler() const;
 	bool CanZoomIn() const;
 	bool CanZoomOut() const;
 	bool GetIsModified() const;
@@ -470,6 +481,7 @@ private:
 	QColor	GetBackgroundColor() const;	// For screen only.  PDF is always white.
 
 	// Pens
+	QPen	m_rulerPen;
 	QPen	m_backgroundPen;
 	QPen	m_darkGreyPen;
 	QPen	m_blackPen;
@@ -484,31 +496,27 @@ private:
 	QBrush	m_darkBrush;
 	QBrush	m_varBrush;
 private:
-	Ui::MainWindow*			ui;
-	QAction*				m_recentFileAction[MAX_RECENT_FILES];
-	QAction*				m_separator;	// At the end of the recent files list
-
 	QNetworkAccessManager	m_networkMgr;	// For checking version against Sourceforge
-
-	MyScrollArea*			m_scrollArea;	// The mainwindow contains a scrollable area ...
-	QLabel*					m_label;		// ... for a QLabel widget that ...
-	QPixmap					m_mainPixmap;	// ... contains an image of the whole board
-
-	QLabel*					m_labelStatus;	// For permanent status bar text
-
-	QDockWidget*			m_dockControlDlg;
-	QDockWidget*			m_dockCompDlg;
-	ControlDialog*			m_controlDlg;
-	RenderingDialog*		m_renderingDlg;
-	WireDialog*				m_wireDlg;
-	HotkeysDialog*			m_hotkeysDlg;
-	InfoDialog*				m_infoDlg;
-	CompDialog*				m_compDlg;
-	TextDialog*				m_textDlg;
-	BomDialog*				m_bomDlg;
-	TemplatesDialog*		m_templatesDlg;
-	PinDialog*				m_pinDlg;
-	FindDialog*				m_findDlg;
+	QAction*				m_recentFileAction[MAX_RECENT_FILES];
+	QAction*				m_separator			= nullptr;	// At the end of the recent files list
+	Ui::MainWindow*			ui					= nullptr;
+	MyScrollArea*			m_scrollArea		= nullptr;	// The mainwindow contains a scrollable area ...
+	QLabel*					m_label				= nullptr;	// ... for a QLabel widget that ...
+	QPixmap					m_mainPixmap;					// ... contains a pixmap image of the whole board
+	QLabel*					m_labelStatus		= nullptr;	// For permanent status bar text
+	QDockWidget*			m_dockControlDlg	= nullptr;
+	QDockWidget*			m_dockCompDlg		= nullptr;
+	ControlDialog*			m_controlDlg		= nullptr;
+	RenderingDialog*		m_renderingDlg		= nullptr;
+	WireDialog*				m_wireDlg			= nullptr;
+	HotkeysDialog*			m_hotkeysDlg		= nullptr;
+	InfoDialog*				m_infoDlg			= nullptr;
+	CompDialog*				m_compDlg			= nullptr;
+	TextDialog*				m_textDlg			= nullptr;
+	BomDialog*				m_bomDlg			= nullptr;
+	TemplatesDialog*		m_templatesDlg		= nullptr;
+	PinDialog*				m_pinDlg			= nullptr;
+	FindDialog*				m_findDlg			= nullptr;
 
 	Board					m_board;			// *** The main object ***
 	HistoryManager			m_historyMgr;		// Class to manage undo/redo
@@ -517,43 +525,48 @@ private:
 	QString					m_fileName;			// The loaded/saved .vrt file
 	QString					m_pdfFileName;		// The saved PDF file
 	QString					m_gerberFileName;	// The saved Gerber file
+	std::string				m_mouseActionString;// For the undo/redo history
 	std::string				m_localDataPathStr;	// The path to the "history" and "templates" folders
 	std::string				m_tutorialsPathStr;	// The path to the "tutorials" folder and "veroroute.png"
 
 	// Cached pixmaps containing pre-colored pads and blobs.
-	QPixmap**	m_ppPixmapPad	= nullptr;	// A pad in the host element
-	QPixmap**	m_ppPixmapVia	= nullptr;	// A via in the host element
-	QPixmap**	m_ppPixmapDiag	= nullptr;	// For filling small diagonal gaps not covered by blob pixmaps
-	QPixmap**	m_ppPixmapBlob	= nullptr;	// A composite shape with all the host element connections
-	QPixmap*	m_pPixmapDiagLT	= nullptr;	// Used instead of m_ppPixmapDiag for custom colors
-	QPixmap*	m_pPixmapDiagRT	= nullptr;	// Used instead of m_ppPixmapDiag for custom colors
-	int			m_radPixmapPad	= 0;		// Half pixmap width ...
-	int			m_radPixmapVia	= 0;		// ...
-	int			m_radPixmapDiag	= 0;		// ...
-	int			m_radPixmapBlob	= 0;		// ...
+	QPixmap**	m_ppPixmapPad		= nullptr;	// A pad in the host element
+	QPixmap**	m_ppPixmapVia		= nullptr;	// A via in the host element
+	QPixmap**	m_ppPixmapDiag		= nullptr;	// For filling small diagonal gaps not covered by blob pixmaps
+	QPixmap**	m_ppPixmapBlob		= nullptr;	// A composite shape with all the host element connections
+	QPixmap*	m_pPixmapDiagLT		= nullptr;	// Used instead of m_ppPixmapDiag for custom colors
+	QPixmap*	m_pPixmapDiagRT		= nullptr;	// Used instead of m_ppPixmapDiag for custom colors
+	int			m_radPixmapPad		= 0;		// Half pixmap width ...
+	int			m_radPixmapVia		= 0;		// ...
+	int			m_radPixmapDiag		= 0;		// ...
+	int			m_radPixmapBlob		= 0;		// ...
 	QPoint		m_mousePos;
-	bool		m_bRepaint		= false;	// Flag to make paintEvent() do something useful
-	bool		m_bMouseClick;				// Flag of click beginning
-	bool		m_bLeftClick;
-	bool		m_bRightClick;
-	bool		m_bCtrlKeyDown;
-	bool		m_bShiftKeyDown;
-	bool		m_bPaintPins;			// true ==> allow paint the component pins (and the board)
-	bool		m_bPaintBoard;			// true ==> allow paint the board only (not the component pins)
-	bool		m_bPaintFlood;			// true ==> allow flood-fill all connected tracks & pins
-	bool		m_bPaintLyrPref;		// true ==> allow paint layer preference to pins
-	bool		m_bDefiningRect;		// true ==> user is defining a rectangle
-	bool		m_bResizingText;		// true ==> user is resizing a text rectangle
-	bool		m_bWritePDF;			// true ==> draw to PDF file instead of screen
-	bool		m_bWriteGerber;			// true ==> draw to Gerber file instead of screen
-	bool		m_bTwoLayerGerber;		// true ==> 2-layer Gerber output instead of 1-layer
-	bool		m_bHistoryDir;			// true ==> have "history" folder
-	bool		m_bTemplatesDir;		// true ==> have "templates" folder
-	int			m_XGRIDOFFSET;			// So we can centre when writing to PDF
-	int			m_YGRIDOFFSET;			// So we can centre when writing to PDF
-	int			m_XCORRECTION;			// So we can fully render pads larger than 100 mil diameter
-	int			m_YCORRECTION;			// So we can fully render pads larger than 100 mil diameter
-	int			m_gridRow;				// Board row corresponding to mouse position
-	int			m_gridCol;				// Board col correspondong to mouse position
-	int			m_iTutorialNumber = -1;	// Tutorial file number 0,1,2,... (or -1 if not in tutorial mode)
+	bool		m_bRepaint			= false;	// Flag to make paintEvent() do something useful
+	bool		m_bMouseClick		= false;	// Flag of click beginning
+	bool		m_bLeftClick		= false;
+	bool		m_bRightClick		= false;
+	bool		m_bCtrlKeyDown		= false;
+	bool		m_bShiftKeyDown		= false;
+	bool		m_bPaintPins		= false;	// true ==> allow paint the component pins (and the board)
+	bool		m_bPaintBoard		= false;	// true ==> allow paint the board only (not the component pins)
+	bool		m_bPaintFlood		= false;	// true ==> allow flood-fill all connected tracks & pins
+	bool		m_bPaintLyrPref		= false;	// true ==> allow paint layer preference to pins
+	bool		m_bDefiningRect		= false;	// true ==> user is defining a rectangle
+	bool		m_bResizingText		= false;	// true ==> user is resizing a text rectangle
+	bool		m_bWritePDF			= false;	// true ==> draw to PDF file instead of screen
+	bool		m_bWriteGerber		= false;	// true ==> draw to Gerber file instead of screen
+	bool		m_bTwoLayerGerber	= false;	// true ==> 2-layer Gerber output instead of 1-layer
+	bool		m_bHistoryDir		= false;	// true ==> have "history" folder
+	bool		m_bTemplatesDir		= false;	// true ==> have "templates" folder
+	bool		m_bRuler			= false;
+	bool		m_bModifyRulerA		= false;
+	int			m_XGRIDOFFSET		= 0;		// So we can centre when writing to PDF
+	int			m_YGRIDOFFSET		= 0;		// So we can centre when writing to PDF
+	int			m_XCORRECTION		= 0;		// So we can fully render pads larger than 100 mil diameter
+	int			m_YCORRECTION		= 0;		// So we can fully render pads larger than 100 mil diameter
+	int			m_gridRow			= 0;		// Board row corresponding to mouse position
+	int			m_gridCol			= 0;		// Board col correspondong to mouse position
+	QPoint		m_rulerA;
+	QPoint		m_rulerB;
+	int			m_iTutorialNumber	= -1;		// Tutorial file number 0,1,2,... (or -1 if not in tutorial mode)
 };
