@@ -20,6 +20,7 @@
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
 #include "GPainter.h"
+#include "SpanningTreeHelper.h"
 
 void MainWindow::DestroyPixmapCache()
 {
@@ -48,7 +49,7 @@ void MainWindow::CreatePixmapCache(const GuiControl& guiCtrl, ColorManager& colo
 
 	m_radPixmapPad	= guiCtrl.GetHalfPixelsFromMIL( guiCtrl.GetPAD_MIL() );		// Half pad width in pixels
 	m_radPixmapVia	= guiCtrl.GetHalfPixelsFromMIL( guiCtrl.GetVIAPAD_MIL() );	// Half via width in pixels
-	m_radPixmapDiag	= (int) ceil(1.414 * guiCtrl.GetHalfPixelsFromMIL( guiCtrl.GetTRACK_MIL() ));
+	m_radPixmapDiag	= static_cast<int>( ceil(1.414 * guiCtrl.GetHalfPixelsFromMIL( guiCtrl.GetTRACK_MIL() )) );
 	m_radPixmapBlob	= guiCtrl.GetGRIDPIXELS() >> 1;	// Half grid-square width in pixels
 
 	m_ppPixmapPad	= new QPixmap*[NUM_PIXMAP_COLORS];
@@ -192,7 +193,7 @@ void MainWindow::PaintTag(const GuiControl& guiCtrl, QPainter& painter, const QC
 	// Paints a short tag connecting a pad to the ground fill
 	const int w = ( iPadWidthMIL == 0 ) ? guiCtrl.GetPAD_MIL() : iPadWidthMIL;
 	const int X = guiCtrl.GetHalfPixelsFromMIL( w ) + guiCtrl.GetPixelsFromMIL( guiCtrl.GetGAP_MIL() );
-	const int D = (int) ( X * sqrt(0.5) );
+	const int D = static_cast<int>( X * sqrt(0.5) );
 	QPointF pD;	// The other end of the tag
 	switch( iNbr)
 	{
@@ -259,7 +260,7 @@ void MainWindow::PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const Q
 		{
 			GStream& os = m_gWriter.GetStream(k == 0 ? GFILE::GBL : GFILE::GTL);	// Bottom/Top copper layer
 
-			for (auto & polygon : polygonList)
+			for (auto& polygon : polygonList)
 			{
 				const bool bPad	= polygon.m_ePadPen != GPEN::NONE;
 				const bool bTrk	= polygon.m_eTrkPen != GPEN::NONE;
@@ -287,7 +288,7 @@ void MainWindow::PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const Q
 		brush.setColor(color);
 		painter.setBrush(brush);
 
-		for (auto & polygon : polygonList)
+		for (auto& polygon : polygonList)
 		{
 			const bool bTrk	= polygon.m_eTrkPen != GPEN::NONE;
 			const bool bPad	= polygon.m_ePadPen != GPEN::NONE;
@@ -300,7 +301,8 @@ void MainWindow::PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const Q
 				painter.drawPolygon(polygon);
 			else if ( bPad && bTrk )	// Fat tracks with diagonals
 			{
-				int i(0);
+				//polygon.Process();	// Not needed since m_bFatEdge already populated
+				size_t i(0);
 				auto iterA = polygon.begin();
 				auto iterB = iterA; iterB++;
 				while( iterB != polygon.end() )
@@ -353,17 +355,15 @@ void MainWindow::PaintCompDefiner()	// The paint method in "component editor mod
 	const double	 dTextScale	= W / 24.0;					// For scaling text when zooming
 
 	// Shift comp to near grid centre
-	const int ROWS( def.GetScreenRows() );
-	const int COLS( def.GetScreenCols() );
+	const int ROWS(def.GetScreenRows()), COLS(def.GetScreenCols());
 
 	const Rect rect(def.GetGridRowMin(), def.GetGridRowMax(), def.GetGridColMin(), def.GetGridColMax());
 
 	QPainter painter;
-	const int reqWidth  = W * COLS;
-	const int reqHeight = W * ROWS;
-	if ( m_mainPixmap.width() != reqWidth || m_mainPixmap.height() != reqHeight )
+	const int reqW(W * COLS), reqH(W * ROWS);
+	if ( m_mainPixmap.width() != reqW || m_mainPixmap.height() != reqH )
 	{
-		m_mainPixmap = QPixmap(reqWidth, reqHeight);
+		m_mainPixmap = QPixmap(reqW, reqH);
 		m_mainPixmap.setDevicePixelRatio(1.0);
 	}
 	painter.begin(&m_mainPixmap);	// Paint to main pixmap
@@ -371,7 +371,7 @@ void MainWindow::PaintCompDefiner()	// The paint method in "component editor mod
 	SetQuality(painter);
 
 	m_XCORRECTION = m_YCORRECTION = 0;
-	painter.fillRect(m_XGRIDOFFSET, m_YGRIDOFFSET, reqWidth, reqHeight, Qt::white);
+	painter.fillRect(m_XGRIDOFFSET, m_YGRIDOFFSET, reqW, reqH, Qt::white);
 
 	m_blackPen.setWidth(0);
 	m_whitePen.setWidth(0);
@@ -408,21 +408,23 @@ void MainWindow::PaintCompDefiner()	// The paint method in "component editor mod
 		painter.setBrush( s.GetDrawFill() ? m_varBrush : Qt::NoBrush );
 
 		painter.save();
-		auto DX = s.GetDX() * W;
-		auto DY = s.GetDY() * W;
-		auto X  = DX * 0.5;
-		auto Y  = DY * 0.5;
 		painter.translate( s.GetCX() * W, s.GetCY() * W );
 		painter.rotate( -s.GetA3() );	// A3 > 0 ==> CCW
+		const int DX	= static_cast<int>(s.GetDX() * W);
+		const int DY	= static_cast<int>(s.GetDY() * W);
+		const int X		= static_cast<int>(s.GetDX() * W * 0.5);
+		const int Y		= static_cast<int>(s.GetDY() * W * 0.5);
+		const int A		= static_cast<int>(s.GetA1() * 16);
+		const int Alen	= static_cast<int>(s.GetAlen() * 16);
 		switch( s.GetType() )
 		{
-			case SHAPE::LINE:			painter.drawLine(-X, -Y, X, Y);		break;
 			case SHAPE::RECT:			painter.drawRect(-X, -Y, DX, DY);	break;
 			case SHAPE::ROUNDED_RECT:	painter.drawRoundedRect(-X, -Y, DX, DY, 0.35 * W, 0.35 * W);	break;
 			case SHAPE::ELLIPSE:		painter.drawEllipse(-X, -Y, DX, DY);	break;
-			case SHAPE::ARC:			painter.drawArc(  -X, -Y, DX, DY, s.GetA1() * 16, s.GetAlen() * 16);	break;
-			case SHAPE::CHORD:			painter.drawChord(-X, -Y, DX, DY, s.GetA1() * 16, s.GetAlen() * 16);	break;
-			default: assert(0);	// Unhandled shape
+			case SHAPE::ARC:			painter.drawArc(  -X, -Y, DX, DY, A, Alen);	break;
+			case SHAPE::CHORD:			painter.drawChord(-X, -Y, DX, DY, A, Alen);	break;
+			default:					assert(SHAPE::LINE == s.GetType() );
+										painter.drawLine(-X, -Y, X, Y);		break;
 		}
 		if ( bCurrentShape )
 		{
@@ -444,21 +446,20 @@ void MainWindow::PaintCompDefiner()	// The paint method in "component editor mod
 
 	// Draw grid points ==========================================================================
 	if ( board.GetShowGrid() )
-	{
-		for (int j = 0; j < ROWS; j++)	for (int i = 0; i < COLS; i++)
+		for (int j = 0; j < ROWS; j++)
+		for (int i = 0; i < COLS; i++)
 		{
 			GetXY(board, j, i, X, Y);
 			painter.drawPoint(X, Y);
 		}
-	}
 
 	// Draw dashed rect around footprint boundary and along central axes =========================
 	painter.setPen(m_dashPen);
 	painter.setBrush(Qt::NoBrush);
 	painter.drawRect(L, T, R-L, B-T);
 	painter.setPen(m_dotPen);
-	painter.drawLine(0, AXIS_Y, reqWidth, AXIS_Y);
-	painter.drawLine(AXIS_X, 0, AXIS_X, reqHeight);
+	painter.drawLine(0, AXIS_Y, reqW, AXIS_Y);
+	painter.drawLine(AXIS_X, 0, AXIS_X, reqH);
 
 	// Draw pins =================================================================================
 	QFont pinsFont = painter.font();	// Copy of current font
@@ -492,7 +493,7 @@ void MainWindow::PaintCompDefiner()	// The paint method in "component editor mod
 			painter.setPen(m_blackPen);
 			painter.rotate(270);
 			painter.scale(dTextScale, dTextScale);
-			painter.drawText(0,0,0,0, Qt::TextDontClip | Qt::AlignCenter, GetDefaultPinLabel(p->GetPinIndex()).c_str());
+			painter.drawText(0,0,0,0, Qt::TextDontClip | Qt::AlignCenter, CompTypes::GetDefaultPinLabel(p->GetPinIndex()).c_str());
 		}
 		painter.restore();
 	}
@@ -509,15 +510,14 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 	const TRACKMODE& trackMode		= board.GetTrackMode();
 	const COMPSMODE& compMode		= board.GetCompMode();
 	const bool&		 bVero			= board.GetVeroTracks();
-	const bool		 bDiagsOK		= ( board.GetDiagsMode() != DIAGSMODE::OFF );
 	const bool		 bColor			= trackMode == TRACKMODE::COLOR;
 	const bool		 bMono			= trackMode == TRACKMODE::MONO;
 	const bool		 bPCB			= trackMode == TRACKMODE::PCB;
 	const bool		 bMonoPCB		= bMono || bPCB;
 	const bool		 bGroundFill	= !bVero && bMonoPCB && board.GetGroundFill();
-	const bool		 bPixmapCache	= !bVero && !bPCB && !bGroundFill && !m_bWritePDF;	// true ==> Faster rendering (Mono/Color modes)
-	const bool		 bDirect		= !bVero && !bPixmapCache && !bGroundFill;			// true ==> Draw track "blobs" and pads directly (PDF/Gerber)
-	const bool		 bExtraTags		= false;											// true ==> Add extra thermal relief tags
+	const bool		 bPixmapCache	= !bVero && ( bMono || bColor ) && !bGroundFill && !m_bWritePDF;	// true ==> Faster rendering (Mono/Color modes)
+	const bool		 bDirect		= !bVero && !bPixmapCache && !bGroundFill;	// true ==> Draw track "blobs" and pads directly (PDF/Gerber)
+	const bool		 bExtraTags		= false;									// true ==> Add extra thermal relief tags
 	const int&		 layer			= board.GetCurrentLayer();
 	const int&		 groundNodeId	= ( layer == 0 ) ? board.GetGroundNodeId0() :  board.GetGroundNodeId1();
 	const bool		 bWiresAsTracks	= m_bWriteGerber && m_bTwoLayerGerber && board.GetLyrs() == 1;	// true ==> Convert wires to tracks on the top layer
@@ -558,8 +558,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 	int gndL, gndR, gndT, gndB;
 	board.CalcGroundFillBounds();
 	board.GetGroundFillBounds(gndL, gndR, gndT, gndB);
-	const int reqWidth  = (gndR - gndL);
-	const int reqHeight = (gndB - gndT);
+	const int reqW(gndR - gndL), reqH(gndB - gndT);
 
 	m_XCORRECTION = -gndL;
 	m_YCORRECTION = -gndT;
@@ -586,9 +585,9 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 	}
 	else
 	{
-		if ( m_mainPixmap.width() != reqWidth || m_mainPixmap.height() != reqHeight )
+		if ( m_mainPixmap.width() != reqW || m_mainPixmap.height() != reqH )
 		{
-			m_mainPixmap = QPixmap(reqWidth, reqHeight);
+			m_mainPixmap = QPixmap(reqW, reqH);
 			m_mainPixmap.setDevicePixelRatio(1.0);
 		}
 		painter.begin(&m_mainPixmap);	// Paint to main pixmap
@@ -599,12 +598,12 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 
 	if ( board.GetFlipH() )
 	{
-		painter.translate(2*m_XGRIDOFFSET + reqWidth, 0);
+		painter.translate(2*m_XGRIDOFFSET + reqW, 0);
 		painter.scale(-1, 1);	// Mirror L-R
 	}
 	if ( board.GetFlipV() )
 	{
-		painter.translate(0, 2*m_YGRIDOFFSET + reqHeight);
+		painter.translate(0, 2*m_YGRIDOFFSET + reqH);
 		painter.scale(1, -1);	// Mirror T-B
 	}
 
@@ -622,7 +621,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 	{
 		// Grow board outline to guarantee separation from tracks and ground
 		const double dEdge = board.GetEdgeWidth();
-		const double R(reqWidth), B(reqHeight);
+		const double R(reqW), B(reqH);
 		const int& X = m_XGRIDOFFSET; const int& Y = m_YGRIDOFFSET;
 		gndPoly << QPointF(X,     Y);		edge << QPointF(X	  - dEdge, Y	 - dEdge);
 		gndPoly << QPointF(X + R, Y);		edge << QPointF(X + R + dEdge, Y	 - dEdge);
@@ -636,7 +635,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 	}
 	else
 	{
-		painter.fillRect(m_XGRIDOFFSET, m_YGRIDOFFSET, reqWidth, reqHeight, bGroundFill ? groundFillColor : backgroundColor);
+		painter.fillRect(m_XGRIDOFFSET, m_YGRIDOFFSET, reqW, reqH, bGroundFill ? groundFillColor : backgroundColor);
 	}
 
 	// Draw rect around whole board area =========================================================
@@ -646,20 +645,20 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 	}
 	else
 	{
-		const int iPenWidth = ( bPCB ) ? W * 0.100 : 0;	// Like GPEN::GKO = 10 mil used for Gerber
+		const int iPenWidth = ( bPCB ) ? static_cast<int>(W * 0.100) : 0;	// Like GPEN::GKO = 10 mil used for Gerber
 		m_blackPen.setWidth(iPenWidth);
 		m_whitePen.setWidth(iPenWidth);
 		painter.setPen(GetBackgroundColor() == Qt::black ? m_whitePen : m_blackPen);
 		painter.setBrush(Qt::NoBrush);
 
-		const int dEdge = (int) board.GetEdgeWidth();
-		painter.drawRect(m_XGRIDOFFSET - dEdge, m_YGRIDOFFSET - dEdge, reqWidth + (dEdge<<1), reqHeight + (dEdge<<1));
+		const int dEdge = static_cast<int>( board.GetEdgeWidth() );
+		painter.drawRect(m_XGRIDOFFSET - dEdge, m_YGRIDOFFSET - dEdge, reqW + (dEdge<<1), reqH + (dEdge<<1));
 	}
 
 	// Draw grid points ==========================================================================
 	if ( !bPCB && board.GetShowGrid() )
-	{
-		for (int j = 0; j < board.GetRows(); j++)	for (int i = 0; i < board.GetCols(); i++)
+		for (int j = 0, jMax = board.GetRows(); j < jMax; j++)
+		for (int i = 0, iMax = board.GetCols(); i < iMax; i++)
 		{
 			const Element* pC = board.Get(layer, j, i);
 			if ( trackMode == TRACKMODE::OFF || ( !pC->GetHasPin() && pC->GetNodeId() == BAD_NODEID ) )
@@ -668,7 +667,6 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 				painter.drawPoint(X, Y);
 			}
 		}
-	}
 
 	// Draw tracks ===============================================================================
 	if ( bPCB || trackMode != TRACKMODE::OFF )	// Force tracks in PCB mode
@@ -704,7 +702,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 				const bool		bWireAsVia		= bWire && bWiresAsTracks;	// true ==> draw small via pad
 				const int		iPerimeterCode	= board.GetPerimeterCode(pC);	// 0 to 255
 				const bool		bVia			= pC->GetIsVia()  ||  bWireAsVia;
-				const bool		bPad			= pC->GetHasPin() && !bWireAsVia;
+				const bool		bPad			= !bWireAsVia && pC->GetHasPin();
 				assert( !(bVia && bPad) );	// Can't be both a via and a regular pad
 
 				bool	bCustomSize(false);
@@ -721,7 +719,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 
 					if ( !bVero )
 					{
-						comp.GetCompPinOffsets(pinIndex, padOffsetX, padOffsetY);
+						comp.GetCompPinOffsets(pinIndex, padOffsetX, padOffsetY);	// Get offsets in mil
 						padOffsetX = (padOffsetX * W) / 100;	 // Convert from mil to pixels
 						padOffsetY = (padOffsetY * W) / 100;	 // Convert from mil to pixels
 					}
@@ -891,12 +889,13 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 					}
 					else if ( iLoop == 1 )	// Draw track "blobs" and pads directly
 					{
-						if ( bBlob ) PaintBlob(board, painter, color, pCentre, pCentreOff, iPerimeterCode, bPad);	// Draw track blob
-						if ( bVia )  PaintVia(board, painter, color, pCentre);										// Draw via same color as track
-						if ( bPad )  PaintPad(board, painter, color, pCentreOff, iPadWidthMIL, iHoleWidthMIL);		// Draw pad same color as track
-						if ( bExtraTags && bPad && nodeId == groundNodeId && nodeId != BAD_NODEID )		// Draw extra thermal relief tags
+						if ( bBlob ) PaintBlob(board, painter, color, pCentre, pCentreOff, iPerimeterCode, bPad);			// Draw track blob
+						if ( bVia )  PaintVia(board, painter, color, pCentre);												// Draw via same color as track
+						if ( bPad )  PaintPad(board, painter, color, pCentreOff, iPadWidthMIL, iHoleWidthMIL);				// Draw pad same color as track
+						if ( bExtraTags && bPad && nodeId == groundNodeId && iPerimeterCode > 0 && nodeId != BAD_NODEID )	// Draw extra thermal relief tags
 						{
 							int iCode(iPerimeterCode);		// Take a copy of the perimeter code
+							const bool bDiagsOK( board.GetDiagsMode() != DIAGSMODE::OFF );	// If bExtraTags gets set true then move this to start of method
 							for (int iDiag = 0, iDiagMax = ( bDiagsOK ) ? 2 : 1; iDiag < iDiagMax; iDiag++)	// First pass ==> Non-diagonal nbrs.  Second pass diagonal nbrs
 							for (int iNbr = iDiag; iNbr < 8; iNbr += 2)	// Even/Odd iNbr ==> Non-diagonal/Diagonal
 							{
@@ -1081,12 +1080,10 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 			if ( bPCB && (bMark || bVeroLabel) )	continue;	// Don't show markers and vero-lables in PCB mode
 			if ( m_bWriteGerber && !bPlaced ) continue;	// Don't write floating components to Gerber
 			if ( bWiresAsTracks && bWire && compMgr.GetWireCanBeTrack(&comp) ) continue;
-			const bool 		 bFound			= compMgr.GetFound( comp.GetId() );
+			const bool		 bFound			= compMgr.GetFound( comp.GetId() );
 			const bool		 bPinLabels		= !bMonoPCB && (comp.GetPinFlags() & PIN_LABELS) > 0 && board.GetShowPinLabels();
 			const bool		 bRectPins		= !bMonoPCB && (comp.GetPinFlags() & PIN_RECT) > 0;
 			const bool		 bHighlightComp	= board.GetGroupMgr().GetIsUserComp( comp.GetId() );
-			const int		 jComp			= comp.GetRow();
-			const int		 iComp			= comp.GetCol();
 			const bool		 bCustomSize	= comp.GetCustomPads();
 			const int		 iPadWidthMIL	= bCustomSize ? comp.GetPadWidth()  : board.GetPAD_MIL();
 			const int		 iHoleWidthMIL	= bCustomSize ? comp.GetHoleWidth() : board.GetHOLE_MIL();
@@ -1119,20 +1116,17 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 				if ( bMark )	// Markers are a special case since they don't actually have a pin !!!
 				{
 					// Only draw markers as holes in ground fill MONO mode and only in places with no track
-					if ( bMono && bGroundFill && board.Get(layer, jComp, iComp)->GetNodeId() == BAD_NODEID )
+					if ( bMono && bGroundFill && board.Get(layer, comp.GetRow(), comp.GetCol())->GetNodeId() == BAD_NODEID )
 					{
-						GetLRTB(board, 100, jComp, iComp, L, R, T, B);	// 100% size square
-						painter.drawPoint((L+R)/2, (T+B)/2);			// A pin is drawn with a circle
+						GetLRTB(board, 100, comp.GetRow(), comp.GetCol(), L, R, T, B);	// 100% size square
+						painter.drawPoint((L+R)/2, (T+B)/2);							// A pin is drawn with a circle
 					}
 				}
 				else			// Regular components/pads/wires ...
 				{
-					for (int jj = 0; jj < comp.GetCompRows(); jj++)
-					for (int ii = 0; ii < comp.GetCompCols(); ii++)
+					for (int jj = 0, jjMax = comp.GetCompRows(), j = comp.GetRow(); jj < jjMax; jj++, j++)
+					for (int ii = 0, iiMax = comp.GetCompCols(), i = comp.GetCol(); ii < iiMax; ii++, i++)
 					{
-						const int j = jComp + jj;
-						const int i = iComp + ii;
-
 						const size_t iPinIndex = comp.GetCompElement(jj,ii)->GetPinIndex();
 						if ( iPinIndex == BAD_PININDEX ) continue;
 
@@ -1160,8 +1154,8 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 						L += padOffsetX;	R += padOffsetX;	T += padOffsetY;	B += padOffsetY;
 
 						// Stop pins vanishing if zoomed too far out
-						if ( L == R ) { L--, R++; }
-						if ( T == B ) { T--, B++; }
+						if ( L == R ) { L--; R++; }
+						if ( T == B ) { T--; B++; }
 
 						if ( bPinLabels )	// Write pin labels
 						{
@@ -1262,21 +1256,23 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 				comp.GetSafeBounds(SL,SR,ST,SB);
 				SL = SL * W - C;	SR = SR * W + C;
 				ST = ST * W - C;	SB = SB * W + C;
-				const double dReqW(SR - SL), dReqH(SB - ST);
+				const int iReqW	= static_cast<int>(SR - SL);
+				const int iReqH	= static_cast<int>(SB - ST);
+				const int iSL	= static_cast<int>(SL);
+				const int iST	= static_cast<int>(ST);
 				GPainter painterTmp;
-				QPixmap tmpPixmap(dReqW, dReqH);
+				QPixmap tmpPixmap(iReqW, iReqH);
 				tmpPixmap.setDevicePixelRatio(1.0);
 				const QColor maskColor = comp.GetNewColor().GetQColor();// We'll mask out pixels with this color at the end
 
 				if ( bFill )
 				{
 					painterTmp.begin(&tmpPixmap);
-					painterTmp.fillRect(0,0,dReqW, dReqH, maskColor);	// This will be turned transparent later
+					painterTmp.fillRect(0, 0, iReqW, iReqH, maskColor);	// This will be turned transparent later
 					painterTmp.setPen(Qt::NoPen);
 				}
 
-				const int iLoopStart = ( bFill ) ? 0 : 1;
-				for (int iLoop = iLoopStart; iLoop < 2; iLoop++)
+				for (int iLoop = ( bFill ) ? 0 : 1; iLoop < 2; iLoop++)
 				{
 					GPainter* pPainter = ( iLoop == 0 ) ? &painterTmp : &painter;
 
@@ -1289,7 +1285,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 					if ( bFill && iLoop == 1 )	// Draw the pixmap created on the previous pass
 					{
 						painter.setOpacity( /*bWire ? 1.0 :*/ board.GetFillSaturation() * 0.01);
-						painter.drawPixmap(SL, ST, tmpPixmap);
+						painter.drawPixmap(iSL, iST, tmpPixmap);
 						painter.setOpacity(1.0);
 					}
 
@@ -1301,8 +1297,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 						case 'S':	pPainter->rotate(270);	break;
 					}
 
-					const size_t numShapes = comp.GetNumShapes();
-					for (size_t i = 0; i < numShapes; i++)
+					for (size_t i = 0, numShapes = comp.GetNumShapes(); i < numShapes; i++)
 					{
 						const Shape& s = comp.GetShape(i);
 
@@ -1331,20 +1326,21 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 						pPainter->translate( s.GetCX() * W, s.GetCY() * W );	// Translate to shape centre
 						pPainter->rotate( -s.GetA3() );	// A3 > 0 ==> CCW		// Rotate about shape centre
 
-						auto DX = s.GetDX() * W;
-						auto DY = s.GetDY() * W;
-						auto X  = DX * 0.5;
-						auto Y  = DY * 0.5;
-
+						const int DX	= static_cast<int>(s.GetDX() * W);
+						const int DY	= static_cast<int>(s.GetDY() * W);
+						const int X		= static_cast<int>(s.GetDX() * W * 0.5);
+						const int Y		= static_cast<int>(s.GetDY() * W * 0.5);
+						const int A		= static_cast<int>(s.GetA1() * 16);
+						const int Alen	= static_cast<int>(s.GetAlen() * 16);
 						switch( s.GetType() )
 						{
-							case SHAPE::LINE:			pPainter->drawLine(-X, -Y, X, Y);	break;
 							case SHAPE::RECT:			pPainter->drawRect(-X, -Y, DX, DY);	break;
 							case SHAPE::ROUNDED_RECT:	pPainter->drawRoundedRect(-X, -Y, DX, DY, 0.35 * W, 0.35 * W);	break;
 							case SHAPE::ELLIPSE:		pPainter->drawEllipse(-X, -Y, DX, DY);	break;
-							case SHAPE::ARC:			pPainter->drawArc(  -X, -Y, DX, DY, s.GetA1() * 16, s.GetAlen() * 16);	break;
-							case SHAPE::CHORD:			pPainter->drawChord(-X, -Y, DX, DY, s.GetA1() * 16, s.GetAlen() * 16);	break;
-							default: assert(0);	// Unhandled shape
+							case SHAPE::ARC:			pPainter->drawArc(  -X, -Y, DX, DY, A, Alen);	break;
+							case SHAPE::CHORD:			pPainter->drawChord(-X, -Y, DX, DY, A, Alen);	break;
+							default:					assert( SHAPE::LINE == s.GetType() );
+														pPainter->drawLine(-X, -Y, X, Y);	break;
 						}
 						pPainter->restore();
 					}
@@ -1358,53 +1354,146 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 					}
 				}
 			}
-			// End draw component fill + outline -------------------------------------------------
+		}
+	}
 
-			// Begin draw component text ---------------------------------------------------------
-			if ( compMode != COMPSMODE::OFF && !bMark && !bWire && !bVeroLabel )
+	// Draw flying wires =========================================================================
+	if ( !m_bWriteGerber && !bMonoPCB && compMode != COMPSMODE::OFF && board.GetShowFlyWires() )
+	{
+		painter.save();
+
+		QPen  blackPen = m_blackPen;	// Used for lines in the component pixmap
+		blackPen.setWidth(2);
+		m_varPen.setWidth(static_cast<int>(0.175 * W));
+		m_varPen.setStyle(Qt::DotLine);
+		painter.setBrush(Qt::NoBrush);
+		const int		dH = static_cast<int>(0.1*W);	// Param for wire rounded rect
+		const double	dW(0.35*W);						// Param for wire rounded rect
+
+		int padOffsetX, padOffsetY;	// For handling offset pads
+
+		std::set<int> visitedNodeIds;
+
+		for (int j = minRow; j <= maxRow; j++)
+		for (int i = minCol; i <= maxCol; i++)
+		{
+			Element*	pC		= board.Get(0, j, i);	// Layer 0 only
+			if ( !board.GetAllowFlyWire(pC) ) continue;
+			const int&	nodeId	= pC->GetNodeId();
+			if ( visitedNodeIds.find(nodeId) != visitedNodeIds.end() ) continue;
+			visitedNodeIds.insert(nodeId);
+
+			const bool	 bCurrentNodeId	= nodeId == GetCurrentNodeId();
+			const QColor color	= bCurrentNodeId ? colorMgr.GetPixmapColor(MY_GREY)
+												 : colorMgr.GetColorFromNodeId(nodeId);
+			m_varPen.setColor(color);
+
+			std::list<QPointF> spanTreePoints;
+
+			GetXY(board, j, i, X, Y);
+
+			if ( !bVero )
 			{
+				const Component& comp = compMgr.GetComponentById( pC->GetCompId() );
+				comp.GetCompPinOffsets(pC->GetPinIndex(), padOffsetX, padOffsetY);	// Get offsets in mil
+				X += (padOffsetX * W) / 100;	 // Convert from mil to pixels
+				Y += (padOffsetY * W) / 100;	 // Convert from mil to pixels
+			}
+
+			spanTreePoints.push_back( QPointF(X, Y) );
+
+			for (int jj = j; jj <= maxRow; jj++)
+			for (int ii = (jj == j) ? (i+1) : minCol; ii <= maxCol; ii++)
+			{
+				Element*	pD	= board.Get(0, jj, ii);	// Layer 0 only
+				if ( pD->GetNodeId() != nodeId || !board.GetAllowFlyWire(pD) ) continue;
+
+				GetXY(board, jj, ii, X, Y);
+
+				if ( !bVero )
+				{
+					const Component& comp = compMgr.GetComponentById( pD->GetCompId() );
+					comp.GetCompPinOffsets(pD->GetPinIndex(), padOffsetX, padOffsetY);	// Get offsets in mil
+					X += (padOffsetX * W) / 100;	 // Convert from mil to pixels
+					Y += (padOffsetY * W) / 100;	 // Convert from mil to pixels
+				}
+
+				spanTreePoints.push_back( QPointF(X, Y) );
+			}
+
+			std::list< SpanningTreeHelper::LINE > spanTreeLines;
+			SpanningTreeHelper::Build(spanTreePoints, spanTreeLines, true);	// true ==> daisy chain
+			for (const auto& o : spanTreeLines)
+			{
+				const QPointF	vec	= ( o.second - o.first );
+				const QPointF	mid	= ( o.second + o.first ) * 0.5;
+				const int		dL	= static_cast<int>(PolygonHelper::Length(vec) * 0.5);
 				painter.save();
-
-				double dCopyTextScale = dTextScale;
-				if ( bPCB )
-				{
-					dCopyTextScale *= m_board.GetTextSizeComp() * (20.0 / 243 );	// Scale to make the Gerber font size similar to regular component font size
-				}
-				else
-				{
-					QFont compFont = painter.font();	// Copy of current font
-					compFont.setPointSize( m_board.GetTextSizeComp() );
-					painter.setFont(compFont);
-				}
-
-				// Use floating point pen width to better match Gerber output.
-				// Scale the pen width down to compensate for painter.scale() scaling things up in the loop below.
-				const double dPenWidth = ( bPCB ) ? board.GetSilkWidth() / dCopyTextScale : 0;
-				m_orangePen.setWidthF(dPenWidth);	// Use colored text for found components
-				m_redPen.setWidthF(dPenWidth);		// Use red text for floating components
-				penPlaced.setWidthF(dPenWidth);		// Use this for placed components
-
-				GetXY(board, comp, X, Y);	// Get footprint centre
-
-				int offsetRow(0), offsetCol(0);
-				comp.GetLabelOffsets(offsetRow, offsetCol);
-
-				X += W * 0.0625 * offsetCol; // Offset for text is 1/16 of a grid square
-				Y += W * 0.0625 * offsetRow; // Offset for text is 1/16 of a grid square
-
-				painter.translate(X, Y);
-				if ( compDirection == 'N' || compDirection == 'S' )
-					painter.rotate(270);
-
-				const std::string& myStr = ( compMode == COMPSMODE::NAME )  ? comp.GetNameStr() :
-										   ( compMode == COMPSMODE::VALUE ) ? comp.GetValueStr() : "";
-
-				painter.scale(dCopyTextScale, dCopyTextScale);
-				painter.setPen(bFound ? m_orangePen : bPlaced ? penPlaced : m_redPen);
-				painter.drawText(0,0,0,0, Qt::AlignCenter | Qt::TextDontClip, myStr.c_str(), bPCB);
+				painter.translate(mid.x(), mid.y());
+				painter.rotate(atan2(vec.y(), vec.x()) * 180.0 / M_PI);
+				painter.setPen(m_varPen);		painter.drawLine(-dL, 0, dL, 0);
+				painter.setPen(fillBlackPen);	painter.drawRoundedRect(-dL, -dH, dL+dL, dH+dH, dW, dW);
 				painter.restore();
 			}
-			// End draw component text -----------------------------------------------------------
+		}
+		painter.restore();
+	}
+
+	// Draw component text =======================================================================
+	if ( compMode != COMPSMODE::OFF )
+	{
+		for (const auto& pComp : sortedComps)	// Iterate sorted components
+		{
+			const Component& comp			= *pComp;
+			const COMP&		 compType		= comp.GetType();
+			const char&		 compDirection	= comp.GetDirection();
+			const bool		 bMark			= compType == COMP::MARK;
+			const bool		 bWire			= compType == COMP::WIRE;
+			const bool		 bVeroLabel		= compType == COMP::VERO_NUMBER || compType == COMP::VERO_LETTER;
+			if ( bMark || bWire || bVeroLabel ) continue;
+			const bool		 bPlaced		= comp.GetIsPlaced();
+			if ( m_bWriteGerber && !bPlaced ) continue;		// Don't write floating components to Gerber
+			const bool		 bFound			= compMgr.GetFound( comp.GetId() );
+
+			painter.save();
+			double dCopyTextScale = dTextScale;
+			if ( bPCB )
+			{
+				dCopyTextScale *= m_board.GetTextSizeComp() * (20.0 / 243 );	// Scale to make the Gerber font size similar to regular component font size
+			}
+			else
+			{
+				QFont compFont = painter.font();	// Copy of current font
+				compFont.setPointSize( m_board.GetTextSizeComp() );
+				painter.setFont(compFont);
+			}
+
+			// Use floating point pen width to better match Gerber output.
+			// Scale the pen width down to compensate for painter.scale() scaling things up in the loop below.
+			const double dPenWidth = ( bPCB ) ? board.GetSilkWidth() / dCopyTextScale : 0;
+			m_orangePen.setWidthF(dPenWidth);	// Use colored text for found components
+			m_redPen.setWidthF(dPenWidth);		// Use red text for floating components
+			penPlaced.setWidthF(dPenWidth);		// Use this for placed components
+
+			GetXY(board, comp, X, Y);	// Get footprint centre
+
+			int offsetRow(0), offsetCol(0);
+			comp.GetLabelOffsets(offsetRow, offsetCol);
+
+			X += W * 0.0625 * offsetCol; // Offset for text is 1/16 of a grid square
+			Y += W * 0.0625 * offsetRow; // Offset for text is 1/16 of a grid square
+
+			painter.translate(X, Y);
+			if ( compDirection == 'N' || compDirection == 'S' )
+				painter.rotate(270);
+
+			const std::string& myStr = ( compMode == COMPSMODE::NAME )  ? comp.GetNameStr() :
+									   ( compMode == COMPSMODE::VALUE ) ? comp.GetValueStr() : "";
+
+			painter.scale(dCopyTextScale, dCopyTextScale);
+			painter.setPen(bFound ? m_orangePen : bPlaced ? penPlaced : m_redPen);
+			painter.drawText(0,0,0,0, Qt::AlignCenter | Qt::TextDontClip, myStr.c_str(), bPCB);
+			painter.restore();
 		}
 	}
 
@@ -1453,16 +1542,11 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 			if ( !(bVeroNumber || bVeroLetter) ) continue;
 			const char&		 compDirection	= comp.GetDirection();
 			const bool		 bPlaced		= comp.GetIsPlaced();	// Only unplaced when a label is created by copying another ("V" key)
-			const int		 jComp			= comp.GetRow();
-			const int		 iComp			= comp.GetCol();
 
 			int index(0), length(comp.GetSize());
-			for (int jj = 0; jj < comp.GetCompRows(); jj++)
-			for (int ii = 0; ii < comp.GetCompCols(); ii++, index++)
+			for (int jj = 0, jjMax = comp.GetCompRows(), j = comp.GetRow(); jj < jjMax; jj++, j++)
+			for (int ii = 0, iiMax = comp.GetCompCols(), i = comp.GetCol(); ii < iiMax; ii++, i++, index++)
 			{
-				const int j = jComp + jj;
-				const int i = iComp + ii;
-
 				GetLRTB(board, 100, j, i, L, R, T, B);
 
 				painter.save();
@@ -1483,7 +1567,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 					label = std::to_string(iNumber);
 				else				// Letters go: A,B,...,Y,Z,AA,AB,...,AY,AZ,BA,BB, ... etc
 				{
-					if ( iNumber > 26 ) label += ( char('A') + (iNumber-1)/26 - 1);
+					if ( iNumber > 26 ) label += ( char('A') + static_cast<char>((iNumber-1)/26 - 1) );
 					label += ( char('A') + (iNumber-1)%26 );
 				}
 
@@ -1514,6 +1598,9 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 			GetXY(board, rect.m_rowMax, rect.m_colMax, R, B);
 			L -= C; T -= C; R += C; B += C;
 
+			const int textW = static_cast<int>((R-L)/dTextScale);
+			const int textH = static_cast<int>((B-T)/dTextScale);
+
 			font.setBold( rect.GetStyle() & TEXT_BOLD );
 			font.setItalic( rect.GetStyle() & TEXT_ITALIC );
 			font.setUnderline( rect.GetStyle() & TEXT_UNDERLINE );
@@ -1526,7 +1613,7 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 			painter.save();
 			painter.translate((bMono && layer == 0) ? R : L, T);							// Mirror all text boxes in Mono mode for bottom layer
 			painter.scale((bMono && layer == 0) ? -dTextScale : dTextScale, dTextScale);	// Mirror all text boxes in Mono mode for bottom layer
-			painter.drawText(0,0,(R-L)/dTextScale,(B-T)/dTextScale, Qt::TextWordWrap | rect.GetFlags(), QString::fromStdString(rect.GetStr()));
+			painter.drawText(0, 0, textW, textH, Qt::TextWordWrap | rect.GetFlagsH() | rect.GetFlagsV(), QString::fromStdString(rect.GetStr()));
 			painter.restore();
 
 			if ( mapObj.first == GetCurrentTextId() )
@@ -1547,9 +1634,9 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 		{
 			m_orangePen.setWidth(lyr == layer ? 3 : 1);	// Current layer shown fatter
 			painter.setPen(m_orangePen);
-			for (auto& o : m_board.GetWarnPoints(lyr))
+			for (const auto& o : m_board.GetWarnPoints(lyr))
 			{
-				GetXY(board, o.ry(), o.rx(), X, Y);
+				GetXY(board, o.y(), o.x(), X, Y);
 				if ( lyr == layer )
 					painter.drawRect(X-C/3, Y-C/3, W/3, W/3);
 				else
@@ -1559,6 +1646,24 @@ void MainWindow::PaintBoard()	// The paint method in "circuit layout mode"
 		painter.restore();
 	}
 
+	if ( !m_bWriteGerber && m_bRuler )
+	{
+		QPointF A,B;
+		GetRulerExact(board, m_rulerA, A);
+		GetRulerExact(board, m_rulerB, B);
+
+		int XA, YA, XB, YB;
+		GetXY(board, A.ry(), A.rx(), XA, YA);
+		GetXY(board, B.ry(), B.rx(), XB, YB);
+
+		painter.save();
+		m_rulerPen.setWidth(C);
+		painter.setPen(m_rulerPen);
+		painter.setBrush(Qt::NoBrush);
+		painter.drawLine(XA,YA,XB,YB);
+		painter.restore();
+	}
+
 	if ( m_bWriteGerber )
 		m_gWriter.GetStream(GFILE::GTO).DrawBuffers();	// Top silk layer
 
@@ -1584,8 +1689,8 @@ void MainWindow::GetXY(const GuiControl& guiCtrl, double row, double col, int& X
 	// Takes a point in the Board and returns coordinates in the drawn image.
 	const int& W = guiCtrl.GetGRIDPIXELS();	// Square width in pixels
 	const int  C = W >> 1;					// Half square width in pixels
-	X = m_XGRIDOFFSET + m_XCORRECTION + C + col * W;
-	Y = m_YGRIDOFFSET + m_YCORRECTION + C + row * W;
+	X = m_XGRIDOFFSET + m_XCORRECTION + C + static_cast<int>(col * W);
+	Y = m_YGRIDOFFSET + m_YCORRECTION + C + static_cast<int>(row * W);
 }
 
 void MainWindow::GetLRTB(const GuiControl& guiCtrl, double percent, double row, double col, int& L, int& R, int& T, int& B) const
@@ -1596,7 +1701,7 @@ void MainWindow::GetLRTB(const GuiControl& guiCtrl, double percent, double row,
 	const int& W = guiCtrl.GetGRIDPIXELS();	// Square width in pixels
 	int X(0), Y(0);
 	GetXY(guiCtrl, row, col, X, Y);
-	const int S = (int) round(W * 0.005 * percent);
+	const int S = static_cast<int>( round(W * 0.005 * percent) );
 	L = X - S;	T = Y - S;
 	R = X + S;	B = Y + S;
 }
@@ -1625,4 +1730,29 @@ void MainWindow::GetXY(const GuiControl& guiCtrl, const Component& comp, int& X,
 	GetLRTB(guiCtrl, comp, L, R, T, B);
 	X = ( L + R ) / 2;
 	Y = ( T + B ) / 2;
+
+	if ( !guiCtrl.GetVeroTracks() )	// Offset the footprint if all pin offsets are equal
+	{
+		const int& W = guiCtrl.GetGRIDPIXELS();	// Square width in pixels
+		int padOffsetX, padOffsetY;
+		comp.GetCompShapeOffsets(padOffsetX, padOffsetY);	// Get offsets in mil
+		X += (padOffsetX * W) / 100;	// Convert from mil to pixels
+		Y += (padOffsetY * W) / 100;	// Convert from mil to pixels
+	}
+}
+
+void MainWindow::GetRulerExact(Board& board, const QPoint& p, QPointF& pOut) const
+{
+	pOut =  p;
+	if ( !board.GetVeroTracks() )
+	{
+		const Element* pC = board.Get(0, p.y(), p.x());
+		if ( pC->GetHasPin() && !pC->GetHasWire() )
+		{
+			int padOffsetX(0), padOffsetY(0);
+			const Component& comp = board.GetCompMgr().GetComponentById( pC->GetCompId() );
+			comp.GetCompPinOffsets(pC->GetPinIndex(), padOffsetX, padOffsetY);	// Get offsets in mil
+			pOut += QPointF(0.01 * padOffsetX, 0.01 * padOffsetY);
+		}
+	}
 }
