@@ -19,6 +19,8 @@
 
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
+#include "PolygonHelper.h"
+#include <QtGlobal>
 
 static const bool ALLOW_SMART_PAN_WITHOUT_CTRLKEY = true;
 
@@ -26,9 +28,6 @@ static const bool ALLOW_SMART_PAN_WITHOUT_CTRLKEY = true;
 static std::chrono::steady_clock::time_point g_lastAutoPanTime;
 static bool g_bHaveAutoPanned = false;
 
-std::string mouseActionString("Action");	// For the undo/redo history
-
-
 void MainWindow::GetPixMapXY(const QPoint& currentPoint, int& pixmapX, int& pixmapY) const
 {
 	const int iToolbarHeight = ( ui->toolBar->isFloating() || ui->toolBar->isHidden() ) ? 0 : ui->toolBar->height();
@@ -66,10 +65,12 @@ void MainWindow::GetRowCol(const QPoint& currentPoint, const int rows, const int
 
 void MainWindow::wheelEvent(QWheelEvent* event)
 {
-	//TODO QWheelEvent::pos() has been deprecated in newer Qt versions.
-	//	To avoid a build warning, use event->position() instead as follows:
-	//	m_mousePos = QPoint((int)event->position().x(), (int)event->position().y());
-	m_mousePos = event->pos();
+#if QT_VERSION >= QT_VERSION_CHECK(5,14,0)
+	m_mousePos = QPoint(static_cast<int>(event->position().x()), static_cast<int>(event->position().y()));
+#else
+	m_mousePos = QPoint(static_cast<int>(event->posF().x()), static_cast<int>(event->posF().y()));
+#endif
+
 	if ( GetShiftKeyDown() ) return;	// Ignore wheel events while trying to group components
 	const bool bBack = ( event->angleDelta().y() < 0 );
 	if ( GetCtrlKeyDown() )
@@ -86,6 +87,12 @@ void MainWindow::wheelEvent(QWheelEvent* event)
 	event->accept();	// If we don't do this, we can get the same event passed multiple times if we're on MS Windows.
 }
 
+bool MainWindow::CanModifyRuler() const
+{
+	return m_bRuler && !( m_board.GetCompEdit() || GetCtrlKeyDown() || GetShiftKeyDown() || GetResizingText() ||
+						  GetDefiningRect() || GetPaintPins() || GetPaintBoard() || GetPaintFlood() || GetPaintLyrPref() );
+}
+
 void MainWindow::mousePressEvent(QMouseEvent* event)
 {
 	m_mousePos = event->pos();
@@ -108,6 +115,19 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 	else
 		GetRowCol(event->pos(), m_gridRow, m_gridCol, dRow, dCol);
 
+	if ( m_bLeftClick && CanModifyRuler() )
+	{
+		const QPoint current(m_gridCol, m_gridRow);
+		if      ( current == m_rulerA ) m_bModifyRulerA = false;	// Do nothing, but prefer end B next time
+		else if ( current == m_rulerB ) m_bModifyRulerA = true;		// Do nothing, but prefer end A next time
+		else
+		{
+			if ( m_bModifyRulerA )	m_rulerA = current;
+			else					m_rulerB = current;
+			m_bModifyRulerA = !m_bModifyRulerA;
+		}
+	}
+
 	if ( m_board.GetCompEdit() )
 	{
 		// Pin/Shape selection
@@ -140,7 +160,6 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 		}
 		else				// Right/middle click quits rectangle mode
 		{
-			centralWidget()->setCursor(Qt::OpenHandCursor);
 			SetDefiningRect( false );
 			UpdateControls();
 		}
@@ -165,7 +184,7 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 					m_board.GetTextMgr().DestroyRect( GetCurrentTextId() );
 
 				if ( textId != BAD_TEXTID )
-					mouseActionString = "Select text box";
+					m_mouseActionString = "Select text box";
 
 				SetCurrentTextId(textId);
 			}
@@ -190,7 +209,7 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 				groupMgr.UpdateUserGroup( GetCurrentCompId() );	// Add/remove current comp (and its siblings) to user group
 				UpdateControls();
 				if ( GetCurrentTextId() == BAD_TEXTID )
-					mouseActionString = "(Un)select part(s)";
+					m_mouseActionString = "(Un)select part(s)";
 			}
 			else if ( !groupMgr.GetIsUserComp( GetCurrentCompId() ) )
 			{
@@ -199,7 +218,7 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 				compMgr.ClearTrax();
 				UpdateControls();
 				if ( GetCurrentTextId() == BAD_TEXTID )
-					mouseActionString = "(Un)select part(s)";
+					m_mouseActionString = "(Un)select part(s)";
 			}
 		}
 	}
@@ -232,7 +251,7 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 			{
 				const bool bChanged = m_board.ToggleLyrPref(layer, m_gridRow, m_gridCol, false);	// false ==> toggle
 				if ( !bChanged ) return;
-				mouseActionString = "Toggle pin layer preference";
+				m_mouseActionString = "Toggle pin layer preference";
 			}
 			else 
 			{
@@ -251,13 +270,13 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 					SetCurrentNodeId(tmp);	// Restore current nodeId
 
 					m_board.FloodNodeId( GetCurrentNodeId() );
-					mouseActionString = "Paint (flood)";
+					m_mouseActionString = "Paint (flood)";
 				}
 				else
 				{
 					const bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, GetCurrentNodeId(), GetPaintPins());
 					if ( !bChanged ) return;
-					mouseActionString = "Paint";
+					m_mouseActionString = "Paint";
 				}
 			}
 		}
@@ -267,13 +286,13 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 			{
 				const bool bChanged = m_board.ToggleLyrPref(layer, m_gridRow, m_gridCol, true);	// true ==> reset
 				if ( !bChanged ) return;
-				mouseActionString = "Clear pin layer preference";
+				m_mouseActionString = "Clear pin layer preference";
 			}
 			else
 			{
 				const bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, BAD_NODEID, GetPaintPins());
 				if ( !bChanged ) return;
-				mouseActionString = "Erase";
+				m_mouseActionString = "Erase";
 			}
 		}
 	}
@@ -284,7 +303,7 @@ void MainWindow::mousePressEvent(QMouseEvent* event)
 		if ( m_bRightClick )
 			SetCurrentNodeId( BAD_NODEID );
 		if ( m_bLeftClick || m_bRightClick )
-			mouseActionString = ( GetCurrentNodeId() == BAD_NODEID) ? "Unselect NodeId" : "Select NodeId";
+			m_mouseActionString = ( GetCurrentNodeId() == BAD_NODEID) ? "Unselect NodeId" : "Select NodeId";
 	}
 
 	m_board.WipeAutoSetPoints();
@@ -336,7 +355,7 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 	const int&			layer		= m_board.GetCurrentLayer();
 
 	if ( GetPaintPins() || GetPaintFlood() ) return;	// Ignore mouse move while painting pins or flooding
-	if ( GetShiftKeyDown() ) return;										// Ignore mouse move while trying to group components
+	if ( GetShiftKeyDown() ) return;					// Ignore mouse move while trying to group components
 
 	bool bSmartPan = GetCtrlKeyDown();
 
@@ -368,6 +387,7 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 	int deltaRow = row - m_gridRow;
 	int deltaCol = col - m_gridCol;
 
+	int oldRow(m_gridRow), oldCol(m_gridCol);
 	m_gridRow = row;
 	m_gridCol = col;
 
@@ -393,7 +413,7 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 	{
 		const bool bChanged = m_board.ToggleLyrPref(layer, m_gridRow, m_gridCol, true);	// true ==> reset
 		if ( !bChanged ) return;
-		mouseActionString = "Clear pin layer preference";
+		m_mouseActionString = "Clear pin layer preference";
 	}
 	else if ( !bSmartPan && GetDefiningRect() )
 	{
@@ -407,13 +427,13 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 		{
 			const bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, GetCurrentNodeId(), GetPaintPins());	// Only allow paint board (not pins)
 			if ( !bChanged ) return;	// No change
-			mouseActionString = "Paint";
+			m_mouseActionString = "Paint";
 		}
 		if ( m_bRightClick )	// Erase
 		{
 			const bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, BAD_NODEID, GetPaintPins());		// Only allow paint board (not pins)
 			if ( !bChanged ) return;	// No change
-			mouseActionString = "Erase";
+			m_mouseActionString = "Erase";
 		}
 		m_board.WipeAutoSetPoints();
 		m_board.PlaceFloaters();	// See if we can now place floating components down
@@ -449,7 +469,7 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 				g_lastAutoPanTime = std::chrono::steady_clock::now();
 			}
 		}
-		mouseActionString = ( GetResizingText() ) ? "Resize text box" : "Move text box";
+		m_mouseActionString = ( GetResizingText() ) ? "Resize text box" : "Move text box";
 	}
 	else if ( !bSmartPan && GetCurrentCompId() != BAD_COMPID && compMode != COMPSMODE::OFF )	// Move user-group components
 	{
@@ -474,7 +494,7 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 			g_bHaveAutoPanned = true;
 			g_lastAutoPanTime = std::chrono::steady_clock::now();
 		}
-		mouseActionString = "Move part(s)";
+		m_mouseActionString = "Move part(s)";
 	}
 	else if ( bSmartPan || (ALLOW_SMART_PAN_WITHOUT_CTRLKEY && m_bRightClick) )
 	{
@@ -487,11 +507,24 @@ void MainWindow::mouseMoveEvent(QMouseEvent* event)
 		if ( deltaCol == 0 ) { if ( pixmapX < W ) deltaCol = -1; }
 		if ( deltaRow == 0 && deltaCol == 0 ) return;	// No change
 		m_board.SmartPan(deltaRow, deltaCol);	// Pan whole circuit w.r.t. grid area, growing/shrinking as needed
-		mouseActionString = "Smart move/grow/crop";
+		m_mouseActionString = "Smart move/grow/crop";
 	}
 	else
 		return;
 
+	if ( m_bLeftClick && CanModifyRuler() )
+	{
+		const QPoint old(oldCol, oldRow);
+		const QPoint current(m_gridCol, m_gridRow);
+		bool bModifyA = ( old == m_rulerA );
+		bool bModifyB = !bModifyA && ( old == m_rulerB );
+		if ( !(bModifyA || bModifyB) )
+			bModifyA = PolygonHelper::Length(current - m_rulerA) < PolygonHelper::Length(current - m_rulerB);	// Choose nearest
+		if ( bModifyA )	m_rulerA = current;
+		else			m_rulerB = current;
+		m_bModifyRulerA = !bModifyA;
+	}
+
 	if ( abs(deltaRow) <= 1 && abs(deltaCol) <= 1 )	// If not moved mouse too fast ...
 	{
 		if ( GetDefiningRect() )
@@ -514,7 +547,7 @@ void MainWindow::mouseReleaseEvent(QMouseEvent* event)
 
 	if ( m_board.GetCompEdit() )
 	{
-		UpdateHistory(mouseActionString);
+		UpdateHistory(m_mouseActionString);
 		return RepaintSkipRouting();
 	}
 	if ( GetResizingText() )
@@ -537,7 +570,7 @@ void MainWindow::mouseReleaseEvent(QMouseEvent* event)
 	else
 		centralWidget()->setCursor(Qt::OpenHandCursor);
 
-	UpdateHistory(mouseActionString);
+	UpdateHistory(m_mouseActionString);
 	UpdateControls();
 	RepaintWithListNodes();
 }
@@ -557,18 +590,18 @@ void MainWindow::keyPressEvent(QKeyEvent* event)
 			int dx(0), dy(0);
 			switch( event->key() )
 			{
-				case Qt::Key_Left:	 dx-=5; break;
-				case Qt::Key_Right:	 dx+=5; break;
-				case Qt::Key_Up:	 dy-=5; break;
-				case Qt::Key_Down:	 dy+=5; break;
+				case Qt::Key_Left:	 dx--; break;
+				case Qt::Key_Right:	 dx++; break;
+				case Qt::Key_Up:	 dy--; break;
+				case Qt::Key_Down:	 dy++; break;
 			}
 			comp.IncCompPinOffsets(pinIndex, dx, dy);
 
 			int X,Y;
-			comp.GetCompPinOffsets(pinIndex, X, Y);
+			comp.GetCompPinOffsets(pinIndex, X, Y);	// Get offsets in mil
 
 			char buffer[256] = {'\0'};
-			sprintf(buffer,"(X, Y) pad offset (mil) = (%d, %d)", X, Y);
+			sprintf(buffer,"(X, Y) pad offset = (%d, %d) mil,    (%.4f, %.4f) mm", X, Y, X * 0.0254, Y * 0.0254);
 			ui->statusBar->showMessage(QString(buffer), 1000);
 
 			return RepaintSkipRouting();
@@ -617,7 +650,7 @@ void MainWindow::keyPressEvent(QKeyEvent* event)
 	{
 		switch( event->key() )
 		{
-			case Qt::Key_R:	SetDefiningRect(true);	centralWidget()->setCursor(Qt::SizeFDiagCursor); bUpdateControls = true; break;
+			case Qt::Key_R:	SetDefiningRect(true);	bUpdateControls = true; break;
 		}
 	}
 
@@ -651,14 +684,14 @@ void MainWindow::keyPressEvent(QKeyEvent* event)
 		switch( event->key() )
 		{
 			case Qt::Key_P:		if ( trackMode == TRACKMODE::OFF || compMode == COMPSMODE::OFF || GetPaintBoard() || GetPaintFlood() || GetPaintLyrPref() ) return;
-								SetPaintPins(true);		centralWidget()->setCursor(Qt::CrossCursor); break;
+								SetPaintPins(true);		break;
 			case Qt::Key_Space:	if ( trackMode == TRACKMODE::OFF || GetPaintPins() || GetPaintFlood() || GetPaintLyrPref() ) return;
-								SetPaintBoard(true);	centralWidget()->setCursor(Qt::CrossCursor); break;
+								SetPaintBoard(true);	break;
 			case Qt::Key_F:		if ( trackMode == TRACKMODE::OFF || compMode == COMPSMODE::OFF || GetPaintBoard() || GetPaintPins() || GetPaintLyrPref() ) return;
 								if ( m_board.GetRoutingEnabled() ) return;
-								SetPaintFlood(true);	centralWidget()->setCursor(Qt::CrossCursor); break;
+								SetPaintFlood(true);	break;
 			case Qt::Key_T:		if ( trackMode != TRACKMODE::PCB || m_board.GetLyrs() == 1 || GetPaintBoard() || GetPaintPins() || GetPaintFlood() ) return;
-								SetPaintLyrPref(true);	centralWidget()->setCursor(Qt::CrossCursor); break;
+								SetPaintLyrPref(true);	break;
 			case Qt::Key_W:		WipeTracks();	break;
 		}
 	}
@@ -690,16 +723,14 @@ void MainWindow::keyReleaseEvent(QKeyEvent* event)
 			case Qt::Key_F:		SetPaintFlood(false);	break;
 			case Qt::Key_T:		SetPaintLyrPref(false);	break;
 			case Qt::Key_Space:	SetPaintBoard(false);	break;
+			default:
+				if ( GetCurrentTextId() != BAD_TEXTID && m_bMouseClick )
+					centralWidget()->setCursor(Qt::ClosedHandCursor);
+				else if ( GetCurrentCompId() != BAD_COMPID && m_bMouseClick )
+					centralWidget()->setCursor(Qt::ClosedHandCursor);
+				else
+					centralWidget()->setCursor(Qt::OpenHandCursor);
 		}
-		if ( GetCurrentTextId() != BAD_TEXTID && m_bMouseClick )
-			centralWidget()->setCursor(Qt::ClosedHandCursor);
-		else if ( GetCurrentCompId() != BAD_COMPID && m_bMouseClick )
-			centralWidget()->setCursor(Qt::ClosedHandCursor);
-		else if ( GetDefiningRect() )
-			centralWidget()->setCursor(Qt::SizeFDiagCursor);
-		else
-			centralWidget()->setCursor(Qt::OpenHandCursor);
-
 		UpdateControls();
 		RepaintWithListNodes();
 	}
@@ -760,3 +791,29 @@ void MainWindow::dropEvent(QDropEvent *e)
 		OpenVrt(fileName);
 	}
 }
+
+void MainWindow::SetPaintPins(bool b)
+{
+	m_bPaintPins	= b;
+	centralWidget()->setCursor(b ? Qt::CrossCursor : Qt::OpenHandCursor);
+}
+void MainWindow::SetPaintBoard(bool b)
+{
+	m_bPaintBoard	= b;
+	centralWidget()->setCursor(b ? Qt::CrossCursor : Qt::OpenHandCursor);
+}
+void MainWindow::SetPaintFlood(bool b)
+{
+	m_bPaintFlood	= b;
+	centralWidget()->setCursor(b ? Qt::CrossCursor : Qt::OpenHandCursor);
+}
+void MainWindow::SetPaintLyrPref(bool b)
+{
+	m_bPaintLyrPref	= b;
+	centralWidget()->setCursor(b ? Qt::CrossCursor : Qt::OpenHandCursor);
+}
+void MainWindow::SetDefiningRect(bool b)
+{
+	m_bDefiningRect	= b;
+	centralWidget()->setCursor(b ? Qt::SizeFDiagCursor : Qt::OpenHandCursor);
+}
